""" deals with bencoded data - http://en.wikipedia.org/wiki/Bencode """

import re, cStringIO


INT = re.compile(r'\-?(0|[1-9][0-9]*)e')
STRING_LENGTH = re.compile(r'(0|[1-9][0-9]*):')


class BencodeReader:
	""" decodes bencoded objects from a str """

	def __init__(self, input):
		if not isinstance(input, str):
			raise TypeError('input must be a str')
		self.input = input
		self.ptr = 0
	
	def peek(self):
		if self.ptr == len(self.input):
			raise ValueError('unexpected end of input')
		return self.input[self.ptr]
	
	def next(self):
		c = self.peek()
		self.ptr += 1
		return c

	def next_object(self):
		
		c = self.next()
		
		if c == 'i':
			mo = INT.match(self.input, self.ptr)
			if mo is None:
				raise ValueError('illformed int')
			g = mo.group()
			self.ptr += len(g)
			return int(g[:-1])
		
		if c in '0123456789':
			mo = STRING_LENGTH.match(self.input, self.ptr-1)
			if mo is None:
				raise ValueError('illformed string length')
			g = mo.group()
			sl = int(g[:-1])
			self.ptr += len(g)-1
			if len(self.input) < self.ptr+sl:
				raise ValueError('unexpected end of string')
			rstr = self.input[self.ptr:self.ptr+sl]
			self.ptr += sl
			return rstr
		
		if c == 'l':
			list = []
			while True:
				if self.peek() == 'e':
					self.next()
					return list
				list.append(self.next_object())
		
		if c == 'd':
			dict = {}
			while True:
				if self.peek() == 'e':
					self.next()
					return dict
				k = self.next_object()
				if not isinstance(k, str):
					raise ValueError('dictionary keys must be strings')
				dict[k] = self.next_object()
		
		raise ValueError('unexpected char: ' + c)
	

def decode(input):
	return BencodeReader(input).next_object()


class BencodeWriter:
	
	def __init__(self):
		self.out = cStringIO.StringIO()
	
	def next_object(self, obj):
		
		if isinstance(obj, int):
			self.out.write('i%de' % obj)
		
		elif isinstance(obj, basestring):
			if isinstance(obj, unicode):
				obj = obj.encode('utf-8', 'replace')
			self.out.write('%d:%s' % (len(obj), obj))
		
		elif isinstance(obj, (tuple, list)):
			self.out.write('l')
			for e in obj:
				self.next_object(e)
			self.out.write('e')
		
		elif isinstance(obj, dict):
			self.out.write('d')
			new_obj = {}
			for k, v in obj.items():
				if not isinstance(k, basestring):
					raise TypeError('dictionary keys must be strings')
				if isinstance(k, unicode):
					k = k.encode('utf-8', 'replace')
				new_obj[k] = v
			keys = new_obj.keys()
			keys.sort()
			for k in keys:
				self.next_object(k)
				self.next_object(new_obj[k])
			self.out.write('e')
		
		else:
			raise TypeError('can only encode ints, strings, tuples, lists and dicts')

	def value(self):
		return self.out.getvalue()


def encode(input):
	bw = BencodeWriter()
	bw.next_object(input)
	return bw.value()


if __name__ == '__main__':
	
	def test(s):
		value = decode(s)
		print repr(value)
		print repr(encode(value))
		
	test('de')
	test('le')
	test('2:hi')
	test('i123e')
	test('i-123e')
	test('i-0e')
	
	test('d8:announce45:http://open.tracker.thepiratebay.org/announce10:created by13:uTorrent/180013:creation datei1227823783e8:encoding5:UTF-84:infod5:filesld6:lengthi721661952e4:pathl34:Monday (Tanaka Hiroyuki, 2000).avieed6:lengthi37118e4:pathl34:Monday (Tanaka Hiroyuki, 2000).srteee4:name6:Monday12:piece lengthi1048576e6:pieces13780:\x8b&[#\x9d\xf7\xb4\x06\xf0q}\xfeo\xd6X\xca\x00~B\x9d\t\x18\x1dTZoi)4\xa1O\n\xb3\x9c\xd0\xd8\x81\x99\x8d0C\xfaS\xca\xd6@;\xf6\xb6\x16\xbd\xcb\x89]\x95\xa6\x1b\xc8\xb0s`\x00eE\xf2\xc3\xdfY<\xc1\x7f\xcf\xfd\xdaI\x11\x91\xc8\x0bb\xeb\x0b\xf3\x97~\xc9b]\x9a\xe1\x8dF\xc5\x9d\x993\x9d\x8d\\f\x8e\xd7S\x03\xe1X\xa3or\t\xd5dz\x14\xaa\x93\xae)\xa4!\xa8\xfbn\xac\xa9\xd8wC\xde.\xf9%c\xb9\xbf\x86=\x96I\xe1\xbb\x08O;\xa9\xd6\xf3F\x88\x0e\x16v\xb6\xd4;\xe3\x7f<\x89\xa5_\xea\x84\xcd{\xcc\x9e\x96%\x9b\xe4\xa5\xfef\xc1z,:\xb2\x9f\xb4\x7fZ\xa4\r\x14vHs\x94PA\xc3\x12\x83a\x0e,V\xd0\x9e\xba\xd0\xbf\x02\xfd\xd6\xf8B\x9f*f1<I\xf3\xa4\xaf\xc4\x1f\x1b `M\x0en\xee\x83W\xe1}\xa9p\xd23\x91\'\xdc1\'\x0e\x04\xedeZ\xee\xdd\xc7\x10H&\xe3W\x16A+\x0b\x8c\xa9r\xb0\x84\xf4i\xb0\x8b:\xdc\xc7\xbd\xfaI\x8c_\x11Y\xfd\xd33\x9e\xee\xf1m\x92/P\x1cx\xd1\xf6\x87\xe5E\x1cR8\xe6\x1e_\x07C\xb2I\x93\xa7\xf8x\xf3\xaf`\x06\xd9\x0b#\xc3E\xe5\xe6\xb09\x02>h+\xfcw\xa4@\xeb\x97\xc0\x07\x8f\xe1r\xcc\xdc\x95\x04\xde\x9a\x8b\x0f\xd1\rz\xff\xbc\xe7\x0b\xc9\xcb\xc5(\xd3|\xb3\x9b\xee\xea\x019\xfc`\x0e\x0c\xf8\x88\x82\xa1\xb3\xae\x1c\xacW^0\x17\x04>t\x1e\x81\xa6\x9e#\x9c<\xae\xba\x0c\xf4\x97DE\xa3G\xedEU3-9n\x8a\x92\xca\x19\xa6\x08\x12\x82{\x102\x99\xa0X\xf3\xb6[\x02\x19b\xa2$\xb1\xca\xd3\xdd\xfb#\xee\x90\xa29\x18\xb6#\xf7\xb6\x8e\xf6\x1e\xb8\x9d\x90v\xcb\x8b\xec\x81\x9e\xbbD\x84\xef\x94\xef\xabm5\x03\xbeA\x87v\xb5\xcaO;;iJ\xc8>\xc9\xe2fzm&\x8e\x81\xc4\xee\x14\xb3\xc1\xc8\x872\xfe\xd8\x8f\xf7\x91\x14\xb4\xbb\x94\xb7\xe1_\xd8k(\xbe\xfac\xe6d%\xeem3\xb8c\xc1\xfb\x88Q\xbc\x87\x142\x03H\xead\xb7\xd8\xb9\xb1t\x9b\xeb\xfd\xfa\x84\xa4L\xfc+A\x175\xaa\xd8\x0fT\xd8\x11\x1d{\xfbRhd,\xb4\xda\x15O\x02\x1b\x03\x17\xf4\xf4\xc6w!|\xd6\xf9\xde\xc5\x9b<4\x8d\xb8R\x9cU\x8ed\xdcg/\x9d\x17\xdbD\xd84\xd0\xec@\xfdmJY&w\xe8\xcf\x01\x8a\xab\x08i\xf1\xb6\x1d\x1a\xcbq\xbb\xdf\xd6c\xb0,\xc2q\xd9\xa9J\xbcL\xdd\x8f\xc1\xe5+\xdb\xc0\xd8A4R\x08\xc1_\xc6\xbc\xaa\xaf\xe6\xe2C\xd1\x9bY\x1bjYq\xa1N\xb5O\xe5H\xeb\x9e\xef\xd8H\x15\xec\xb68\xde\\\x18/Qc\\D156fR\x96\xdaH(\xf0\'\xb1\xeb\x078\x1dKv\xf486\xdd\xd1\xcaBU\xf8\t\x00\x0c\x85\xd5\xfe\xd4Y\xf2\x1fE\xa7\xcbQ\x96\x9dyn\xc4\xf8\xf1\x0b\x96\x99ij\x9f\xad\xc8\xb4\xd8uz\xae\x92\x8c\x01\x1e\xea\xd1\xbe+:T\xc0\xd7\x95\xe9;\x92\x81\xf6\x954:du\xde >QU).\xf6\x95\x91\x0fK\xda\xe6XB\xdc\xabT\xfe\'3\xfb\xbb\x13\x98\x93\xf3\x10\xe3\x8a\xaa\'\x17\x1a\xe8/<\x81?\xc1\xfe^2\x13\xd1K\x01\xc8U~\x87\x1a\xa8\x1e\xc3\xd3\x85$^^Y\xc9\xb3\x9e hn.\x1e\x05\xc0+\x0c\xe1\x99\xba\x04\x94\xebI\xd2\x9cV\x05\xb7\xee\xbft\xacG\xa2\xdd\xbe\xf9\xed%\xf70\x86\xcc8\xfe~\xa6}\xc2\xef\xadx\xd7SY\x8a\xe0\x01\x8f\x1f\x94\x96\x16\xcb\xe5\xc9/}RJn<\x05\xfct\x91F\xb9a\x04\xce\x08W\xcc\xbd\xe5V\x05z\xf9h\x96\x81F\xe2P\x9c\xa1\x9a\xbf\xac\xa6\x0c\x07\x02\xe4a\x00\x18\xb1\xf6\xf8\x99\x04~I\xfb\xa2<v\xf4\x10\x0b\xd9\xe2\x13(\x1bD{\xea\xbdqY&\xf9\x9c 9\x00\x00V`\xc1\xd5\xcc\x81L/\x90TD\xee\xa8\xde\xfb\x9d\xa1o\x8d\n&\x9c3\xa7\x1f\x97\xfb\xf3\xaa\xf6\x13\x8bQ\xeb\xbd\x12m/\xa7\xda\x1cv\x97\xee"m"\x97\xbd|\xbf\x9c\x8a\x86\x8e\n\xb5a\x98t\xb4\x1e\x8a\xa1\xadg\xb4k\xa1\xb9Mr\xf5d\xe5\xa3\x85\xb4\x89\xea\x85\xc4\xcc\x96\x0ev\xa8\xb5\xf7u@\x7f\rA\x8e\xb6[h\x93\xa0\n\xd8\xbe\xa0GF\xd6\x14\x03y\xbe\xbb\xc5b\xe3vt\xc20\xaa\xf8\x11\xf2\x10\xc8\x19j\xa9\xeaN\xdaU\xf9\xdbs\x13\xc4D\x95\xfe{\xbb{\xa0\xd1\xee\xf3)\xe2k7\xad\xcd\xcb\xde\xde\xc6\x91ii\xafh\xfe\xa9\x93D\x1eK\x91\xd3\xb7\xd3V\x07\tF\xc7N5I\x7f\x89\xde\xf0B\xc2\xd3\xd1\xaf\xed\xed\xc5\x9f\xbdX8U\xa8\xd0"\xd2\x05\xcaj\xe7\xb0\xe0\xbc\xe7\xcc\xe3^\x99\xd93\xfd\x04\'\xb8\xed\xb9\xd4\xdf\xa8\xfeR\xd8\xde\xc2\'\xb3\xb0\xb1z\x1ec\x08~\x87\x14C\xbb\x0b\x99\xdf\x1b\xb5\x8d\xdd\x97\xd5>\xac\xe9\x1aQ:\x83T\x9b\x1f\x8c|\xb7\x17}\x80h?\xae%\xcd\x00\xe2>\x1dB\x96\xc1\xfb\xe0r?s\r\x04\xe5=w\xb1\xc9m\xdf\xbb\xd6t<\x988\x81%\xd6\xa9\nPV\xd6\xcb\x8ed\xe2\xc6\xeb}\x16:\x07\xde\x9f\x8d\x00\x1c\x15\xfa\xe4\x92\nC\xf2\xde\xb0\x10\xe4\x00\xb3UJU\xb7\x13\xae/\xe1\xb4I\x9d\x14c]^v\xd1~\xe5\xfe\x81\xfa)\xf3\xc4\x82G\xc0R\xa3O#9\xa3z?\x1e\xbd%f\r\xe3h\xc9"\x92\xbd%p{,z2\xc1\xdc\x1a\xc4\xb54\xbcp8\xc8\xe5\x13\x86\xd5s\xc2\x9c\xb2\xa6\xd0\xfa\xb2\x87[b\x15\xacFs\xef\xfeh!\xd3I\x83\x0c\xd6\xf9~\x05\x9eK\xee"\xffQ\xb4l\xef\xb93\xd7|\xf9M\xdd\x19H\x9cCAX*\x91\xc1\x06\xa2\xb3\xc6R\xb8\x86\x8b<q*\x13\x14\x19\xab\xa3\xab%\xff\xb6uH`NU\xb4_\xca\x8b\xc5\\\x1a\xf2\x91\xf1\xca\xb6\xc5\x844[si\xddK\x079\xa0\x1a\x15%\xb6\x14)\xc94\xe6\\P5#\xb6?\xfeS\xa5\xe9\xa6\xb8\xaf\x86\x13Y\xef[l6m\x949\xe0\xd3\xe3\x8a:\xe5Cu\xde\x1e\xbf\x05\xbd\xdc\x17\x9c\xf4\x829L\xec\xae\x06?GG\x93c^b\x029#\xed+\xb34\xb5!\x10g\x10\xdf\x17\x13\n\xbfI\x01=\xf4\x80\xe2p?/J\x14\x7f\x11\x1f{-\x8a\xe5\xc6\x7f0\xda\xdf*\x82\n\xeb5\xf8\x96\xc6_o>PsA==\'\xae`\x13-\x9e\x00\xfe\x15\xab\xc1\xec?\x94\xdd\xe0d\xdc\x84\x02\xec\xc7\x0eFc \xa4\x96\xd2R7\x1d\xa8Q@M\x0f\x13\x89g5.\xea\t\xb4&\xdau\xdd#L\x15\xb6|\xb7\xe3qIR\x16!\xdf\x1a\xc9&\xe3H\x8a\x8d\xc1O\xa6K\xcf\xc0\xfd\x0c<\xda?\x05\xec\xd2X\xe3;\xa7\x89\xbdB\x84\x0c/:(u3\x89\xb3\xfaMN\xc9X\xed\xa8\xdb\xcf,\x0eE\xd8\x9am\xa6i\x85\x8cO\xb53\x97O\t\x0cm\xe1\xa1\xc95\xe9\x1a\xc0O\xbf\x18x)\xf4\xce\x08C\xce1\x11\xb5\xa6\x91x\xba\xd9\xf7\xfb\xcdQ\xa2l\x9ej$\x02\xfa\xe5\xfe\x8c\x9b\xbc&\xefj.qk"\xbf<y\x12\xa0G\x05\x15\xe0\x12\x00\x05\xce\x10\x94\xeb\xf1\x9b\x88#\xbb\xe9\x8c,Q\x1c}%\xfdJ\xf3\xca1,\x9f\xe8\x96\xa3\xfdd\x9b\xc0\xfd\x061}\xc5\xe4"\xbe\xfc\x84\xa84;b\xe8\xd4\x89\x9elm\xf7\xa8\xef\xbf\xfcFq\xf3v*\x01\xd3\xd6\xe96\xfe\xbah\x15_)\xb6\x8d\xc8\x7f\xbb\x92\x97\x11\xe8\xce\xf2\x9dH\xef\x80\xcb\xdf\xacB\xf5\x1dM\xaeu\x88U\x05\x11\x98\x9f~\xb6Q\xceIZ&\xb4\xf3>r\x01\xf7\n\xe7N;~\x9bZZ5\xb6\xbf\xc6J9M2]\x99\xf5@f\xc3\xd0\x9d\xa1\x026A61\xa3H\xc5\x01\xae\x95\xdeu\x92\xc4\xa7Yfi0\xd6p\x91\xe2\x12[U\xbd\xed\xf8\x02\xf0&h\xaf\x19J\x03\x1bq\xf3H\xad\xb9q\xf9\x1e~\xe0eu*Hg\xb5\xfb\x883dn\x05\xd6\xb1\xb3\xaaY\x1e\x00\xab\x8c\xbe\x18%\x00\xe6H+Zv\xd7/_\xf93\xbew\xb5\xc5H\'\xdf\xbb0?\x93V|\xfc\xf0Ey\x1aB\x90\xfa\xe2\xcf\x1b\xbc\x84hM\x01\x1bx\xc9\xfd?\x9e\x13\xecj)7\x03rF\xbc\x96\xd3!\xe1\x08\x85\xf1;@\xeeq\n\xce\xd3\x94\xb0\x94v\xad<\xcb<\xf5u\x8f\xe7\xa1w\x0bK7\x17e0\xa2\x8eV%i\xea-\xcd\x83\r\xc54hR\xe3\x94\x02\xd2\xa8\xd7\xdc\xba\x9a\x93in\x1bx\xd2`\xd6M>\xc9M\x7fw-\xdf\xa4\xd0=\x99\x90\xf2Q5<\xd0\xeb\xbe\xde-\x80\x82T-If\xdc%?z5\x92\x06=l\xa6D\x88\xc1\xc0\xd6\x8eGB\x98\x91\xa3\xf3\xed\xd1\x883**C\xf4\xdeu/\x14d\x0f>\xaf\xcb\t\'\x96`\xca>\xc6\x90\xd6\xbf\xa9\xfb\xfa\xe38\xb9#\x05\xa0\xbf@ \xc1li\x9bo\xa3\x99\x86k\xaa)jZ\xe0\xe4\xa5\x1a\xd8\xbe7\x1f2UT\xf5\x93\xd0y\x8f\xf9\xf7\x18a\xaa\x93=~MX\xd7\x8bo#\xd3<G\x99\x14\xb0\x05\xea\rs\xbe\xa5$1\x8e\xb6\xbb\x13\xb08\x18D\\\xe2ZR\xb1o\xa40\xcf\xc7\xab\xa4\xad\xc2\xd7\xd4fv/\xfb\x02m\x8b\x93_\xcf\x83%\xef\xa1i\xac\x90\x91\n3\\\xc4\x8ax\x86\xfd\xb2\x83\xbb\x0e\xb2?\xdf\xd6_q\x80H(\x11\x0f\xcf;\x8b/-&\xcc5+\x9b\x92\xf0\x94\x8fT\xc1\xdc\xce\typ\xea1\xc93D#\'\xcc\xd1\xd8\xf6\xb3\x1afg\xdb\xabF\x99\xafzQ\x07\xf9l\x96\x17\x02\x1d\x8a5\xd0[\xd9\x9d\xabh\xaa\xa6x\x8di\xa6z\xa3`\x10\xc4\xcb~\xda\xecL#\xd4\tC\xf8\x85x\xcaH\x19m\xe4p\xf5\xf8\xbf\xe1\xf0\xa1\xa8\x0f\x90\x7f:\xd4\xf2\x82\xdd]\\G\xff\xdf?\xef\xe3\xe6\x99\xa1Vh\x8b\x88\xee.\xc2\xdby7`\xdf\xb6\xdd\xc0\xe3\xdf\xa7 \x90k(\xb3Vq;8\xbbX\xcf\xec30f:k\xbd~\xd5lx\xf6@Cj\xbb\xe3>\xfeZ\xa4^`n\xf4Z\x96\xa2\x9cGy\x02\r;\x98\xa0K\x05+\xf4 \'r\xdd>\xe5H\xa2\xf5\x1b\x1d^\xc0\x1b\xa1\xf5\x0b\x05\xa0\x83\xf2\xea\xc9B\xec\xa5\xa5g\xa3YYp\xf3\x00\xe1Z-+\xe3\xce\x0c\xeb\x8a*\x1e9D\xe0\xffW\x01\x81\x82lP\x00\x87\x07i\xb7e\x12r\xd0\x1c\xcaVm3\x11\xd9\xf55c^\x1d\xca\t\\\xfd\x87\xf5.\x8a\xbbV\xc2\xa2U\t\xe9\x81K|\xbe\x9d1dGCyz\x10\xd3[\xbd\x17:\xfb\xbf5\x1agC\xd3\xffi/S\x9b\xe7\x02Y#W\xe2J\x12\x1eF\xd7a.\xc2v\x04\x1a\x8c\xe02\xd8\xe4\x93\x15d\xcf\x00\xfa\x981\x15\x18\xcbR@\xc8O\xd6T\x81\x12\xfe\x9c=Oz\t\x93\x95\xcfpx\x8fn\x8f\x85\xcat\x1f\x15\x97\xb9g\x0b_\xab\x92\x03\r\xe9\xca\xc2\x8c~\xc5\x89\xd0d\x1bJNrund\xd2\r\x01\xec\x8b\\\xe7l\x10r\xa1\x14\xc2?\xb1\xa9\xc1\xcd\xac\x8c\x84!\xca5\xbf\x1b\x02\xc8\x1d3-8}\xce\x8eFKoA}\xfeX\xbf\x13\xb6\xd6,sH~Dv\xe8w\x05\x0b\x1b\xbc\xf3*)2j\xca\xaa\x9b \xae\xa0\x1fU\xb0\x8b\x90QO8\x971\x00\xf3\x14G\xa4+\x9d\xed\xe7\x11\xba\xc0\xaa\xb5\xa0n\xee]\xea{\xce\x9a\xcd\xa9f@,:`\xeffY\xd7rgyF\xdf\x81\xd1\xebi\x0cW\xefs\xa0\xe2\xf2\xe1Yk\x83\xb7\xfe?}\xee\xc0Y(\x91\xd4\x1b\xb9\xf3x\xa1\x15\x9d\xe8\x00\xf8V\xa3q\xea\xfd\x93\x9e\xa7\x9bO\x7f\xd1\r\xd4z\xab\xfe\x14\xd8;~\x8f\x8c\x12\x83b\x1a\xcf\x89\xfd\x81\x91\xa5p\x15,\x96t\xf0\xef\x9b\xfb\x85\xd73_&\x9b\x9d\xc4\x19\xed\xcf\xa6<F\x84\xca\x08\xf7U\x88\xf9\xa0v\x81\x881D\xdb\xe4\xc73\x0fG\xcfe]C\x95b\xf9\x8d\x97\x7f\x13\x90\xecM\x86\x8a\xcd\xbf{\xce"\xba\xe76\xebo\x9aKF\t\xe3G\xc3G\x95{\xdfM\xb17p4Zn\xeat\xda\x7f\x92W\xe3\xae\xbd0\x1d\xc0S\x8f\xa7\xbdg\xc8\xbbq\xf0\xe95\xcd\xa5\xc0h{j\xbaATE\xcea7ZU\xc8\xf8~=.\x05@\x91\xf5\'[U4*\xfb>\xea\x8bh\xc31\xf5 R_\xa8\xb8\xf7\xf8\xf9\xc2\xa5\xf6\xda\x14QC\\~\x04\xf0r\xd0\xc3\x03e\x85\xbc\xe0\xf7\x0b\x0b0\x19A\x85N@\xf3\xcb\x8e\xbc#\xd2\x83\x05\x05V\x1f\xacJ\xb3\x8aqg5\xc4\xe3\xc1\x19\xf1I]\xfc\xb9\xbfxq\x98\x96\xf8\xe9w5\xb93#9\xf2\x8d\xc2\x1c\x7f\x99\x9d\x17 \xf7\xf4<\xe3d\xdf\x85\xe8*\xa4UQ\x88\x9a\x9cy\xa1o\'\xfdX\xc6E\xc9\xf4#9\x8f.\xc0\xa2\xa9VY/\x15\x9b\x82\xae\x8a\xfc\x94\x99\x91\\%\xc7\xda\x90P\x9b\x89~\xe1i\x06\'u\x1e\x1c-\xce\x8d\xf7O\xfe(\xb6t\xd2[8\xaf(\x96\x86+\xa4a\x1fH\xd6\x13\x9f\xec\x05\r6=C\xbd\xe9(\xcfr\xdc\x14\xcdM@i\xd4\x081\x82\x8f\xb5P\x0e\x9df.\xa5M\x01\x14\xde\xd3\xc3\x08i\x9a\x0f]\xd2\xde\x89\'-@P\xee\xa8\x07\'\x86\xaf\x8c?\x17;M-\x9c\xd5\xe1\xf3\x88(t\xc1\xdarS\xae\xfe\xffY\xc3\xb7teZ\x05\xf9x\x9b\x88\x06\xc8\x1e\xb9\xcc\x08%NuA\xd70\x83\xe6\'\x15~N\x01\x87\xa2\xfa\xf9!^\xaaC\x91VR\x87B\xc3+R\xe6\xf3no\xf2\x83>K\x97Q\xa3\xc3\xdf\xd6\xb6*\xb4\x80&\xdb7\xbdwB\x8b\x01\x9b\x7f\x8e\x84-\xbf\x04b\xf9#\x131\x1eT~4\x1d\xc3\xa3\'\xf7\xda\xf4\xdbDm\xe7%#;\x99\xd1\x047\xc8\xbe1\xf0*p\xd3\xbe\xc9&\x1a@\n\x91;\x1f\x07(\x1b\xfa\xa9\x1f}\xde$\xef\x11\xcf\x10\x19\xfcb\tl\x1f\x05LU%a"\x05\xf9\xcd\x8f=\x03\xae\xaaT\xd2hh\xd1$\xf4%\xb7\xa6m\xa5\xb19\x17\x8dWU\x82\x0cj_\x90\x88\xd9\x14\x1c\xc3\xe5\xf0G \x18\x05o\x0b\x8c_\x87\xfd=\xd9\xf9\xed1<\xdfcR\xc9\xaf\x1eP\xbb\xd0\xc2N\x97\xcc\xc5,\x16\xcfm\x02:\x07\x15t5\xc7\x1a\x1d5\xe4\x8e)\x9a~%\x9e\xcb?.\xde\xda\xcey&m@=\x91!\xbe\xd1?}\x06\xac\xd6\xcd\x1c\xe2\x8c1\x7f\xe1\x0c\x964\x9aE\xab\xe4\xf4\xeb\xe1K\xbdV?hPO?9\xb3yo-"\xdb\xfc\xe2\x13q\xf1\xd9\xa3g\x12\xebO\x91pH.U\xf52\x14\x85\x06fo\xbdtH\xc2\xbc\xba(\xd3Q\xc5\x10c\xf3\xc8\xcd\x8a2;\xbe(\xcf\xe1\xe5\xdaOn\x86+xl\xf8D1\x16\rH\xe4\xd7\n\xc3`a\xa2\xdc\x08\xc1X\x96Qp\xb1\x0f\x06\x8e\xda\xb9\xc3\x8b\xca)v}$\xa5{\xe2a\xdc@\xfa3\xe2#\xf1\xd2\x01\xa3\xef.\xa0\xa1Sf\x05\xeaL`e7\xfb\x15\xb8.\x01\x07\xd0\x1d\xa5%xH\xf3L&2\x8a\xda\xa0\x9d\x00\x8b8\xca\x05\n\xe6\xdf~\x88\xd6\n\x18\x8db\x01;+#qw\x9a\xffJ\xf7i\xc5\x92p0raj\x00\xc7c\x88\x82\xbe\xfbg\x07)\xccg\xa4t\xa4\x9b\xe6h\xf6\xfd\xc9\xd6]t\xb2\xf4\xefT\xab\x8dDj1\x91\x86o\xb4\x93\xc2!ic\x1ak\x87jw\xf3\xab\xa2\xf2 \x9fp\xd9\x00\n\x0c\xf5b\xd4!\xfd\xe4O~\x91\x90\xf2\x97\nE\x1aC\x12\xf1\x83\xdb0Po\xc6\x95\x11 \xa2\x95\xa6\xcf(\xee\xf6^x\x89\xe2=wu\xf1\xc2\x1f\xc0)C\xab?=\x9f\x07\x7f\xb8\xb6\x88\r\xd3\x19\x16\xab\tN\xbcG\x9d\xe0<\xb6\xab#\xe1b%\xb8\x9eZn_\xddG[eA\x13\x19\xfe0\xbb\xc4\x85\xf1\x92\x17\xcb\x7f\x07c\x03S\x97\x04\xc5b\x9e\x17\xac\x16\xf6\x88\x82g\xf4\x1e\xf2I`\xb5\xcd"\x0b\xaf\x94\xbf\x84N\x9cr\xd2\x19\xbcC\xb2o\xa5U\xc6\xf1\x11\xda0\xd0W\xfb\xd4\x90Y\x10\x06\xfa\x96\xab>\xf0\x98\x8f\x99\xcf{\x19\x11Z=\x01\xb32\xe9T\x88j\x11\xe0\x01\xfcS@\xfcm\xd8\xbas\xbf6\x18\x1d`\x1e]\xbb`\x1d\x0f\xb4\x02N\xf2]\xd1\xfb\xb2\x97\x18q\x94A\x0f\x87\xc1\x8c\xd7\xb0\xc6b\xae\x9aE\xe8"\x0c\xb7\xf3$\xfe\xa7=\x1a\x96\x92\xa5\x1b\xa8\xf5\x98\x89#\xcaU\x87\xb2\xa3\x9a2\xc1q1\x97\x13c\xe6\x9e\x17\xbd\xc6(T\xc2\x86\xc5\x0c\x0e\x04&\x19@\xe2V{?\x88\xc6\xf2\xd6`O\xa0\xeb\x95u\x90\xd6\xd4\xb2\xe6\x10\x18\x08;\xe2\xfa-\xe0-n^\x17\xa1\xea\xf8\xba|+\xb3\x90\xc2b\xa4\xd9\xeb+O\x02\xf0\x17%\xe8\xbb\xecU;\xb4g\x1d7\xa5x\x17\x86\xb8\xfc\xd7\xd2\x08\x1b\x8e\x18\xb8;\x03M\x02\xf74\xaaR\xbe\xb7\n\xf2\xbf\xecK.W\xadc\xc5\x82\xb2\x05M\xec5\xbd\x01\xedn:\xb3\xc7L\x9f*\xba\xec\x12\xdd\x9e^\x1f\x8b$\xa1\x83\xb1\n\xfc\xd7ax\xfcA\x80\xdb:\xb3R\xb0\xbafF{\x04\x00G\x11\xb3=\xf1\xe6\xb2\x8c\x88\xe8x\xad\xdb\xff\xc1o,^\x0c\xd1\x01\xd4\xb9\x04\x0e7.\xc0\xf9\xa3\xf03\xef\x85J<\xd5\xa7\r\xa7\x1e\x80\x83\x08\xa2\xb6\x12\xe0#\xe3\xab\xed\x96U\xe8\xad\xf2\xbc\x1e\xe3\x03\xed\x0b\xa3\xa2\xfa\x9e}\x9b\x01W{\xbe\xf0\x08\xa3\x9e\r\xc8\x1ds\x04l\x8e\xeaw\xdc5\xeb\x0f\x90\x8f\x8c\xa3\xc8\xfc\xb9t7[\xfb\xc4\x8cI\x89\xcch\x8bJ4&iyGk\xa7\x8c\x93\xe8\xd0\xdb\xd3K\xae\x9a\xad\x07\xfaV\x03[Cf\x89ea\x96\xd2\x92\x08\x0cu\xc4\xc9\x82\nla\xc9\xdc\xaf\xe8\x08\xfa\xfayBOP\x94\xb8\x8b\xab\xb4li\x8b(4\x8d\xa4\xee\xe3\x00\x10-,A\xbe\xee\xca\xf7!\xaa\x83\xf3C\xd0\xfb\xff\xb6>\xeer\xff\xadd\'@V\x01\xeezB*\xf5\xb5\x83W8z\n\xbdr\xdf@e\xc4a=\x8cG\x1d2\xbao\x80\x17\xc4\x85\xc5\xec\x8c\x1f^\xea>\x12X\t:|Mv\xf1\x1d\xf3xY\x9c&S\x84e\xf5S;}V\x81\xe7\x8a\xa89W\xdf\\\xfa\x12\xb1v\xd9,U\x9c\xcc\xd6f\x14b.p\t+\xe9L\xc7\xf4\xd3\xff\xa2\x1a\xba\xe4\x04U\xa7\x06#\xce\xbf8\xba(P\xfb\xb8\x91{\x97\x99\xf7S\xba\xcbO\x88\xb8\x9a\xfd\xea\x16\x81\xfa\xe4L\x19c\x9d\x0c\x14=j~\xd8\xddT}X%\xc8R\xedr\xc4\xcb\xedI\x10$Ue\xcf\x1f\x84&\xda\x07\xea\xe0\xa0U\x82\xbaS(\x87\xa3n"9\xd9\x8aV\x1e\xe5\xbcz0V\xdb\xbeEj\x80d\x8d\xfc\xaa\xab\x08\xb9&\xde\x96\xe0\x97\xcdg\xaa\xc8\x15\x96\xd67~.\x88\x93\xe6\x98\x16\xc5\xdb\xd2!|w#\x14\r\x1c&\x8f\xfd\x9chp\xd3a\xdb+\xf7\x8fu\x19P\x0b\xda\x9b\xdb\x10#\xfb\xae#F\x9a\xe7\xed\xf1\xaeR\xbep\x87V@\xf2}\xb2\xa0>\x0f\n\x87\xe1\xdb\xa4\x90a\xfb\xdf\xb3&\xfe=\xa3\xf3$>Q\xd5<T\xe8\x96$\xa1Y\x1dHW\x16%\xca\xb8\x01\xe92j\xe2\xe2M\xd2X\xa4\x17\xa5\xba\x94\x16u\x899\xe1\x13\xe0`\xfd@\xd4\xd8b:\xcbgA\xe9t\x95\xf2\xe2\xab\xbdVy\x9c\xa3\x02\\\xa4ds\xc3^\xce\x9bNs\xffe\xf0_J\x0f\xfc\xd8\xf8\xb9DFR\xf78\x19{\xf7\x83_\xfa\xa8S\xb0\r\n:\xb2\xc5\xaaV\x19\xc7\x082\x08-\xf6D\x14\xc7\x89\xb6V\xf1\xa0Nc\xe8\'\x11l\xca\x19\xcb\xfd\x90\xa88\x03\xa0\xa3-\xe7\xb3o\xbb\xa3jD\xef\xb8\xa7\xc7\x99Q\x8a\x10_\\\x14\xa9T\x18I\xc9\xbe\xef/\x08\x0e\xa5\xdaK\xadWb2\x82i\xed\xbf\xc5\x96}\x0c<,\x167\x1b\xe1$W\xdddT[D\xa0\x87\xe7\x0c\x12a\xa41\x0f\xb5\x7fa\xbf\xa0YW\x83\xb0\xa1K\xbax\xc9\xf6w\x99Y\x1dY\xe4M\x05\xf9@\xa5\xa0\xf6\xe4r\xeb\xaef\x94vXw\xde\xe5\x0bN\r\xc3\xa8[}\xc5\x964\xbdy\x9d\xae\x9a^\xe7\x12\xd5\xf7Cy,\xe3\x02\xe0\xa3\xf1\x88/,v\x89FF\xe5\x8f1\x82\xbc\xa7\xacs\x83\xa9\x90\x06\x07D.Q\x97\t*\x15\xe0\x0c\xc0\xe6\xf4\xd9\xbai\x1d\xf3\xd7\xccN\x0c\xd6\xb1\xa5|\xff\x8c\r\x06\x9c\xcd\x8d\x92\x1aa`\xa7u\xedE8\x83\xa7"f_\x8f\x90C.J\xc4[@d\xfdV\x9a\xb6\xf7\xae\xaf\xacu\xa1\xea>\xfe\xecOj\xdcEX\xccT\x9eYqQ56\xdb\xda4\xa5\xa3\x10\x90\xb1\x1bT\xe5\xb2\xd0\xda\t\xdam\xb69\xc0\x08A\x04Yu\xde\xf0\xd9\xf4;\xdb\xabV\xedh\\T\x1d\xc5 e\xe0c\x86\xd0\xd8\xf2\xc6\xad"\xca\x0f\xfb\x05uta\x03\xc3\x8e\xe6n\x10\xf2V\xe6\xcdT\x06\xd9\xdfL\x85\xccV\xc2:Z\x97\xbe\xf8\xfa\xdc\x9b\xf7\xce\x00s0\x0cn\xe76\xc3\xb2~|\x88V\xae\x7f\t^\x9e\x87eg\xf0\x8f7\xd3i\x8dx\xce\xc2\x1b\xdd\xff\xd6\xedHZ[\x9aB\xf6\x15\xe2R\xa86].\xd7\x9b\xacq(\x8e\x01\x98j\xc4\xf4\xa6\xcd\xebH\x12\xf9\xfb\xdbs\x82,R\xfd\xcd3\x1f{2\xc6N\x8d9\xb9m\x01\xf17\x8a\xb2\xa3\xb6\xbf\xf5\x89`\x9e\xf7e\xe1\\\xa4\x0b\xee\xb0a\xaa\xd1\xad\xa32\x9f\xb0B\xb6{\xf0U\x88\xdaJ\x98\x81~\x8b\xafk\xc4\x06r\x084\xf84\x1b\xa4\x02;^\xbd\xd5@\xebZ\xaf2\xad\x87\xb3\x12fC\xde/jho\x92e\x00\x96\xdfw\xb7\xfe(\x05*Ow}Q\x19\xd8?\xbc\xea\x8e\xcc\x14\xd0\xa8\x02Ro\xba\xe2}\xd7\xf1dP\xfc\xff\xfb\x05\tz\xe4\x14l\xdag\xde\x88\xaf^\xad\n\x18\x8d\xec\xa5I"0b&\x04S"\xee\xdc\x0eWc\x14\x8e\xf0\xf5\x00\x80\t\xd0\x8e\x98"\xf3P\x8b\xf7\x0e\x81\xad\xb7\xc0}\xfb\xcd\xb5`\x1eY\xc2\xa6Q\xa1\xe13\xde\xd9D\x9e\x87\x11\x01\xd4\x9d\x1e\xf9l} G/\xc19\x9fm\x1a\xdfp\xcf\x01\xf2p\x08\xa7G\x0e\x02\x8f\x8ds*0c\xc4\xe6\xad!\x95\x9f:t\x8c\xac_2\x8a\x9aN\x9c\xeb#\x8b 3\xe6j\xc9\x87\xe6\xf0@!\xf9\xfe\x90\xdb`\xde\x99sN\xd7\xd1=\xed\x1b!\x16\xc7\xbc\x13\xab\x02\xfe\xa0\x00)}Dx>\xf8\xd4\xddIrkH\xfd\xed\xbcjR:\x06\x93x\xbfP\xeex\x86\xc5\x87R\xb1O\x89w\xd7~!\xee\xea\x83\x99\xa7W\xceY\x19\x0b2\xd3\x9e^\xec\xef\x12\xee]\x19\xc9GZ\xe0\x90\x0b\x14\x9eQ\r\r\x05\x13\x1f\xb7\xef\xdb0\xb4o\x98\xe9>\x82\xf1\x82\xf3,\x12d\xaer\x968^\xe6\xaa\xcc\x19\xcf(0\xd1\x0f\xfb\xa2\x1cMN\xc0OS\x1d\x1c\xc9\xff\xefBgE\xb3\xe4M\x04\x04"\x17=\x98y\xf8c\xe56\xc3\xe8\xe8dQ?d\x06\xe9\xe6\xa6\xee\x0cQ1\xcd8(M@\x8c\xb6\xbes\xd0f\x89\x1e\xf1} \xe1u\xba\x84F\x15(\xbcz\xfc\x93n\xcf\x1d\x99\xbd\xce\x82\xcfv\x9e\xc3s\xcd6\xcfP\xe9\x0fN\xbb\xc1\x05<\x95\x8aq\x9b\xd5\xa4\xfe\xd1\xe0w\xbe\xff\xbd\x9cz@nT\x80n^m\xefW\x98\xcc\x1b\xe1\xd1\xb3\x91\xd3\x89 \xdc\x86\xf6\xd1n+\x0eW\xab\x04\xda\xe0\xae\xf8l\xe0^^\xc6\x06\xb0T\x7f\xce\xcbI\xaaOy\xe6\xbf>\xd1c\xffS\x93\xf5\x99\xf5p\x9d\xe0p!\x86\x04\x1a\x88\xc8V\xcbyd\x81\xb4\x90a\xf1\x99;)w\xbd\xea\xab\x81\xd5\xd1\x12+8J\x16\xfa\xb9\xe5\xa3\xc1\x87\xa4\xb5Tq&\xbcL\x90\xec\xe6\x9aP\xbb\x199\x04k\x9e\x93`\xfaKf$\x9beQ\xccu\x14\r\xdc\xe8\xc8\x88\xec\xb1\xf2\x9d\xca\x9a\x87\xb6\x1c\xdfP\x14\xa4)\xca\x949\xb6\xa9\x9c\x8f\xde\xe1@\x1e\x8e\xd41\x85F\xca\xe5\x9e^!\xfe\xd5\xf1\x03\xe7*2\xd2\xa3\xd2\xba\x83\x03#J\xc1q\xfa\xebp\xd9\x805eliS=\xd3f{\x18]\x89%\xe7\xcb\xb4u?\xe5\xff\x8e\xf0\x8e\x01\x89w\xcc\x01\xd4\xa1\x8a%s\xde\xc3L\x1a \x90$\x96u\xd7\x8b\xa7w\xcc\xf0\n\xfb:$@\xe4o\\\xaf\xaf\xe4L\xda\x19\xb2\x16\x8a\xc0\xc2\x7f\xa6K\xd0\xac\x1dV\x0c\xa0\t\x02\xf4\xd9\x1e\x8a2\x05qj\xb8f-\x8d\xe3\x07\xc3&\x81\x96\x96/\xb8_Du\xc2\xf79x\x80|!e\xa0_\xfe\xf0\x01q\x9e\x041E\x93\xb3\x89T\xf9\xff\x96\xff\xb6\x8fL\xbe\xbd\xdf%K\xebK+\x87X%Y"J\x9b\x0b\xdb\x1a\x18\x9e0\x96\x8c\xb747\x85\x82\xcb\xd0\xb9\xabV\x8d\xfb\x7f2\n\xfd\xa5Mtg\xe0\xddQ?%Y\xc6q\xf4!\xf3x\x8ea\x12\xb1\xadQ\x14\xca\xac\xfd\xc1/m\x8e!\xbd\xeaW\xb8(\xb2\xd8F\x96k\xb78\x1aZ\xb6\xf5\xc84g\xed\x10\x80\xc3\x86\xbc\x81\xf2\xe9\xde\xff\xd1\xea\x12\xea4\x19\xd3\x85\n(\x1b~\x0c\xcb\x1a5\xc6tZu\xf8\xbd\xaf\x83L\xd3\xfcXx0\xb7\x81 \x0c\xe3\xbc\x0b\xbd\x154\xad\x87\xc2\x9c\xc5\x80\xc5s\xaa\x1d\x00\xa09\x9d9\xc4 \x93j#\x16\x835f\x04\xc2\x05\x1fK1~3\xf3 \x0e4P1\xba\xf9\x81)\xd2]du\xf3\xe4\x06\xd3%g\xda\xbc\xe2\x02T\xbe\x7f\xe2\x99\xe2\xb0\x8b#\xcfz\xc4\xa4\xd8S\xadGuG\x00s\x8a\x948C\x1cQ\xbf\x17\\\xf54t9\x1eDU\x0c\x7fo\xfa\xd3Raw%\x90-\x8f?\x89\x10\x0c[\xa4\xbc,\xa6\xe78F\r4\xa8\x05\xf94\xa4\x13\xe2\x15\xb3\x8b\xfa\xfe\xc4;\xfe{\xdc\xc3?\xf0\xcd`n\xb3\xb3c\x9cf\x0e\xdem\x9c\xe6\xa6\x14\x15:\xa3\xff\xb3\xc6\xf6&\xd3\x98xc\xd7\x91\\\x8c\xb4\x1c\xbe\'\xc0\xc0\xa6\xcc\x1f,\x9fP\xb8\xcb\x0f\xc7\xff\xb8ea\n\x1f\xae\xb8\x8c\xf7\xb9j?/\xd8\xe8*\xb0i\x866\xe6D\xdb\x15\xc6\xe0V!\x04\x19\xdb\x05\xfeA\xaf\x06\xf6DB\xea\xb0w\x7fS.4\x9d\x90.\xc1\x08\xa1h\x85\x05\x9d\x93L\xf2J\xb8Gr\xd2D\x88\xc7\x03eN\xc3\x1c\x8c\xd2?\xfd\x07\x7f\xba\n\xc7\xc9\x97[QzU0\xde$\x04\x03\x1eJ\x85\xe3\xe4\xc6\x1c!\x92d~\x92\x8d\xb1\'\xd2`\xab\x05\x1c\x0b7\xcc\xb4A\t\x94\xaa\xf1(\xc9\x19\x17\x9a\x14\xcd,K\xa2\xa3\x14\x8a\xa74{\xdaH(;\x0b\x82\x0fk.\xb6_\xd94X\xd5\x0b\xeee\xd5\x16\x19=Z\xcf\x94\xd3\xb7\xe2\xbb\x16\xc0\xed\x15\xb5)\'!\xe2\xb6\xf1\xf3@\x89\x0f(\x80{\x8d\x83\x0c\xca\x1a\xee\xab\x99\x08\x88\x11\xe4w_\x9d^\x1b\x0c\x05\xc8\x08\xaa\x11\xd63\xea\xf8\xfe\xbfM|\x11\x93\x15)<\x9b\x14v\xd1\xf5&\xa0\x13\x96\xa7b\xc7\x17:\xbc\xdfP\xd2\x85\xcb\x0b6\xb2y\xd9p\x06\xdf@\xb5R\x00\xfaH\x1c\x8c\x16\xb4D\x7f\x90D%\'\xd4\xb8\xac\x8b\x8a\xe7\xfd\xcb\xbaFu\x0bh\x9f\x95\x14\x95&\xc1s@\xbe\xa4\xe7\xe5\xe1\xf8\x0e\xb1\xcc\xef\x05\x81\x8b\xed[cc@qG\xfc\xa6\x1b\xd1\xfb-~\x05\xb9\xee\x98\xbd\xffk&\xf3=|\xe1!\xab\xa72\x17\xef\xbd\xde \x04{\x92\xd9\xce\x92\xcd\xa7\x9d\x94\xb6@\xf3\xc4\xd1\x08h\x96L\x05\xbbE\x9c\xe2I\xeeC\xcbt{\xdb\xa2\xb1\x92\x19\xa3\xfa(M`\xd2va\xa0\x01\x0eZ\x1b\xe9_]*\x14\x1e\x11uG\xac\xba\xe3\x1b\x01\xb8V\x0c\xfd1\xb35\rI{+\xaf\x1b\xea\x13\x0c\xaf\xd7\xaa\xa6\xfb\xda\x02\x94G\xabe#{\x90l;\r\x9a\xba\x0c}\x8e\x08!\xf31\xef\x18oe7\xfc\xc8\xb9\xd6\xaeB\x90\xbf\x03\x82\x19\xaf\tNr\xf8\xe9Ndr\xf6b\x16S\xe1\xf0\xaa}\xf0\x83\x85\\sY\xe5r\xc4\x7f\x07\x95\xdffN\xf4k%\x17\xd2ay^\x0b\x889\x83\xd7$\xee\nF\x00\x03\xaa)\xb8\xbc\xe9\xe2`v\x90\x90\xda\x02<\x85\xc7P\x92Img\x8c: Y\x0ey\x8a\xeb\x9a\x94\xd0\xd8\xd0\x9c\xb9\x07\xecJ.P\x82\xe9\xf6\xb3\xc7\xf7\xd3=\xab\xca\xa9\x9f\x93\xd8\xe8\xcf\xeeNR\xb61\xc6\x86\x97\xa9\xd8\xadJ4\x9e \x9d\x87\xaa1U\x84\xdd\xc2\x19lv\xb9\xb8a\xa9s\x8a\x93\xbfz\x9d\ni\xb6N*\xfd\xa2=\xfb=\xcdQ\xf0\xc7\xc8\xd6\xa9v\x7f7P\xc2kl\x06P\xac\xdf\x1a\xe1[\x1dUJ#D\xae+\xbcUHq4%\n\x9b@_\x19\x14\x90\xf0>\x97\xa17\xbf\xc2\xa0\xdd\xf4\xe1\x17A\xb2DT\xba\x19\xf3\xa7\xa4p\xdb\xdd\x99h\x18\x8b\xd8o\x95\xa6\x83\x12\xd0>\x9d\xa6\xdb\xac\xe7\xb8\x96lH\'a,\x02\xc6\xc0\x9e\xe9\xe3t\xa4\x0bx\x03\xbc$\xf1>-\x9d\'\x97>T\xe0\xba\xd2\xf6_\x18\x83\x85\xca\x1a\xe6\x05\x1e$\xe8\xb8\xa0\x08\xb9\xd4kt\x8fy\xa0O\xc8\xec\xf2\xd3\xd4\xef\xaf\xba\xd7\xde{\xfc\x0c\xaaP\n\xff\x19\xcd=\xc5\xfe \xeaH\x97.\x1c\x94\x8a\x80\xf4\x10\x0f0G\x8f\xf3\xef$\x8d3\xf5\xe6\x15\xd7\xbc\x17\x9a\x14)q>\xffFB\xaeG\xb6{G=v\xfb2\x91H\xf7O\x1d\x96\xa4\x04\xe8\xf2B62\x8bw\xc1Y\x9a\x13\x83sC\x02\x15\xbaMt+\xc7\te8\r\x81\xe6\xd6\x9e\xc5\xe1\xf7\xb6SE\xe7\x94\xf6v\xbc\x06\\\xa3H\xb8fI[<~%\xeaX\x86\xfc\x8b4\'\xd2\\q\xeb\x0c\xc6\xfb\x95\xb94\xf0\xe5\xdf\xf4P\x95LS\xcc\xc9W`\xebc\\\xf2k*\x9e4\xf0\xb6\xa7\xf8&\xf9CB7\xa1\x02Fr*8"\\\xf0\xdf(\x8b*}\x18\x03\x06\x89\xaa\x1bN\xa1\xe5fe\xdd\x96U\xf0\xbfbw\xba)\xeaK\xa9\xd7\xe3@eg!\x8f\xd0\xc0\xd677\x85\xb4\xd0\xcc\x07\x82\\\xe0h\x0e\xdf\xe6\xb3\xe94\x1c\xd1\xe21\x18\xb3\x84f\x80R\x01\xf9\xe1A\x80\xa7\xd8|r\xa3\x17o\x0b>-^\x15\xc9G\xf7\x05\xf1sK\xba\xb49\x0cu\x96I\xealS\xd7\xe1\xc4^\xe4\x98~\x00\x1e\xe8\xc0 V\xd7.\xe8\x86\xef\xed7M\x0c\x8b8R-(\xd9\xc7\xb2\x94\x01Y\x08\xbf\\k\x99\x82\x9b\xadu\xb1\x9b\xaa\x90G\xca\xf5\xf5\xe0\x8d\xf5\x85\xef7\xf5\x0bG3\x1d\xa0\xb3\xb9\xea\x93k\xe9\xc0\xfd\x8c\x13[|\xf1\xf3\xc2v\xc9\xdb\xb7\x06\xcc\xb7\'\x80\x08\x8bd\xb5\xe1V\xc3T\x0c>\xa7\x8a\xd5\xea|\x86\x9b\xcf\x81\x10\xa6\xb1\xe6v:S\x12\xb3\xa7\xfeYP6\x82ivi\xa4<Y`*_=\x19\xa6\xdc\x02\xcb\xd3\x07\xe07z\x86|\xa6$+\x0b\xa1\x98\x9e\x93\x08\x80~zc|B\xaa^\xe4\xee;i\x9f \x1cr\x07\x1a\xbdt\xd8\xa5\x1fI\xf6\x8alN\x190\xb1\x08\x86\xa7\xbb\xc0 >\xed\xcb\x8dl\xcb0\xaf\x9a|\xa1\xa3\xfb\x9c>\x8f\xa4\xc6\x94\xe4,\xa6_\t\xc2\xb4?\xff\x8a\xa8,T \xb1\xfb\x18H^\xa2\x86+&]\x9b\xb1\xefI;\x1c\x1a\xd7{\xe0(\xe9\xad\xf6\x0f*R\xccz^u.\xc6\x91\xf3R\x99\x96\xb1\xd73\xefYy\xeet\xfc\xd6\xbab\xf8\x18\xb3\x0c\xa2(a9\xc7!\xf0\x14^vL\xb5\x11\xc7\xf0QN\xcf\x1cUB\xe2\nm\xb6\x8d\x98\x99\xb8\xf3\xe2\x10\xff[\xad\xfa\xb1\xdb\x88I{\xdeu\x8a\xae\x93H\xb2\x98\x84\x85\x84\x12\xc6\xeeD\xa8\xcc\x96\xbd\xbc\xba\x8fis\x91\x9e\xc5\x80\xa0\x93\xf9rx\x8c\xb5S\xed\x1c\xbb\x9e)\xd5y\x0f\x01\xd1\xd4\xa8E\xe8~\xcb\x7f|\x7f\xfeo\xc2X\xc7\x1d\x1frS\x15\x01\x19\x165\xd3\x9a\xfabS3\xe1\xd4\xac\x81\xed\x9c\xa3\x1c\xcf\x84OiDGI\xe6x\xe7\x8f\x93\xe3\xa7\x97U\x9a\xf8\xd9X>\xfcV\xe6\x93\xaa\x97\x8c\xc1\'\xfd\xba\xa2:N\x07_\xfd\xdd;\x1e\x8at`M\xea\xf88\x0e\xbar\x97\xbb=\xb2(\xfe\xf2\x07\xb8\x8d\x87\r\xf8\x1a0(\x8e\xfb\xe6\xbf\r\x15\xc2\xe5V\x96\x91\xe6JF\xa3\x1e:+\x8c*\x1e\xec\x89z\x8fD\xf1&\xf3\x0b\xf6\xd6\x89X\xa4M\xa3\xadR\xa8\x8a6e\x90\xa8T\xf7\x1b\x95\xaf<\xe1z\x8f\x04\xf2\xe4\xa8W\x15r\x92L\x83\xb7\xdf\x0e-0$\xbc\x94\xb8\x994\xc0\x86\xeexS\xadtq{\xae\xb0W\xcf|\x88\xf0\xa7\xc3>\xda\x12\x04d\xefa\xb12C\t\xddvj\x8d\xe4\x10\x9e\xe0\xa2\xd7\xb9\xea\x1c;\x07\xf4w\x8e+\xd24\xb3?\x07\xf0V\xaa"kJ\x8f\xc9\x00\x11\x10\xdd\xee\x92\n\x06.3\x8a\xfe\xa6\x97\x7f\xe5Bi\xa9Ih`\xb2]\x9a\xd2G*Y\xcak\xab\x01&^?\x97\x98\xed7B\x89)\xda\xe8ag\xccI\x03\xbe\x07R\x91ph+q\xbcQ\xe8\xd7\x89\xfcNH\xfd\xf8\xebx\xa4\xd6\x94[_-\xfe\xc2\xcc\x89\xd6*|\x84$J\x8f\x06\x9a\xf3D\x9dLxJ\xf2{\x19-\x1bc=h\xe3\x0b\xe0h\xc3S\xd5\xbb\xfd\xfaiw\xb8\xc7#\xe8w\xd9\xbc\xaaS\xa6\xf3\xf9\xa0\xbdft\'d\x18\xd4\xe8\xde\xe2)\x14\xa7\x0e\xb7\x92\xef\xe8\xfd\xad9;G;\x10\xebi\x1f\xffR\xdd"eO\xb69\\\xcf\xafQO\xbf\xa6!q\x19\\\xb6\xf3\xa3\xf1\x98\x17\xd8`\xfe$\x18@8\xf4\xc7\xf9\xfd\xd9:\xee\'\xa5\x83+\t\xf4\xc5\xdf*\xba\xe9]\xe1|l\xe6\xbaa\xd1\x1c\x87\xd7p\xd8\x89d\xe8^W@\x81T\xa2\x98\xef\x93\x9aD\x9b\xbc\xf5c\xae\xaf\xb2\xde\xba\t\xbc\xe8\x87!\xfd\xb3~\xe6\xa1\ra\x13\x8d\xce`\xd1\x9f\xdf,\xf6\xa4\x10\x19\x96\xd0\xc3\x80,s\xc7\x95\xd2U\xf5\xc7e\x1e\xbeA{\x0bW\t\'\x92S\x9f5\x83\xe5Ow\xe7\xe7\r\xe9E_0\xc9\x8a\x05\x13\x0e\x7f\x0f\xf6\x18g\xd5\x95\x01\xb6V\xbe;\x94\xa4\x18\xe7\xb9h\x9a\xfc\x03t\xf7\xcf\xbb\xd4\xcf\n\xd1#\x00\xff]\xe2N\xa7\xc2I\xc9h\xec\n\x8c4\xcf\xd8\xe6nb\xaes-B\x97a\xe1\xdb"e\x9c\x13K\xbc\xcfF2\xb7#\xe1I\xed\xefw\xd2|*\x03\xe1\xda\x92.\xa5X\xbfk\xe9\xfer\x8b=\xaf\xe4\xa0\xc4L\x85Q@pt]5\x9fL\xddH\xdd8\x93\x8b\xf8\x99\xec>R\xe7d\x0c\x18P5Q\xd1>\x90Mc\xab\xc6M\xfd\xef\xec\x88(\xcf\xce\x17^\xcf\xd2\x0c \x98tn?\x19\xf1;GD{\x12\xa4\x02\x94\xc4/\x18*\xed\x07\x85\xde5\xe8bmvS\xde\x00P\x83\xd6!\xaf\x92\x0b\xe9SX\x0ep\x1aP\x9b]\xad:\xb2\x0e+8\x01O\x9asyX\xe5I\x93\x00+N\xf9\xc07\xa5nh\xf3\xdc\x1e\xc7D\x86DaD\xa5BB\x7f\xe3\xcd\xdc\x90\x80P$ry\xce\xbf\xa4&d)D\xd7W\xc2\x16\x852ie\xf9\x1c\xca\'\xe7\x0f\xbdx\xe20|[\x8c26~\xdaZA\xd7\xc3\x87s\x14z\xe94\x14P\x8d\xdc\xa7\xf4@\xd6\xa5(\x02\x99rf\xdf1f.\x0b1\xaa.\x7f\x07\xa7\xaea\xba\xd6\x99\xf4\x1c>j\xbb\x8c\xfd\xac\x84@\x83\x1f\xb2j\xca,\x88\xc3\xed)\x1c\xfd\xf1"h\x14\xe3\x1e>\x11bs\xd0\xf5\xde\x02\x15\x08,y6F\x0e\xe5\xf1\xe6\x0f\xf4\xf3\x10\x07\x1bB\x7f\x93\xe9\xe8<7\xfc\xd3\xf7\x9cp\xc3^\xe4\xde\x9d\x9d+gSf\xcaP\x91\xe9\xbb\x1ct\x0c2\xd2\xe2\xb1\x08\x86\xf0\xce\xb5\xcf\x85qZ(gf\xbf\x98\xc2,w\x0f\xd2\xfe\xe9\x06SyE\xc0LC\xa5\x9d!l\x05\x93o\xc5\xa7t\xbc-;\xaaK\xf2\x80\xfcJ\xf6\x10\xae\xc8\x9e\xf1\xed\x94x\xd8\xaaC\x8bL:\n\xa2H\x89\x07`q\x9b*\x8e\x16i\xe4VD.\xae9:\xea\x02\x82\xd2\x86U\x19k\xf2\xd97L\xe7\x08GH\xbb\x07g\xe4)\xb6a\x9e\x94\x1d\xfb\xd0g\xdfJ}\xaeH\x9b\xea\x06\x81\xb4Z\x02\x88\xce\xd6\n\t10\xda[a*\xa9F\x84,~\x0b\xafL=UX\xcbl\xe1\x91\xe4\xe7b\xc7r4\xd9\xbd\xe1\x951\x9dL\xd2\xc9M\xb93\xea\x98\xbf\xc5\xd6\x14O\xfdib\xe8{[^q\xcd\r[E6Fw\xbe\x1bl0,\xcb\xb9\xee\xedH\xbf\xca\x12j\xfa,,\xd9\x99\xf2\xfd\x02\x8c\x14\xd1/\xf8\xb3 \xf5j,5\x8a6\xcb8W\xa1\x96\xe6\x07\xf7\xdb\x8a<hI\xfe-\x91\xbf\x8e}\xb0\x89\xcb\xc4\'^W\x8a\xa70\xd6-\xe7]Y\xc6b\x91\xc8\x80\xe6\xba\x95\xe8\xf8\xe6\x17b\x0c^\xbbr\xa2T\xccq\xfc\xba\xdf\x16\xe8f,;\x96a\xe8\x8fo[\xa5\xdf\x7fS\x82\xf6\x15\xf8\xcf\x02\xd6d\xd7\x16\x1f\x07\x9f\xbd\xdc;]\x15\xd9\xf1g\x95\x7f\x05#\x08\xa8\xf1KgS\xac\xb3\x16\xc3\xbae\xbe\xd9v\x819\xb0\xd5\'\x8d\xd1\xe9A\x94/\xf1v&\x07h\xa1\x8f\xea\xf8\x14\x909z\xd4:l\x80U\xe8\xadaO\x0c|YqY\x86\xa6\x83p\x9c\x84\x91\xe1tw\xe4D\x11|\xb0\xf2\xf5\xd0G\xac\x90\'4\x16\xe7-sh\x0e\xe2\xfdM\xa7\x0b\xd1\xbc;\xbbyc#!{\n\xc1\x0b\xf4\xdfv\xc0Y\x8bI\xec\x10\xa0\xfe\xee\xab\x89\x0fr\xe3\x9b\xa0\xaa\xc7\xe3\x1b\xbc\xbck\x87\x1b\xdb\x8d\x1a\xc9\x93\xa1c=;\xe9~@qz\x17\xcfm\x80:B7\xc4%\xc6hs\xb6kN>\x80\xd6\x8c\xc8|\xa1)\xe1\xa4\xe1\xbc\xf5\x88\xdf\xba\xaeU\x18Zd\x1b\x05\xado\xf1\xa01BI\x10M\x8fd\x1f\xb4O\x1e\x18h\xe2g\x85\xa5\x90\xd3\x0e\xa4o\x15\x98\xd4WD\x0e\x83\x0e\xf1\xd1C;,\x07\xf4\x7f\xb5\x89\xf1\\+\xd0anuM\xdc\xa5 u\xd45\x0e\xaf\x88\xc55\xac\x18\xdf\xa4\xdfm\xb5\x1a/\x14Z\xe71\x17\xb2\x10\xec\xc9\xd2:\x0ewwKF\xc6\x91\xfc\xbfe\xc5T\t\xc9\xb8%5\xa5\\+,I\xc3\x9b\xe6H;\x166\x13\x8d\xaaA\xef\xbdP\xd9\x8e\xf4\xb2\x9a@\xa1\xe7\xd8\x9c\xec\x86\xd4\x1f\x1f\xc5\xc7\x8f\xec\x8c|\xa7\xd0\xc2P\xa2\x89\x06_y[\xd4>\x99\x86\x05\x8a\xe4\xbdX\x86<\x12\xc8\xe8S\xb3^\x05X\xf1\xa5\xc6\xbb\xbe\x1egV\xbbrk&\t\xb5\x80Y\x98\xfa\xae*\xce=\xa1\xa6\xf5\x98\xb106\xfb\xbb*\x00S\x88\x8e\x95\xf1\xd9\xa7\x08v\x0f\x88N\x14\xeeN1CM\xe3\xb0\xe7^h\x1ft\x15b\xef\\\xa4 l\x8b\xc8\xd9*\xa1I\x96k\xa5i%\x07LQ\x93\x10\xf6o\xb9\x96\xa5\x19\r\xa8=\xd4t\xccX\rJ\x89\xcc\xfe\x85J\xe4\xb1AT5\xde\xb0\xd8\x11\xe8\xc6E\xa3\xd2\x8fv;v\x1c\xe9<\xba\x86\xf9\xbdP\x8e\x15\x92$\xf3eA\x89I\xbf\xa8j\x8d1;t\xe6\xc1fB\xad\xc3s\xbekP\x02\x1c,\x1a\xba\xa8\xd6\x06\xd7\xe7\xac]L\xc3\xcf\x02\x9d\xc1\xa3\xe2\xeb?cU\xa3@)\x9a.\x97.Z\x93\xc6\x08\xd2EZ\xc5\x03L8\x0c\rX\x04\x12<\r\xf7t*[c\x1fL\xedN#\x0ey\xb0\xec\xf6\xfc0%\xe1\xe7rsX\x87Y\xf8-\xd4v_\xab\x199\x87\xbd\x1d\x13/H\x86B\xea\x80{I\xd5\xb7[\xf0\xad\xf7\xf6\xe3\'\x1e\x1f\x10\x95\xc0\x07\x89_C\x0cYb@\x81\x82j\xe4\x85\xaan\x07\x9f\xd2R\x07\xa8\xe7C{N\xbd\x04\xe7&\xbf\x82k\xaa\xc5\xe4\x02\xef\xc9\xa1d\x86\xc6g\x11\x91\xc5\xea\xe3\xd2\x02\xc1\xe7S\xc8\xf8\x0b\xc9\xeeR\xe8\x8b?K\xb04\xe7lS\x9f\xaa\xbf\\\xa3pWp\x12\xe6|0\x8d\x95\x14\t)2A\xd5`\x9eS\x9a\xd6!\xfa|\xdf\xcd\x97\x1c\xe1\xcd\x8f\x06\x96\x10\xdd80\xb3-\xe1\xc2e\xbas\xf3\x1a\\v\xfe\x88\xdc0T\xa4\xd7S\xfft-1\x85\xf2{\xce\x1a\x8d\x91~h\xd7\xf92\x92\x04\xb8\xffo5\xd7Y\xd63\xbd\x95y\xe7t9\xb1\xdd\x0cA\xad\xfb\'\xaf4\x80A\xe8p\xe8\x9f\x90\x150\x13\xc6%\x8b\xdeTk*\xf5\xdeoM\x01\x84\x8a\xe4\x88\xb62\x1d\xedT[Gl\xb3\x92t\x11\x00\xb3\x88\xa5\n\x13\xfd\xb7=\xde\xf7\xf9\x91\xf4\xdeP\x82\x13P\xa1\x97r?X\x82P\xed\xab\x86P\xd70\x8d\x95m\xe0\xd1~\x9cX#\xa5\x92\xeb\x8d\xca\xfb\x00p\x0e\x8f\xa7\xc00"\xbf\xb3a\xc8\x14\x8f\xb8$\xa1SU\xad\xd1\x1ek\x0b\xb6\xcb\x8b\x11\xd6\xb3\x81)H\xa4\xc0\x18\xfc\xbb/\x89\xa7\x16Iy\xaf\xca\xee{\x07g?*\x05&s\xfb\xec\xa5\x1a/1\x80\xf1\x18\xf1\xb3A\xfeB\x14\x84\xcc\x90\xb2^3\xfc\nS\xd5j\x86\x94$\xe2\xaf\xe9y\xf37\x1a+\x968\x8f\xaa+o\x1ch\xd2\xf8t\xd7\xd2\xa8i\xe6\xce\xa4_\xb2\xc5\x12A\x92AM{\xda\xb6@\x88\x1c\x8b\x0e\xe2\xd4-\x8f\xa2D6O\xac\xd0\x15\xba\xee\xef\xfd\xca\x0e4-\xb2-o==\x1a\x0cK\xb85\x80\xf5{X!V\x94\xe5\xa7w\x95\xb5 8\xf3\xe2\xe8\x1b\xd4D3?\xf5\xcb\x18#\xb3\xf4\x89gzX\xdbHH9\xad\xe5N\xb6\x81ZC*Sq\xc6\xa2\x96\xc4\xd8\x06\x85\xaf}\x1e=^\x9d\x94\xb3\xb7\xc0\x8f7\x05\x13SO\xb2\x12\xech\xb1\n\xddB\x17\xc2\xa0\xd7\xb0\xcb\'&\xc2\xf2\x8f\x8dC\xed\xc6\x1b\xf8\x08\xf6\x80\xebF\x856\x06\x0b\xc3\xa8\x91\x8a\xaa\x88WZ=\xe5\xaa\x14\xfeg\xdeh\x12\xa7G d\x03\xeb\x82T"\xde\xaa&\xc6\xcb\x99\x8a}y\x02d\xfc\xf2\xae}\x82\xc2\x98\x07\x92\xab\x12\xe9Yc\xfes\xe1\x00#\xc6K\xfb\x1bt\x0b\xdd\x89\n\xcb\xb2C\xc2\xb9B\xaa\xe1\xfc\x13\xbe\xb3c\xd0\xbb\xf0\xb4\xc1+d\xee\xff\x19L\xa1fW*Q5\xfe\xa5p\xd3:i=U\xfexT\x97\xff\x85\xe4\xc3! \xcf\xbeQ\x90\x82*\xb8a\x8f\x8a`\xee\xf9\x8el7\t\xec\xeb\xb2\xfd3\x1f\x19Va\xb0\xb4\xe8\x1f\xcd\rq\xa8f(`\x88\xd1@\xfd\xd5\xb7\xb9\xb2@\x13\xdd\x916\xf8\x83\xfbxD\xfe2\xac*\xae\xe0\xa7\\\xc6\xf7\x11z$\x7fL\xcc\xd1\xe1\xb0\xbb\xc2\x8e\xf6\xd1cv\x1cC6\xf2\xaa\x00\xfd3\xf7\xbby\xde\xe8\xd8\x059\xea\xfco\xd8w\xdd\x96\x10K\xe4\xda\x14{\x08\xcd+-\x04b\xd7l\x85\x02\x9a\x99\xbd(Q;\x10\xcdyNB\xb7\x81\xa6\x8f\xa7\xf80\x00\xf3\xc3\xd5q\xb3\x8e\x00|G\x96\xacSA\x92\xa1z\x88\x15\xef\xc1]5\xd3,7\xe7\x88\xf0\x93\x16k\x11X\xb2@(\x08\xaf\x91\x80\x03\x00\xad\xcf*Ya\xad\x97&\x16Q\xc2\xdb\xa9\xf4\x16\xfd\xd8E\xbb\xf1\xf4\t(\xbd-\xeev\xc0\x1d\xdeS\x9a\xd8s\x9a\xc0\xbe\xf6.\x06\xb5O\x07\xa5\xd0?r\xd4\x8a\xcc\x83\x10R\xf5\xce\xc6\xe9#?j\xb5s\xa1V\\\x87\xca\x84!O\x11\xbe*w\x92\xf1\x18\xd2\xdb\xedO3\x0c\x7fu\xf5R\x8a\xb4\xa3b\xb0\xce\xf2)\xedfv\x06\xc9\xd95Bjhf\xd7\xce.\xc1\x12=|\x84u8\x96M\xe7P^0\x13\xb4\xb7\xa2\\B\x1d\x90\xfddJ\x04\xec\x15\xc4\x80\x7f\x05\x02\xa7\x0c\xaf\xce\x8d\x94~\xc8(I7B{\xd4\r\xd8\xcb\t\xa0\xe8\xec\xa3p\xf9s\x02T\x9f\xfb1B\xb3%_\x01H\x04\'\xc6\x924\xb9&\x89\xc6d\xa2\xce\x1b\xd1\xce\xec\x8c6\xd9,\x11b\xc1>\xd0a\x835nl\x94C\x06\xa8\xafmj\x01\x13q\xdc\xce\x1d\xddM\xc9\x1a\x02p,\x98\x80\x05`\xcc\xc1\xd0\xac\x87\xb1\xb5\xbfv\xf7\xb4\xeb\x1f\xcc\xc9S\xc3\xde\xbb"\xa5`}g\xd6\x99\xb3\xf9\x12\x8a\x8f\xbb\xe0\xabb\xcc/u\xa3<\xc0\x91\xff\xf3\xe8A\xb1\xec\x0c\x03#\xc4m\xf3]\xa7m\xc7\xae\xdb-O\x05`\x9c\xa5szn\xf6\xa9\x8e\xa0X\xd0\xee\xa1/\n9W\xc6\xbaT6\xef]\x1fe\xb4ktk\x1d:x\x173XV\x063\xc4\x18\x14\x15O&K\xe9>\xc2\xfa\xf3\xf7\'Y\x16\xb3d\xd8\x9bm\x17v\xd4k\xc2\xe1Y\x7f\xe0\xb6o \x8c}\x81&N\xde\x0b\xf3@l6\xcd\x9a\xaeG\x0c"\xc1\xaa\xf1gC\xed\x82H\x98\xa4\xdfmC\x93\x8c\x0c\x16\xf6\tz\x1e\x04o\xe5\x0b\x19<jA:\x9c^\xb7f\x16G\xe72K\xa5x\xe14Lw&g\xea\x11\x00\xc3\xf9\xac\x8bB\xbbN\x1f\x07\r\xf1\xf3\x08\x88\xa5\x92\xd4\xf2o\x0f\xeeK\xfbHp8\x8b\xa5\xad}z/&[$\x14\xd1\xfd\xbdfq)\x82\x8d\x1f\x9cR\xf1\x11\x98\x06ER\xc9\'\xa8}\xb1\xaf\x0e\xc6,\xc2N\x8e\x91W%?\xf8\xc3\xaa\x0f\xc87\x01\xdd\x9f\x06x\xcf\x89h\x18e\x94<Io\x0c\xc0\x107\x15G\x9e\xf7\'\x05e\xa2\xf0\x0f\xb8$\xa7\xcb|\xdc\xeb\xcfq\x8f\xa7\xc1}A\xec\xc7\xe7*c3m\x0f~hZ\xfe\x18\xc7\xbb\xba\x1e&\xf0_j\xbb\xd1S\x0f\xde\x17E\xdc\x8d\xf1\xe0\xe0\xa2\xa4\xe8\x1c[d\xc4TR\xa3i\xe8Q\xa6\x10\'\x9a\x16\xd5\xbc\x92\x9f\xca\xddj\x02\xd41\xaa\xcd\xf3d\x1bPh~\xb3\xc8\x14hv\xc2B\x992^\x00\xec\'\xe8=\xbf\x07#\xbeb\x9d\xa0\xc5\xdf\x05x9sU#\x04\x17\x8a\x19\x9a\x12\x97\xaa\x07Y4\x13I\xeb\x0c\xbe\x99 \x18\xd5\x0e\x8c\x94\x9c=9G\x84x;\xca\x83\xf1\x85SX\xeb\xfbj\xa1W\x1e\xd1eV\x0f\xf7\x1a\x83I\xa0\x9bi\xb6%\xb3H\x03zh\xbf,\xbeq\xb3\xc8\xff\xf7\xf4Q\x98%I\xd2}Vf\xa9\xde\x12`F\xdfG\x00\\\xa2j\\\x92ow\xd6\x1f\x7f\x1d\x8d\x9d8\x9d\xd1\x98]#T\xa8s$\xb4\xa9\x9ab\xf9\xc1s\x87\xd5\xeff\xb9\xc8\x1b\xca\x82\x00\x19\xf9\x00\x9bGu\x11\x19b^<\xb4\x8bn\x95\xf6\xdbp\xe4\x89\xcf][@\xce\x14\x9d\x91`\xae\xef\xd9\xf54\t\xc1\xac \x11\xe8~\xee\x0e\x0c\xc7T,>\x9b\xa8}~\x15\xf6\x99\x15\xac\xe3\x08\xb7|\xbc\x8b\xab\x0f7\xb6c\x02\xd5>\x9bk\xcb9\xfa\xc2%h^\xda5_\xd9QT;\x81c\xb6\xc8\xd5C\xdc.\xf5\xc8R\xb9\xed\r6\x92S$\xa0\x08N/\xf5\xbbT\x83\xb3)\x89\xdfE\xd23%\xb5\xe7\x1c\xe1\x90\xdd2\xaa\xfc\x1cS\xd1\xc9P\xa4oJ\xd9\x07W\x81\xe6\xe81\xc0\x1f`e\xa9\xbfn\x85\x87\xb4gS?\x93\xd2\xe2]\xdc4G\x80\x99\xben\xedK~\x84$;(\x82p\n~\t\xe3L\xffX\x1el/3)/\xd6\xa6\xcd\x7f\xfa\r\x87}S\xa6_\x8e\x9f{\x84\x93\xe2\x8b\x86\xd9\xaf\x00\xdd\xa6\xfc\xbd\x80\xf8\xe7-\xb9\x18\xcf\xa3D0\xdd\xe5L\xfa\xeb\x17\x90\xde\xc5\xc7IC\x11jX\x9e\xd7\xe8}\xd9\xbe\xc0\xbd\x0c\xa9\x9ev\xbbT\x05\x9a\x85/B\x93>\x97i!VY\r\x14Jq7\xf7\x08\xb3\x9f\x87\xa8p?>z\xb2\x95\x18\xbc%\x8fc\xb4\xfd\xdb>;),\xa2]\xed*\x100i\xc6\xba\xdf&luQ\xc1\x1fxc\x1b\x8e\xd1\xbc\xe8=\x14\x8b\x14M\xfbvz\r\xdd\xfa\xa2\xdc(\xf0x\xf8Z\n\xae\xb3\n\x06\xb6\xed\xaf\x89\xb2\x14U\xa1\xe8\xa0\x92\x02\xe7\x00!\xac\x0b@P[\xd0\xe2/\x9d_\x1eTl\rL\xea\xb2#\xfe\xc7\x0eDC\xab\xc1\x10g\xe23\xc3]F\xee\xd4\xfd\xae\xda$\x0fR\xd5\xc2H\xf4\x1f\xe2\xa8y\xf3\xeaAm hi\xf9\x13j\xf6\xd6L\xe1f\xa5\xf2\xca\xd5\\N\xfc\xe0\xd2\xe2\xf1\x10\xc1@t\xa2\xb1^\xd4]\xeb3\x00\x15\x8dMid\xdd\xa4\x1c2\xdeW\xfc[\xf6V\xb24\x1e\xf0\xefV\x00D1\x90g+\xb9\xa5i\xd3\xfc0R9M\x0e\xb4\xde\x9f0?\x00B2\xccQi\xbd\x97\x04\xcd\x93\xc8x\x8e$_\xb2\x9f\tyB\xb9\xf9\'\xbe\x1f\xc4\\\xb7\xf8P9\x89x\x94*\x1aG&\x1a\x9b\x19t \x8f\xf0\xc4>\x0e\n\xa4\xa2{\xf8\x01%\xaa\x8c\x1f\xc9\x86\x10k\xe2%c\xb2\xa8$T\x14\x06FU\xfcM\x9f\\<C\x8b\x14c5NX\xe4\xa6\x8f\x97k9\xa6\x94\xd41\xae\xbbM\x9b)\xb8\xae\x7f(\x00\xc79 \xd0\xc7"\x18\xf4\x88\xd9\xcb\xed\x08f9\xdc\x89\xa1}\x04ak@\xb5{\x80\x9f\xce[p\xf8\xe1\x85\x9b\xd8\xe6&\xfc)\xeb\xcd&\xb5\xb0\x0fS\x02F\x89\xaf\xa2\x08.\xc4\x83r\xd9d\xdf\x0e\xc9\'M*#\xcaQ\xaa\xc3\x92\x8c\xf0\xf2\xf6\xcfI\xa0ch\xee\xba\xfd\xe4\xf9a\x16J\xa9\xfe\xaf\x10\xf4a\xc9\xb9\xd6G\xc2\xca\x14\x0f\xb8\x1eZ\xf7\x9dY\x10Fjy\x9cf3w\x08\x99km\xf0\xae]a\x81\xb7\xc3Q\x08Q\x86Z\na\xe7\xa9\xed\xfa7\x89|\xca\xca\xf5\xb6\xce\xd3.\x02E|}\x1e\x96;\xf3\xce\xed\x1f\xa3\xb7L9\x1f\xd2\x84\xa0\n/\x19\xff\t?\xe3\x08\x8eN\xa9\x92\x06\t-i\xf9\xe8Q\xe7t\xa6\x05R\xa7\xb7\xca\xa1\x18\x04B?\xac\x18\t\x97\x99z\xf1\xb2M\xba\x93\xc9}\x8b\xfeD\xb1\x17\x9a\x9b\xa8$\xcdoMX\xb5\xef;\x85\xd4\x02\n\xbd-T\xb2\xae\xf0\x91\t4H\xa6\x84\x86\x07\xb4?\xb4\xc0\x1d\xff0\xf1w\x88\xd3\xabC\xe8,\xfe0\x99\xa1\x04v]H\x08\x9d\x92\x0e\xe3[\xffI\x96\xb95\xb2\xa0\xc8\xd2\xf6\x12\xa4\xffG\x7f\xa4\x13\x9569\xb1v\xa82\x85\x85\xdfO^\xb4Q\x8d5I\xbb\xb7\xf24\x16\xfd\x08\x8e\xbd\xc9\xf7\xfde\xde\xca\xc3\xba\xe9\xab\x0fZ\x06\xd2|\x05\x14\xc8\xc0Z\x15\xb9p\xf0\x06\xeb\xd0\xfc\x08\xdc\x88\xdd\xec\xdaz\xaa\x1c\x8b\xd0\x9a\xb5\x06\xe7.\xf1\xe0(r\xd1W\x85<\xdcD56\xf4\xb6Gd $\xee\xf0\x0bg\x0c\x1aG\'n\x87\xddbX\xd4\x8e\xcf\xed`A\xb9\xf3\x98\x8f\xbc\x10eC\x94\xc4\x05\x19\x8e\xd9v\x05\xbdts\x0b\xdd\x1bUB\xef\xf7_\xf2\xa7:\x05\xec,\xe2\xf9OX\x96\x13M\xee\x86\xb2\xa45h\x0b\x96\xfeC8\xd00\xef\\\x10\xa6\xe6\xd7\x18\x88\xf7\xfb\xe2\x04\xbb\x11>K\xcd[\xb1\xa9]p\xa0V\t\x89\x93\xca\xcbN\xfe\xde\xef\xcc\x1d\xad\xbc\x0e\xd2b\xba\xc7\x06\x1f\xbc\xcb\xbf\x18\x8b{\xa3\xd2D\xed4\xf4CbRl\xb6\xd3\x9d\x9f\xc3\x0fzUPE\x07WIv\x88\x84\x95\xb7?\xf5\x104G\xdba\xba"\x88\xe4\xf1\xa74\xaa\x17\xd8\x9f\xd8\xa8\xea\x920\xc178\x9a2\x94\x96B\xfe\x80@\xa7\xa8P\x1arg\x00~\x04@\'qH>\x14\n9\xb3{\xcfb\xca\x8d\x14|#%[\xd5\xf9\x84E\x80\xe8\xed\xc0\x86\x87fl\x06\xb6tu\xfa\x19XF\xc5y:\xd0\xb0\x82\xe7Q0\xaf\x91\x90\xbd\'\xa6\xc4\x10\xe9`e\xa3\xc1z1\xd7B\x88Nk\x94\xb0\xc4e\xd5\xab\x03\x93C1\xcbd\xe9\xd7|5\xbe\xd3\x95\xf6\xeaz\x80HEY*\x0b\xd0\xb2\xdcV)u\xcd\x1ct\xd1\x87\x03\x9b\x88[k\xf5\xa1\xf0"P4\xcc_\xf2\xc9q\xddI\xd8\xb8\x1f\x0fm\x93\xe9<\xec@\x9baW\x8dEw\x7f\xda\xde\xaf\xcd\x92\x13E\x1a\'F\xac{\xc8w\x18\x04\xe0R\x9b\xbfR\x9f\xe4S/T\xcf\xb9\xaeG)\xcd\xf1rh\xa2\x99W\xaf\xcfn\xc1N\xafs\xf1\xbeX\xdb\xa2?h\x85\xdcT,\xda0`#)6\xee\x80\xe9D\xebL2\xa9~h\xe1\xecF\xb6J\xeeF\xee\xc9\xf3-eI\x1a\xc5\xba\xe7h1\x13\x05f\xe2h\x0f\xc8\xf6\xdb\xd4\xbeJ\xf7\x14\xebS\xdb\xef\xc2\xfd0\xc7\xf8\x92\x06\xd2\x92g\xb7;\x0f\xd2\x15\xbaB\xe2H\x88|\xb4\xd7!Q\x9a\xfd|"\xe66\xd4\x1d"R\xe0\xb9O\x83\xac\xb0a\\,c\xe52~\x9c;\xe7/\x0b\xbf\xff\xf1% \x10\xe2\x8e\x86\x85\x08\xe6Z\x8a\x1c\x0b\x82`\xef\xf2M*\xcd\x14S6ZC\xa6s\xaa\xe4Uj\xfa\x93y\x9d(s\x0bS\xe6\x9f\xffHsL,R\xa2\xaf\xd9\xd0\x12\xd3+\t\xe7\xd8\x99\xdfi\n\xf4\xef\xd3=\x94\xb6c \xec\xdb\xa4\x92\xd3\xb0\xfag\'\x14\xf1\xed\xaaa<p\x17\x8e\xf9Vbh\xc3M\xfc\\\xcau\x87\x02,\x92nOI\x19\x83\xe8#J\x8a\x84\xf6\xd0\x011\xfb\x14\xa9\x82i\xc6\x9f\xe6\xfe\x9e\x8f2\xa2z2\xa3L\\5\xe9\xe4`\xcb\xd1\x84\xaa\x01|~\xaa$\xb3\xa44\xf6\r%\xca\x1dL\x83\x14\xec\xd1F\x91\xd3N\x96\xc6"\xe7lliRq\xd680\xd1\xa0\xad\x1aj\'\x90\x81\xcb{4\x94\x03Y\x01!\x8cdu\x04\xdf\xf8^j\x8d\xc72e\xbe\x9a\xed\xb44\x83i\xe3\x80\xdc\x06a\xec,v\x93\x1f\xb1n\xc2e\xf7\x07\xb7\xc9}\xfc\xab\x06\x9c\xebh\x0cZ\xb8\xd2\xb9\xb6)I\xcbH\xb5\xa8\xbfV\xab\x90\xe1\t\x81nC;\xd8[2\xee\xa0\x86\xb4:\xb0i\xaf\x94\xc1\x10\xad\xdeJ\\$\x9b\xa2<\xe9\xa4\x91\xadBe\x1e\x9b\xa4\xc9\x05\x1d\xf1\x01TT\xd1\x9e__\xe3N\xe2\x0e-\x1a\x04\x96?\xe5\xf3!\xa4z\x94\x0fS\xab\r\xa7\x83\\X\xd5\x84\xa0>\xba&\x17@B\x9054V\x96\xc7\xc3\xeenn\x86\xab\xe6.\xdc\xbd\x19|\xfc\x9a\xef \xd6f\x84\xae\xfcY\xac\x945\x8b59v\x99\xa3S\x05<\x8d\xc2h\x11\xb1L7lu\x96\xbe\xd1\xff\x86\x1d\x06\xa5]#\xb4\xdf\xcak\x95R\x8f\x98\xe7\x90\rX\xec\x13\x8fd\xa5\xd3L8\xbe\xc0\\\xec\xa9b\x1fZ\xe5\xb7\xd8\x88<5\xb6&\x1d\xd3Y\x93>\x9fC,\x9d\xdd\x80\x80\xb5\x96\x0eT\xd4\xba\\|\xda\xc1s\xc5q\x02\xb6\xeab%\xbc\xc7Fz@#m\x12)\x07\x08\x1cM\x8f\x0fJ\x0f\xb5\xd2&\x1dW}\xef\xd5\xed\n\xdb\xbf}\xf1\x19\x03>E\xb0\xdaZ\xc2\xcf\xfao\x07@.\xe9\xb3\xb3\x1bcQ\xd3\xdd\x88{}^P\xf9\xdc)\xa9\xa7\x9b\xfbS\xb2\xbd\xea\xb2\xc7\xbc)\xe1-\xb3WT)%\x18\xee\xa7\x90\xaf\xef3\xaa\x85F28w\x92\xe6\x1d\t\x94#\xe4\xdb\x8c\x91\xec\x17i\x00EX\x87\x9fw\xc1\xb8\xee\x93\x04\xda\xde\x83D\x16 \x86W\\\xfc?F\xf12\xc1\\7\xb9t\xd0\x0cd\x14"\x03 \xa4j\x8d\xaa\x04\xe1\xcd\xbc \x99\x0fgJ\xb4K2\xef\xf8\x96}\xed\x1a\xdb\xc1\xe5x\xb42g\xaaj\xb9\xcf\x97\xfdf*\xb4\xf4-\xe6?+\xc6\xcc\xdd\x88\xb5\rFH\xf8\x15\xfd\xb3\xc8+\x0f\xd6\x03\xc6\xbe9\xf3\xf6$+\xecl\xa8\xff(\xcd\xa9\x06T\xc8S\xf6\xd1\rw\x0cm3\x99\x01h\xb5\xd6\x19\xb9\x18t\xd1~IC\x9c\xc9\x91\x82\xa9nd\x10i\x11E\xa6\x97;-:\x0b\x06l\xda\x13\xd0`\xb8\xc7=\xa1\xe2y\xc9\x9bey\x89D\xec\x97~\xa2\x96\xebF~\xfa\\Pm12\xf4\xda\x1da\xd7\xe3\xddX\x99\xb9d8\xa7I!\'\xf5\x1d\'\xcc\xdd{\xd0S\x18h\x1cc\x0c\x94\xded?\x7fZ@\xb0\x89<\xac\x90\xe0\xda(%\xf7\x8fk\xbd!\xb1\x18J\xff\xe9oX/;\t\xe4\xd2\xc2\xf5\x0c\xc3\xb6\xb1G8\xa0\xc2\xe4\xc8\x14q\x1c8\xd2\x1a\x9e\x8f\x99W\xc5\x1fg\xb1\xee\xd1\xdaG\x19h\xe8Z\xb9\xc9w\x80\xb6\x8fd\x8b\xa4\x99|Yj\xb8\xd4\x9f\x10\r\xf7`xm\x9b\xc2U\xb8\x86l\x97\x0c~z\xfc\xdfk\x88R\n;\xe93\xdceU\x1c\x97\xfc\x7fKk\xd2j\n\xa2\xf8\xdb\xdc\xf4\xd7\xfc@\x1b\x93\xdb\x9a\\\xd5\xaf\x00Eyh\xb35\xa6uC\xda\x93\n\xc0\x94n\x02pQ\xac\x9a\xadB\x05\xc4\xf6\xff\'k\x16z?\xc9t\x07;{\xd5\xad\xba\xdd\xb64\x9c\x04\x1fxG|\xf2\xb2\xdd^P\xd3\xc1\xa3\xfe\xde\x10\x90VPY\xba\xe9\x91\x86R\x97\xf7\xc3\xf9\xf5\xa6/\xebNp\x92\x1c\xa0\xdd-\xdd\x9b6 \x96\xd0\x17v\xd9\x8d\xcea\x93\xb8C\xe9\x7f\x0e\xaa\x93\x1d\xac\xb1w\xf6\x92\x19\xcb\x0e\x80\xa8T\xa7b\xad\xfdI\x1b\x0fe\x07\xa7?\xb1j,8\x9bGH@\xc3\x0e>\xe6v+\xa5\xb0=\xc4yV\xe5\xfd\xe8\xb9\xfd\xa4\x93\xf3\'\x92\xa8\xaezp\xc9\x08\xcefd\xeb\x9dO8\xe8\xa6i\xea\x87\x19\x98\xdc\xc1\xf7\x9e\x01L\xc2\x07c\x9f\t\xc2\x96f\x18\xd0\xef8\xcf\xdbC\x91w\xfd\xb4\xfd\x8d`,\xd9\xea\\$\xec\n\xe1\xb6\xefwA\x1fM=\xcd\xe6\x10\xd9\x98\x8c\xe7\x9b\x94\x8fz\xb1Cb\x8d\xc7l\xabee')
	