""" fastcgi connection handler, doesnt support multiplexing over a single connection, because most web servers dont either """

import  struct, cStringIO
from __init__ import ThreadedServer, Request, Response, read_checked


FCGI_LISTENSOCK_FILENO = 0
FCGI_HEADER_LEN = 8
FCGI_VERSION_1 = 1

FCGI_BEGIN_REQUEST = 1
FCGI_ABORT_REQUEST = 2
FCGI_END_REQUEST = 3
FCGI_PARAMS = 4
FCGI_STDIN = 5
FCGI_STDOUT = 6
FCGI_STDERR = 7
FCGI_DATA = 8
FCGI_GET_VALUES = 9
FCGI_GET_VALUES_RESULT = 10
FCGI_UNKNOWN_TYPE = 11
FCGI_MAXTYPE = FCGI_UNKNOWN_TYPE

FCGI_NULL_REQUEST_ID = 0
FCGI_KEEP_CONN = 1

FCGI_RESPONDER = 1
FCGI_AUTHORIZER = 2
FCGI_FILTER = 3

FCGI_REQUEST_COMPLETE = 0
FCGI_CANT_MPX_CONN = 1
FCGI_OVERLOADED = 2
FCGI_UNKNOWN_ROLE = 3

FCGI_MAX_CONNS = 'FCGI_MAX_CONNS'
FCGI_MAX_REQS = 'FCGI_MAX_REQS'
FCGI_MPXS_CONNS = 'FCGI_MPXS_CONNS'


RECORD_HEADER_STRUCT = struct.Struct('!BBHHBx')
END_REQUEST_STRUCT = struct.Struct('!IBxxx')
BEGIN_REQUEST_STRUCT = struct.Struct('!HBxxxxx')
UNKNOWN_TYPE_STRUCT = struct.Struct('!Bxxxxxxx')


def encode_record_header(type, request_id, content_len):
	return RECORD_HEADER_STRUCT.pack(FCGI_VERSION_1, type, request_id, content_len, 0)


def encode_record(type, request_id, content):
	return encode_record_header(type, request_id, len(content)) + content


def decode_nvp(input):
	
	def read_len():
		a = ord(read_checked(input))
		if a>>7:
			return (a&0x7f)<<24 | ord(read_checked(input))<<16 | ord(read_checked(input))<<8 | ord(read_checked(input))
		return a
	
	dict = {}
	while True:
		try:
			name_len = read_len()
			value_len = read_len()
			name = read_checked(input, name_len)
			value = read_checked(input, value_len)
		except EOFError:
			break
		dict[name] = value
	return dict


def encode_nvp(dict, output):
	
	def write_len(x):
		if x>>7:
			if x>>32:
				raise ValueError
			output.write(chr(x>>24 | 1<<7))
			output.write(chr(x>>16 & 0xff))
			output.write(chr(x>>8 & 0xff))
			output.write(chr(x & 0xff))
		else:
			output.write(chr(x))
	
	for k, v in dict.items():
		write_len(len(k))
		write_len(len(v))
		output.write(k)
		output.write(v)


class FastCGIException(Exception):
	""" thrown when the FastCGI protocol has been broken """


class Record:
	def __init__(self, file):
		s = read_checked(file, RECORD_HEADER_STRUCT.size)
		self.version, self.type, self.request_id, content_len, padding_len = RECORD_HEADER_STRUCT.unpack(s)
		if self.version != FCGI_VERSION_1:
			raise FastCGIException('unsupported version: %d' % self.version)
		self.content = read_checked(file, content_len)
		self.padding = read_checked(file, padding_len)


class RequestPart:
	def __init__(self, id, role, flags):
		self.id = id
		self.role = role
		self.flags = flags
		self.params = cStringIO.StringIO()
		self.stdin = cStringIO.StringIO()
	def keep_conn(self):
		return bool(self.flags & FCGI_KEEP_CONN)


class ConnectionHandler:
	def __init__(self, request_handler, session_handler):
		self.request_handler = request_handler
		self.session_handler = session_handler
		self.name = 'fastcgi'
	
	def handle(self, file):
		rp = None
		while True:
			rec = Record(file)
			
			if rec.request_id == 0:		# management record
				
				if rec.type == FCGI_GET_VALUES:
					nvs = decode_nvp(rec.content)
					ret = {}
					if FCGI_MAX_CONNS in nvs:
						ret[FCGI_MAX_CONNS] = '10'
					if FCGI_MAX_REQS in nvs:
						ret[FCGI_MAX_REQS] = '10'
					if FCGI_MPXS_CONNS in nvs:
						ret[FCGI_MPXS_CONNS] = '0'
					file.write(encode_record(FCGI_GET_VALUES_RESULT, 0, encode_nvp(ret)))
				else:
					file.write(encode_record(FCGI_UNKNOWN_TYPE, 0, UNKNOWN_TYPE_STRUCT.pack(rec.type )))
			
			else:						# application record
			
				if rec.type == FCGI_BEGIN_REQUEST:
					if len(rec.content) != 8:
						raise FastCGIException
					role, flags = BEGIN_REQUEST_STRUCT.unpack(rec.content)
					new_rp = RequestPart(rec.request_id, role, flags)
					if rp != None:
						file.write(encode_record(FCGI_END_REQUEST, new_rp.id, END_REQUEST_STRUCT.pack(0, FCGI_CANT_MPX_CONN)))
					elif new_rp.role != FCGI_RESPONDER:
						file.write(encode_record(FCGI_END_REQUEST, new_rp.id, END_REQUEST_STRUCT.pack(0, FCGI_UNKNOWN_ROLE)))
					else:
						rp = new_rp
				
				elif rec.type == FCGI_PARAMS:
					if rp != None and rec.request_id == rp.id:
						rp.params.write(rec.content)
				
				elif rec.type == FCGI_STDIN:
					if rp != None and rec.request_id == rp.id:
						if len(rec.content) == 0:
							headers = decode_nvp(cStringIO.StringIO(rp.params.getvalue()))						
							request = Request(headers, cStringIO.StringIO(rp.stdin.getvalue()), self.session_handler)
							response = self.request_handler(request)
							request.store_session()
							request.set_sid_cookie(response)
							
							#send in chunks
							stdout = cStringIO.StringIO()
							response.write(stdout, request)
							stdout.seek(0)
							chunk = stdout.read(65535)
							while chunk:
								file.write(encode_record(FCGI_STDOUT, rp.id, chunk))
								chunk = stdout.read(65535)	
							file.write(encode_record(FCGI_STDOUT, rp.id, ''))
							file.write(encode_record(FCGI_END_REQUEST, rp.id, END_REQUEST_STRUCT.pack(0, FCGI_REQUEST_COMPLETE)))
							
							if not rp.keep_conn():
								break
							else:
								rp = None
						else:
							rp.stdin.write(rec.content)
				
				elif rec.type == FCGI_DATA:
					pass
				
				elif rec.type == FCGI_ABORT_REQUEST:
					if rp != None and rec.request_id == rp.id:
						file.write(encode_record(FCGI_END_REQUEST, rp.id, END_REQUEST_STRUCT.pack(0, FCGI_REQUEST_COMPLETE)))
						rp = None
				
				else:
					raise FastCGIException


def listen(addr, request_handler, session_handler):
	ThreadedServer(addr, ConnectionHandler(request_handler, session_handler))