""" a session handler needs load(id) and store(id, data-dict) methods """

from __future__ import with_statement

import copy, random, time, threading, sqlite3, cPickle, re


DEFAULT_MAX_LIFETIME = 60*60*2
ID_LENGTH = 16
ID_REGEX = re.compile(r'^[-_a-zA-Z0-9]{%d}$' % ID_LENGTH)


def generate_id():
	return ''.join([random.choice('-_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789') for i in xrange(ID_LENGTH)])


def get_valid_id(input):
	return input if (isinstance(input, basestring) and ID_REGEX.match(input)) else generate_id()


class MemoryHandler:
	""" a session handler storing sessions in process memory - fast, but they disappear when server is restarted """

	def __init__(self, max_lifetime = DEFAULT_MAX_LIFETIME):
		self.lock = threading.Lock()
		self.data = {}	#id => (last-save-time, data-dict)
		self.max_lifetime = max_lifetime
		self.last_cleanup = 0

	def load(self, id):
		with self.lock:
		
			#delete expired sessions?
			if (time.time() - self.last_cleanup) > self.max_lifetime:
				self.last_cleanup = time.time()
				exp = []
				for k in self.data:
					if (time.time() - self.data[k][0]) > self.max_lifetime:
						exp.append(k)
				for k in exp:
					del self.data[k]
		
			try:
				s = self.data[id]
				if (time.time() - s[0]) <= self.max_lifetime:
					return copy.copy(s[1])
				else:
					return {}
			except KeyError:
				return {}
	
	def store(self, id, data):
		with self.lock:
			self.data[id] = (time.time(), data)


class SQLiteHandler:
	""" a session handler storing sessions in an SQLite database, slower than in memory sessions """

	def __init__(self, file, max_lifetime = DEFAULT_MAX_LIFETIME):
		self.file = file
		self.max_lifetime = max_lifetime
		self.last_cleanup = 0
		self.local = threading.local()
		self.cursor().execute("CREATE TABLE IF NOT EXISTS sessions (id TEXT NOT NULL PRIMARY KEY, time INTEGER NOT NULL, data BLOB NOT NULL)")
		self.cursor().execute("VACUUM")
	
	def cursor(self):
		try:
			return self.local.cursor
		except AttributeError:
			db = sqlite3.connect(self.file)
			db.isolation_level = None
			self.local.cursor = db.cursor()
			return self.local.cursor
		
	def load(self, id):
		cur = self.cursor()
		
		#delete expired sessions?
		if (time.time() - self.last_cleanup) > self.max_lifetime:
			self.last_cleanup = time.time()
			cur.execute("DELETE FROM sessions WHERE time < ?", (time.time() - self.max_lifetime,))
		
		cur.execute("SELECT data FROM sessions WHERE id = ? AND time > ?", (id, time.time() - self.max_lifetime))
		row = cur.fetchone()
		return cPickle.loads(str(row[0])) if row else {}
	
	def store(self, id, data):
		self.cursor().execute("INSERT OR REPLACE INTO sessions (id, data, time) VALUES (?, ?, ?)", (id, buffer(cPickle.dumps(data, True)), time.time()))
