""" img2-specific general purpose stuff """

import urllib, urllib2, re, hashlib, time, os, os.path, random, traceback, tempfile, shutil, logging
import config, cookies, parser, util, sessions, templates, cgi2


BOARD_REGEX = re.compile(r'^(?P<board>\w+)/')
TIME_REGEX = re.compile(r'^(?P<number>\d+(\.\d+)?)( )?(?P<unit>second(s)?|minute(s)?|hour(s)?|day(s)?|week(s)?|month(s)?|year(s)?)$')
TIME_SECONDS = {'second': 1, 'minute': 60, 'hour': 60*60, 'day': 60*60*24, 'week': 60*60*24*7, 'month': 60*60*24*30, 'year': 60*60*24*365}


class Img2Exception(Exception):
	""" generic exception for img2 """

class NotFound(Exception):
	""" thrown in views when a standard 404 page should be displayed """

class Forbidden(Exception):
	""" thrown in views when a standard 503 page should be displayed """


class Post:
	""" helper object for when passing 3 separate args around is a pain """
	
	def __init__(self, board, thread_id, id):
		self.board = board
		self.thread_id = thread_id
		self.id = id


class View:
	def __init__(self, regex, handler, extra_dict = {}, check_age = True):
		self.regex = re.compile(regex)
		self.handler = handler
		self.extra_dict = extra_dict
		self.check_age = check_age


class Handler:
	""" called with a cgi2.Request - expected to return an cgi2.Response """
	
	def __init__(self, general_views, board_views):
		self.general_views = general_views
		self.board_views = board_views

	def __call__(self, request):
		""" rename the lower __call__ to __call__2 to enable profiling """
		import hotshot, hotshot.stats
		p = hotshot.Profile('profiler.log')
		response = p.runcall(self.__call__2, request)
		p.close()
		stats = hotshot.stats.load('profiler.log')
		stats.strip_dirs()
		stats.sort_stats('time', 'calls')
		stats.print_stats(20)
		return response

	def __call__(self, request):
		from cgi2 import Response
		
		path = request['PATH_INFO'][1:]
		use_view = args = None
		
		#first check if non-board-specific url
		for view in self.general_views:
			mo = view.regex.match(path)
			if mo is not None:
				d = mo.groupdict()
				d.update(view.extra_dict)
				use_view, args = view, d
				break
		#otherwise check if it is a board-specific url
		else:
			mo = BOARD_REGEX.match(path)
			if mo is not None:
				board = get_board(mo.group('board'))
				if board is not None:
					board.request(request)			# notify the board a request arrived for it
					path = path[len(mo.group()):]
					for view in self.board_views:
						mo = view.regex.match(path)
						if mo is not None:
							d = mo.groupdict()
							d.update(view.extra_dict)
							d['board'] = board
							use_view, args = view, d
							break
		
		if use_view is not None:
			
			if use_view.check_age and config.SHOW_AGE_CHECK_PAGE and not cookies.check_overage(request):
				return render_to_response(request, 'age_check.html')
		
			try:
				return use_view.handler(request, **args)
			except NotFound:
				return Response(render_to_string(request, '404.html'), '404 Not Found')
			except Forbidden:
				return Response(render_to_string(request, '403.html'), '403 Forbidden')
			except:
				logging.exception('exception in request handler')
				return Response(render_to_string(request, '500.html'), '500 Server Error')
		else:
			return Response(render_to_string(request, '404.html'), '404 Not Found')


def is_mod(name, password):
	return (name in config.MODS) and (util.md5(config.UNIQUE_STRING + name + password) == config.MODS[name])


def get_mod_name(request):
	""" returns the name of the logged in mod, or '' """
	return sessions.get_mod_login(request)[0]


def add_global_template_vars(request, dict):
	""" adds variables that should be accessible on every page """
	dict['mod_name'] = get_mod_name(request)
	dict['mod_logout_confirm'] = sessions.get_mod_logout_confirm(request)
	dict['timer'] = util.Timer(request.time)


def render_to_response(request, template, dict = {}):
	from cgi2 import Response
	return Response(render_to_string(request, template, dict))


def render_to_string(request, template, dict = {}):
	add_global_template_vars(request, dict)
	return templates.render(template, dict)


def get_board(dir):
	for board in config.BOARDS:
		if board.DIR == dir:
			return board
	return None


def format_time(time, digits = 0):
	""" formats a time into a readable yet inaccurate format """
	d = '%%.%df ' % digits
	if time < 60:
		return (d+'second') % time + ('s' if time != 1 else '')
	time = time/60.0	#now in minutes
	if time < 60:
		return (d+'minute') % time + ('s' if time != 1 else '')
	time = time/60.0	#now in hours
	if time < 24:
		return (d+'hour') % time + ('s' if time != 1 else '')
	time = time/24.0	#now in days
	if time < 30:
		return (d+'day') % time + ('s' if time != 1 else '')
	day_time = time
	time = time/30.0	#now in months
	if time < 12:
		return (d+'month') % time + ('s' if time != 1 else '')
	time = day_time/365.0	#now in years
	return (d+'year') % time + ('s' if time != 1 else '')


def parse_time(time):
	""" parses a time in the format returned by format_time, returns time in int seconds or None with invalid input """
	mo = TIME_REGEX.match(time)
	if mo is None:
		return None
	else:
		dict = mo.groupdict()
		number = float(dict['number'])
		unit = dict['unit']
		if unit[-1] == 's':
			unit = unit[:-1]
		return int(number * TIME_SECONDS[unit])
