"""
img2 templates are translated into python code. these are the tag types.

output tags:
	{{ expr }} -> output += escape(expr)
	{$ expr $} -> output += expr (expr must be a string)

expression tags:
	{%-or-: exp %-or-:} : instead of % at start causes an indent reversal, at end it causes an indent on next line
	{:} -> tags containing only x ':'s cause x indent reversals
	example: {% for x in y :}
	example: {% x += 2 %}
	example: {: elif t == z :}
	example: {: x = x*4 %}

include another template:
	{" path "}

comment tags:
	{# comment #}

spaces between tag start and end bits and middle bits are optional. eg. {{poop}} and {{       poop }} are equivalent (except they arent, due to bug),
only comment tags can contain newlines

the global namespace for exp is this module

the local namespace is the dict argument to the render method, with the addition of 
a special __app function to append strings to the output, __out - the actual output list and __dict containing local variables
included templates get a shallow copy of the parents local variables as their own

"""

from __future__ import with_statement

# stuff imported here is accessible as globals in templates
import re, os.path, random, copy, threading, StringIO
import config, common, uploaded_files, util, parser, json, cgi2
from util import file_get_contents


tags = (
	r'\{\s(?P<esc_output>.+?)\s\}',
	r'\$\s(?P<output>.+?)\s\$',
	r'\s*(?P<rev_ind>:+)\s*',
	r'(?P<exp_start>%|:)\s(?P<exp>.+?)\s(?P<exp_end>%|:)',
	r'"\s*(?P<inc_path>.+?)\s*"'
)
regex = re.compile(r'\{(' + '|'.join(tags) + ')\}')
comment_regex = re.compile(r'\{#(?P<comment>.*?)#\}', re.DOTALL)
whitespace_regex = re.compile(r'(\s)\s+')

lock = threading.Lock()
template_cache = {}	#relative path => (file_mod_time, Template)


def render(path, dict):
	purge_modified()
	out = []
	render_sub(path, dict, out)
	return ''.join(out)


def render_sub(path, dict, out):
	t = load(path)
	try:
		t.render(dict, out)
	except Exception, e:
		raise TemplateException('exception in %s: %s %s' % (path, type(e).__name__, e))


def load(path):
	with lock:
		if path not in template_cache:
			template_cache[path] = load_from_file(path)
		return template_cache[path][1]


def load_from_file(path):
	full_path = os.path.join(config.TEMPLATE_DIR, path)
	try:
		mod = os.path.getmtime(full_path)
		tpl = Template(unicode(file_get_contents(full_path), 'utf-8', 'replace'))
		return (mod, tpl)
	except Exception, e:
		raise TemplateException('error loading template %s - %s' % (path, e))


def purge_modified():
	""" remove all modified templates from the cache. they will be loaded upon next use """
	with lock:
		for path, tpl in template_cache.items():
			full_path = os.path.join(config.TEMPLATE_DIR, path)
			if os.path.getmtime(full_path) != tpl[0]:
				del template_cache[path]


def reduce_whitespace(input):
	return whitespace_regex.sub(r'\1', input)


def strip_comments(input):
	return comment_regex.sub('', input)


class TemplateException(Exception):
	pass


class Template:

	def __init__(self, string):
	
		string = strip_comments(string)
		if config.REDUCE_TEMPLATE_WHITESPACE:
			string = reduce_whitespace(string)
		code = ''
		line_count = [0]	# indent => count
		prev_end = 0
		
		for m in regex.finditer(string):
			#got another literal before this match?
			if m.start() > prev_end:
				code += '\t'*(len(line_count)-1) + '__app(' + repr(string[prev_end:m.start()]) + ')\n'
				line_count[-1] += 1
			
			d = m.groupdict()
			
			if d['inc_path']:
				code += '\t'*(len(line_count)-1) + 'render_sub("' + d['inc_path'] + '", __dict, __out)\n'
				line_count[-1] += 1
			
			if d['exp']:
				s = d['exp_start']
				e = d['exp_end']
				if s == ':':
					if len(line_count) == 1:
						raise TemplateException('reverse-indented too far')
					if not line_count.pop():
						code += '\t'*len(line_count) + 'pass\n'
				code += '\t'*(len(line_count)-1) + d['exp']
				line_count[-1] += 1
				if e == ':':
					code += ':'
					line_count += [0]
				code += '\n'
		
			elif d['rev_ind']:
				for c in d['rev_ind']:
					if len(line_count) == 1:
						raise TemplateException('reverse-indented too far')
					if not line_count.pop():
						code += '\t'*len(line_count) + 'pass\n'
			
			elif d['esc_output']:
				exp = d['esc_output']
				code += '\t'*(len(line_count)-1) + '__app(cgi2.escape(' + exp + '))\n'
				line_count[-1] += 1
			
			elif d['output']:
				exp = d['output']
				code += '\t'*(len(line_count)-1) + '__app(' + exp + ')\n'
				line_count[-1] += 1
			
			prev_end = m.end()
		#check for trailing literal
		if prev_end < len(string):
			line_count[-1] += 1
			code += '\t'*(len(line_count)-1) + '__app(' + repr(string[prev_end:]) + ')'
		
		try:
			self.code = compile(code, '<string>', 'exec')
		except Exception, e:
			raise TemplateException('invalid template syntax: %s' % e)

	def render(self, dict, out):
		dict = copy.copy(dict)
		dict['__out'] = out
		dict['__dict'] = dict
		dict['__app'] = out.append
		exec self.code in globals(), dict