""" stuff to do with validating, saving, deleting etc uploaded files """

from __future__ import with_statement

import os, shutil, re, urllib, socket, threading, time, logging
import config, common, util, cgi2, bencode


SCRAPE_REGEX = re.compile('(?<=/)announce(?=[^/]*$)')

scrape_cache = {}	#info_hash => (time_fetched, TorrentScrape instance or None for fetch failure)
scrape_lock = threading.Lock()


class UploadedFile:
	""" subclasses of this are uploaded file meta data classes """
	
	def __init__(self, name):
		self.name = name
	
	def get_dir(self, post):
		return os.path.join(post.board.get_file_dir(), '%d/%d' % (post.thread_id, post.id))
	
	def get_path(self, post):
		return os.path.join(self.get_dir(post), self.name)
	
	def check_dirs(self, post):
		try:
			os.makedirs(self.get_dir(post))
		except:
			pass

	def get_url(self, board, thread_id, post_id):
		return '%s%s/%d/%d/%s' % (config.UPLOADED_FILE_URL, board.DIR, thread_id, post_id, self.name)
	
	def save(self, temp_file, post):
		self.check_dirs(post)
		shutil.copyfile(temp_file.path, self.get_path(post))
	
	def copy(self, src_post, dest_post):
		self.check_dirs(dest_post)
		shutil.copyfile(self.get_path(src_post), self.get_path(dest_post))
	
	def delete(self, post):
		util.quiet_unlink(self.get_path(post))


class ImageFile(UploadedFile):
	""" uploaded jpg/gif/png meta data """
	
	def __init__(self, name, size, thumb_size, type):
		UploadedFile.__init__(self, name)
		if name == 'thumb':
			raise ValueError('invalid name - "thumb" is not allowed')
		self.type = type
		self.size = size
		self.thumb_size = thumb_size

	def check_dirs(self, post):
		UploadedFile.check_dirs(self, post)
		try:
			os.makedirs(os.path.join(self.get_dir(post), 'thumb'))
		except:
			pass
	
	def get_thumb_path(self, post):
		return os.path.join(self.get_dir(post), 'thumb/' + self.name)
	
	def get_thumb_url(self, board, thread_id, post_id):
		return '%s%s/%d/%d/thumb/%s' % (config.UPLOADED_FILE_URL, board.DIR, thread_id, post_id, self.name)
	
	def save(self, temp_file, post):
		UploadedFile.save(self, temp_file, post)
		if self.thumb_size:
			mts = post.board.MAX_IMAGE_THUMB_SIZE
			os.system('convert "%s" -thumbnail "%sx%s>" "%s"' % (temp_file.path, mts[0], mts[1], self.get_thumb_path(post)))

	def copy(self, src_post, dest_post):
		UploadedFile.copy(self, src_post, dest_post)
		if self.thumb_size:
			self.check_dirs(dest_post)
			shutil.copyfile(self.get_thumb_path(src_post), self.get_thumb_path(dest_post))

	def delete(self, post):
		UploadedFile.delete(self, post)
		if self.thumb_size:
			util.quiet_unlink(self.get_thumb_path(post))


class SWFFile(UploadedFile):
	""" uploaded swf meta data """


class TorrentFileEntry:
	""" a file entry inside a torrent file """

	def __init__(self, name, length):
		self.name = name
		self.length = length


class TorrentScrape:
	""" data scraped from a tracker for a torrent """

	def __init__(self, seeders, leechers, downloads):
		self.seeders = seeders
		self.leechers = leechers
		self.downloads = downloads


class TorrentFile(UploadedFile):
	""" uploaded .torrent meta data """
	
	def __init__(self, name, board, file_name, input):
		UploadedFile.__init__(self, name)
		
		self.info_hash = None
		self.title = util.clamp_str(file_name, board.MAX_TORRENT_TITLE_LENGTH, '...')
		self.trackers = []
		self.files = []
		self.comment = None
		self.created_by = None
		self.creation_time = None
		
		def get_path(dict):
			if 'path' in dict and isinstance(dict['path'], list):
				path = '/'.join([unicode(e, 'utf-8', 'replace') for e in dict['path'] if isinstance(e, str)])
				return util.clamp_str(path, board.MAX_TORRENT_FILE_NAME_LENGTH, '...')
			else:
				return None
		
		def get_string(dict, key, max_length):
			if key in dict and isinstance(dict[key], str):
				return util.clamp_str(unicode(dict[key], 'utf-8', 'replace'), max_length, '...')
			else:
				return None
		
		def format(s):
			return cgi2.nl_to_br(cgi2.strip_html(s)) if s else None
		
		d = bencode.decode(input)
		if not isinstance(d, dict):
			raise ValueError('torrent file doesnt contain a dict')
		
		if 'info' in d and isinstance(d['info'], dict):
			info = d['info']
			if 'files' not in info:		#single file only
				name = get_string(info, 'name', board.MAX_TORRENT_FILE_NAME_LENGTH)
				length = util.psint(info.get('length'))
				if name and length:
					self.files = [TorrentFileEntry(name, length)]
				else:
					raise ValueError('torrent file doesnt contain any files')
			else:						#multiple files
				files = info['files']
				if isinstance(files, list):
					for file in files:
						name = get_path(file)
						length = util.psint(file.get('length'))
						if name and length:
							self.files.append(TorrentFileEntry(name, length))
					self.files = self.files[:board.MAX_TORRENT_FILES]
		if not self.files:
			raise ValueError('torrent file doesnt contain any files')
		
		if 'announce-list' in d:
			al = d['announce-list']
			if isinstance(al, list) and len(al) and isinstance(al[0], list):
				self.trackers = [util.clamp_str(unicode(t, 'utf-8', 'replace'), board.MAX_TORRENT_TRACKER_LENGTH, '...') for t in al[0] if isinstance(t, str)]
		elif 'announce' in d and isinstance(d['announce'], str):
			self.trackers = [get_string(d, 'announce', board.MAX_TORRENT_TRACKER_LENGTH)]
		self.trackers = self.trackers[:board.MAX_TORRENT_TRACKERS]
		if not self.trackers:
			raise ValueError('torrent file doesnt contain any trackers')
		
		self.comment = format(get_string(d, 'comment', board.MAX_TORRENT_COMMENT_LENGTH))
		self.created_by = format(get_string(d, 'created by', board.MAX_TORRENT_CREATED_BY_LENGTH))
		
		if 'creation date' in d and isinstance(d['creation date'], int):
			t = d['creation date']
			self.creation_time = t if util.valid_time(t) else None
		
		self.info_hash = util.sha1(bencode.encode(d['info']), hex = False)
	
	def get_scrape_url(self):
		for url in self.trackers:
			url, n = SCRAPE_REGEX.subn('scrape', url)
			if n and url.startswith('http://'):
				return url
		return None
	
	def scrape_data(self):
		
		with scrape_lock:
			#clear out-of-date entries
			for key in scrape_cache.keys():
				if scrape_cache[key][0] < (time.time() - config.SECONDS_BETWEEN_TORRENT_SCRAPES):
					del scrape_cache[key]
			#return from cache if entry still alive
			if self.info_hash in scrape_cache:
				return scrape_cache[self.info_hash][1]
		
		socket.setdefaulttimeout(1)	#bring on 2.6
		url = self.get_scrape_url()
		if url:
			url = '%s?info_hash=%s' % (url, urllib.quote(self.info_hash))
			try:
				obj = bencode.decode(urllib.urlopen(url).read(64*1024))	#dont read over 64kb to help stop maliciousness
				file = obj['files'][self.info_hash]
				seeders = util.psint(file.get('complete'))
				leechers = util.psint(file.get('incomplete'))
				downloads = util.psint(file.get('downloaded'))
				scraped = TorrentScrape(seeders, leechers, downloads)
			except:
				scraped = None
			
			#store fetched data/None
			with scrape_lock:
				scrape_cache[self.info_hash] = (time.time(), scraped)
				return scraped
		else:
			return None


def calc_thumb_size(board, size):
	w, h = size
	mw, mh = board.MAX_IMAGE_THUMB_SIZE
	if w > mw:	#fit to width
		h = h * (mw/float(w))
		w = mw
	if h > mh:	#fit to height
		w = w * (mh/float(h))
		h = mh
	return w, h


def check(file, board, boardDB, general_db):
	""" returns (file_md5, file_meta, temp_file) """
	
	from common import Img2Exception

	temp = util.TempFile()
	md5 = util.copy_file(temp.path, file.file)
	name = util.sanitize_file_name(file.name)

	# non type specific checks
	
	if not name:
		raise Img2Exception('Empty file name.')
	
	if general_db.file_banned(md5):
		raise Img2Exception('This file is banned.')
	
	if (not board.ALLOW_DUPLICATE_FILES) and boardDB.file_exists(md5):
		raise Img2Exception('This file exists in another post - duplicates are not allowed.')
	
	file_len = os.path.getsize(temp.path)
	if file_len == 0:
		raise Img2Exception('Zero length file not allowed.')
	if file_len > board.MAX_FILE_LENGTH:
		raise Img2Exception('File too large: maximum allowed size is %dkb.' % (board.MAX_FILE_LENGTH/1024))
	
	# type specific parsing/checking
	
	if file.name.endswith('.swf') and ('swf' in board.VALID_FILE_TYPES):
		return md5, SWFFile(name), temp
	
	elif file.name.endswith('.torrent') and ('torrent' in board.VALID_FILE_TYPES):
		try:
			return md5, TorrentFile(name, board, file.name, util.file_get_contents(temp.path)), temp
		except:
			raise Img2Exception('error reading torrent file.')
	
	else:
		try:
			f = os.popen('identify "%s"' % temp.path, 'r')
			res = f.read();
			f.close()
			rs = res.split(' ')
			if res.startswith('identify:') or len(rs) < 3:
				raise Img2Exception('unrecognised file format.')
			type = rs[1].lower()
			size = rs[2].split('x')
			size = int(size[0]), int(size[1])
	
			if type not in board.VALID_FILE_TYPES:
				raise Img2Exception('Image type not allowed')
			if size[0] > board.MAX_IMAGE_WIDTH:
				raise Img2Exception('Image too wide, maximum allowed width is %dpx.' % board.MAX_IMAGE_WIDTH)
			if size[1] > board.MAX_IMAGE_HEIGHT:
				raise Img2Exception('Image too tall, maximum allowed height is %dpx.' % board.MAX_IMAGE_HEIGHT)
	
			if (size[0] >= board.MIN_IMAGE_SIZE_FOR_THUMBNAILING[0]) or (size[1] > board.MIN_IMAGE_SIZE_FOR_THUMBNAILING[1]):
				thumb_size = calc_thumb_size(board, size)
			else:
				thumb_size = None
	
			return md5, ImageFile(name, size, thumb_size, type), temp
		except Img2Exception:
			raise
		except:
			raise Img2Exception('error reading image file.')


def save(temp, board, thread_id, post_id, file_meta):
	try:
		file_meta.save(temp, common.Post(board, thread_id, post_id))
	except:
		logging.exception('error saving uploaded file')


def delete(board, thread_id, post_id, file_meta):
	try:
		file_meta.delete(common.Post(board, thread_id, post_id))
	except:
		logging.exception('error deleting uploaded file')