""" important stuff """

#IDEA: option for sorting threads differently: interleave sorted by started with sorted by updated
#IDEA: python 3000
#IDEA: web server integration for fullpage caching - eg. mod_magnet on lighttpd
#IDEA: make all code more readable/documented
#IDEA: have the index "/" page display threads/posts from all boards OR boards chosen by user - choices saved in cookie


from __future__ import with_statement

import time, os, os.path, random, logging, threading
import pagecache, cookies, uploaded_files, util, sessions, util.http as http, util.templates as templates, util.json as json, database


VERSION = '0.2.9'

_config_lock = threading.Lock()
_config = None	#current Config


class Img2Exception(Exception):
	""" generic exception for img2 """


class Post:
	""" helper object for when passing 3 separate args around is a pain. instances are hashable. """
	
	def __init__(self, board, thread_id, id):
		self.board = board
		self.thread_id = thread_id
		self.id = id
	
	def __hash__(self):
		return hash((self.board.DIR, self.thread_id, self.id))
	
	def __cmp__(self, other):
		return cmp((self.board.DIR, self.thread_id, self.id), other)


class Config:
	""" represents a loaded config file. immutable. """
	
	def __init__(self, path):
		
		# load from path
		self._filemtime = os.path.getmtime(path)
		
		globs = {'Board': Board, 'ModPostAction': ModPostAction, 'ModThreadAction': ModThreadAction, 'ObjDict': util.ObjDict}
		locs = {}
		exec open(path, 'rb') in globs, locs
		self.__dict__.update(locs)
		for board in self.BOARDS:
			board.config = self
			for bd in self.BOARD_DEFAULTS:
				board.__dict__.setdefault(bd, self.BOARD_DEFAULTS[bd])
		
		if not self.UNIQUE_STRING:
			raise Img2Exception('you need to set the UNIQUE_STRING')
	
		if not self.BOARDS:
			raise Img2Exception('you need to define at least one board')

	def get_board(self, dir):
		for board in self.BOARDS:
			if board.DIR == dir:
				return board
		return None
	
	def is_mod(self, name, password):
		return (name in self.MODS) and (util.sha1(self.UNIQUE_STRING + name + password) == self.MODS[name])


class Board:
	
	def __init__(self, **d):
		self.__dict__.update(d)
	
	def get_default_name(self):
		f = self.DEFAULT_FIRST_NAMES
		s = self.DEFAULT_SECOND_NAMES
		if f and s:
			return random.choice(f) + ' ' + random.choice(s)
		elif f:
			return random.choice(f)
		elif s:
			random.choice(s)
		else:
			return ''

	def get_mod_post_action(self, name):
		for action in self.MOD_POST_ACTIONS:
			if action.name == name:
				return action
		return None

	def get_mod_thread_action(self, name):
		for action in self.MOD_THREAD_ACTIONS:
			if action.name == name:
				return action
		return None
	
	def db(self):
		return database.BoardDataBase(self)
	
	def get_file_dir(self):
		return os.path.join(self.config.UPLOADED_FILE_DIR, self.DIR)
	
	def __ne__(self, other):
		return not self.__eq__(other)
	
	def __eq__(self, other):
		return isinstance(other, Board) and self.DIR == other.DIR
	
	def __cmp__(self, other):
		if isinstance(other, Board):
			return cmp(self.DIR, other.DIR)
		return -1
	
	def __hash__(self):
		return hash(self.DIR)

	def __str__(self):
		return self.DIR


class ModAction:
	""" represents a named action a mod can take quickly without going to any separate control panel """
	def __init__(self, name):
		""" name is what is displayed next on the link. it must match the regex "[-_a-zA-Z0-9]+" """
		self.name = name


class ModPostAction(ModAction):
	def __init__(self, name, delete_text = False, delete_file = False, ban = None, ban_file = False, mark_reviewed = False):
		""" ban is either a tuple (seconds-long, reason) or None """
		ModAction.__init__(self, name)
		self.delete_text = delete_text
		self.delete_file = delete_file
		self.ban = ban
		self.ban_file = ban_file
		self.mark_reviewed = mark_reviewed


class ModThreadAction(ModAction):
	def __init__(self, name, delete = False, toggle_sticky = False, toggle_lock = False):
		ModAction.__init__(self, name)
		self.delete = delete
		self.toggle_sticky = toggle_sticky
		self.toggle_lock = toggle_lock


def check_age(view):
	""" decorator for view functions to enforce the age check. use only on views with a board argument. """
	def new_view(request, **kwargs):
		board = kwargs['board']
		if board.SHOW_AGE_CHECK_PAGE and not cookies.get_overage(request):
			return render_to_response(request, 'age_check.html')
		else:
			return view(request, **kwargs)
	return new_view


def cache(view):
	""" decorator for view functions that returns a cached page under certain conditions. slightly magical. """
	def new_view(request, **kwargs):
		pagecache.clean(request.config)
		if sessions.get_mod(request) != None:
			return view(request, **kwargs)
		else:
			sk = kwargs.keys()
			sk.sort()
			key = '%s=%s' % (view.__name__, '-'.join([str(kwargs[k]) for k in sk]))
			page = pagecache.load(key, request.config)
			if page is None:
				response = view(request, **kwargs)
				if response.status == 200:
					pagecache.store(key, response.body, request.config)
				return response
			else:
				return http.Response(page)
	return new_view


def render_to_response(request, template, dict = {}):
	return http.Response(render_to_string(request, template, dict))


def render_to_string(request, path, locals = {}):
	# add global (every page) template vars
	locals.update({
		'mod_name': sessions.get_mod(request),
		'mod_logout_confirm': sessions.get_mod_logout_confirm(request),
		'timer': util.Timer(request.time)
	})
	globals = {'config': request.config, 'VERSION': VERSION, 'uploaded_files': uploaded_files, 'json': json, 'util': util, 'random': random}
	return templates.render(request.config.TEMPLATE_DIR, path, locals, globals)


def get_config():
	""" return a Config object, caching and reloaded as needed. """
	global _config
	with _config_lock:
		if _config is None or _config._filemtime != os.path.getmtime('config.py'):
			_config = Config('config.py')
			pagecache.purge(_config)
		return _config


def bootstrap():
	""" start img2 """

	import sys, os, logging, logging.handlers, signal

	#log to a file and stdout
	logger = logging.getLogger()
	logger.setLevel(logging.DEBUG)
	logger.addHandler(logging.StreamHandler())
	fh = logging.handlers.RotatingFileHandler('log', maxBytes = 10*1024*1024, backupCount = 3)
	fh.setFormatter(logging.Formatter('%(asctime)s %(name)s %(levelname)s %(message)s'))
	logger.addHandler(fh)
	
	if len(sys.argv) == 2:
		host = 'localhost'
		port = int(sys.argv[1])
	elif len(sys.argv) == 3:
		host = sys.argv[1]
		port = int(sys.argv[2])
	else:
		print 'you must supply arguments of the form: <port> OR <host> <port>'
		sys.exit()
	
	# test the config file ASAP - dont wait for a request
	try:
		get_config()
	except:
		logging.exception('error loading config file')
		sys.exit()
	
	import img2.util, img2.util.sessions as sessions, img2.views
	try:
		sessions.unpickle('databases/sessions.pickle')
		img2.util.http.HTTPServer((host, port), img2.views.Handler(), 1, 10)
		sessions.pickle('databases/sessions.pickle')
	except:
		logging.exception('error running server')
