"""
handles the saving/loading of cached pages.
cached pages are board index, thread pages and list pages
pages for mods are not cached.
"""

from __future__ import with_statement

import os, time, threading, glob
from os.path import join


MAX_MINUTES_OF_LIFE = 5			# max time to cache pages for - needs to be lowish for torrent scrapes and time difference display

_last_clean = 0
_lock = threading.Lock()


def _check_dir(config):
	try:
		os.makedirs(config.CACHE_DIR)
	except:
		pass


def purge(config):
	""" removes all files from the cache """
	with _lock:
		_check_dir(config)
		for file in os.listdir(config.CACHE_DIR):
			os.unlink(join(config.CACHE_DIR, file))


def clean(config):
	""" if MAX_MINUTES_OF_LIFE have elapsed since the last cleanup(), clear the cache of items older than MAX_MINUTES_OF_LIFE """
	global _last_clean
	with _lock:
		_check_dir(config)
		if (time.time() - _last_clean) > MAX_MINUTES_OF_LIFE*60:
			
			_last_clean = time.time()
			max_mtime = time.time() - MAX_MINUTES_OF_LIFE*60
			
			for file in os.listdir(config.CACHE_DIR):
				file = join(config.CACHE_DIR, file)
				if os.path.getmtime(file) < max_mtime:
					os.unlink(file)


def load(name, config):
	""" return a utf-8 encoded byte-string from the cache that has the specified name, or None if it doesnt exist """
	with _lock:
		_check_dir(config)
		try:
			with open(join(config.CACHE_DIR, name), 'rb') as f:
				return f.read()
		except:
			return None


def store(name, page, config):
	""" store page in the cache. page must be a utf-8 encoded byte-string """
	with _lock:
		_check_dir(config)
		with open(join(config.CACHE_DIR, name), 'wb') as f:
			f.write(page)


def thread_modified(board, thread_id, config):
	""" clears the cache of any items related to the specified thread """
	with _lock:
		_check_dir(config)
		for file in glob.glob(join(config.CACHE_DIR, 'view_thread=%s-%d' % (board, thread_id))) + \
					glob.glob(join(config.CACHE_DIR, 'view_board=%s-*' % board)) + \
					glob.glob(join(config.CACHE_DIR, 'list=%s' % board)):
			os.unlink(file)
