"""
simple python-programmer-centric template system where templates are translated into python code.

these are the tag types.

output tags:
	{{ expr }} -> output += escape(expr) (expr must be a string, int, float or bool)
	{$ expr $} -> output += expr (expr must be a string)

expression tags:
	{E exp E} -> where E = "%" or ":". ":" at start causes an indent reversal, at end it causes an indent on next line. "%" doesnt cause any indent change
	{:} -> tags containing only x ":"s cause x indent reversals
	example: {% for x in y :}
	example: {% x += 2 %}
	example: {: elif t == z :}
	example: {: x = x*4 %}

include another template:
	{" path "}

comment tags:
	{# comment #}

spaces between tag start and end bits and middle bits are optional. eg. {{poop}} and {{       poop }} are equivalent (except they arent, due to bug),
only comment tags can contain newlines

the global namespace the globals argument to TemplateDir

the local namespace is the dict argument to the render method.
the following local names are reserved: __app, __app_escaped, __out, __locals, __render_included

included templates get a shallow copy of the parents local variables as their own

example usage:

t = templates.TemplateDir('templates')
print t.render('test.html', {'msg': 'hello'})

this module has a single external api function; render(dir, path, locals = {}, globals = {})

"""

from __future__ import with_statement

import re, os.path, copy, threading, StringIO
import util


_TAG_REGEX = (
	r'\{\s(?P<esc_output>.+?)\s\}',
	r'\$\s(?P<output>.+?)\s\$',
	r'\s*(?P<rev_ind>:+)\s*',
	r'(?P<exp_start>%|:)\s(?P<exp>.+?)\s(?P<exp_end>%|:)',
	r'"\s*(?P<inc_path>.+?)\s*"'
)
_TAG_REGEX = re.compile(r'\{(' + '|'.join(_TAG_REGEX) + ')\}')
_COMMENT_REGEX = re.compile(r'\{#(?P<comment>.*?)#\}', re.DOTALL)
_WHITESPACE_REGEX = re.compile(r'(\s)\s+')


class TemplateException(Exception):
	pass


_lock = threading.Lock()
_cache = {}	#path => (file_mod_time, Template)


def render(dir, path, locals = {}, globals = {}):
	"""
		return a string rendered from the template loaded from the file at os.path.join(dir, path).
		use dir as the base file path for any included templates to be loaded from.
		use local and global dicts for the template rendering.
		throw TemplateException upon error.
	"""
	_purge_modified()
	out = []
	_render_to_list(dir, path, locals, globals, out)
	return ''.join(out)


def _render_to_list(dir, path, locals, globals, out):
	tpl = _load(dir, path)
	try:
		tpl.render_to_list(locals, globals, out)
	except Exception, e:
		raise TemplateException('exception in %s: %s %s' % (path, type(e).__name__, e))


def _purge_modified():
	""" remove all modified templates from the cache. they will be loaded upon next use """
	with _lock:
		for path, tpl in _cache.items():
			if os.path.getmtime(path) != tpl[0]:
				del _cache[path]


def _load(dir, path):
	with _lock:
		full_path = os.path.join(dir, path)
		if full_path not in _cache:
			try:
				mod = os.path.getmtime(full_path)
				tpl = _Template(dir, util.file_get_contents(full_path).decode('utf-8', 'replace'))
				_cache[full_path] = (mod, tpl)
			except Exception, e:
				raise TemplateException('error loading template %s - %s' % (path, e))
		return _cache[full_path][1]


class _Template:

	def __init__(self, dir, string):
		self.dir = dir
		
		string = _COMMENT_REGEX.sub('', string)
		string = _WHITESPACE_REGEX.sub(r'\1', string)
		code = ''
		line_count = [0]	# indent => count
		prev_end = 0
		
		for m in util.re_msplit(_TAG_REGEX, string):
			
			if isinstance(m, basestring):
				code += '\t'*(len(line_count)-1) + '__app(' + repr(m) + ')\n'
				line_count[-1] += 1
			
			else:
				d = m.groupdict()
				
				if d['inc_path']:
					code += '\t'*(len(line_count)-1) + '__render_included("' + d['inc_path'] + '", __locals, __out)\n'
					line_count[-1] += 1
				
				if d['exp']:
					s = d['exp_start']
					e = d['exp_end']
					if s == ':':
						if len(line_count) == 1:
							raise TemplateException('reverse-indented too far')
						if not line_count.pop():
							code += '\t'*len(line_count) + 'pass\n'
					code += '\t'*(len(line_count)-1) + d['exp']
					line_count[-1] += 1
					if e == ':':
						code += ':'
						line_count += [0]
					code += '\n'
			
				elif d['rev_ind']:
					for c in d['rev_ind']:
						if len(line_count) == 1:
							raise TemplateException('reverse-indented too far')
						if not line_count.pop():
							code += '\t'*len(line_count) + 'pass\n'
				
				elif d['esc_output']:
					exp = d['esc_output']
					code += '\t'*(len(line_count)-1) + '__app_escaped(' + exp + ')\n'
					line_count[-1] += 1
				
				elif d['output']:
					exp = d['output']
					code += '\t'*(len(line_count)-1) + '__app(' + exp + ')\n'
					line_count[-1] += 1
		
		try:
			self.code = compile(code, '<string>', 'exec')
		except Exception, e:
			raise TemplateException('invalid template syntax: %s' % e)

	def render_to_list(self, locals, globals, out):
		locals = copy.copy(locals)
		locals['__locals'] = locals
		locals['__out'] = out
		locals['__app'] = out.append
		locals['__app_escaped'] = lambda s: out.append(util.html_escape(s))
		locals['__render_included'] = lambda path, locals, out: _render_to_list(self.dir, path, locals, globals, out)
		exec self.code in globals, locals