""" every function in this module returns a specific page """


import logging
import common, parser, util, database, uploaded_files, pagecache, cookies, sessions, tripcodes, util.json as json, util.http as http
from util import get_limited, clamp_str, limit, sint, psint, valid_ip
from util.http import Request, Response, RedirectResponse, NotFound, Forbidden
from common import Img2Exception, render_to_response, render_to_string, check_age, cache
from database import GeneralDataBase


def boards_api(request):
	data = database.get_boards_api_data(request.config, request.server_name)
	return Response(json.encode(data), 'application/json; charset=utf-8')


def confirm_overage(request):
	response = RedirectResponse(request['referer'] or '/')
	cookies.set_overage(response)
	return response


def index(request):
	return RedirectResponse('/%s/' % request.config.BOARDS[0].DIR)


@check_age
@cache
def view_board(request, board, page):
	data = board.db().get_view_board_data(page)
	if data is None:
		raise NotFound
	return render_to_response(request, 'view_board.html', data)


@check_age
@cache
def view_thread(request, board, id):
	data = board.db().get_view_thread_data(id)
	if not data:
		raise NotFound
	return render_to_response(request, 'view_thread.html', data)


@check_age
@cache
def list(request, board):
	data = board.db().get_list_data()
	return render_to_response(request, 'list.html', data)


def fetch_post(request, board, thread_id, post_id):
	data = board.db().get_post_data(thread_id, post_id)
	if not data:
		raise NotFound
	response = render_to_response(request, 'includes/thread_reply.html', data)
	if sessions.get_mod(request) is None:
		response.expire(5)
	return response


@check_age
def new_thread(request, board):
	
	if request.method == 'GET':
		forked_thread_id = psint(request.get('t'))
		forked_post_id = psint(request.get('p'))
		return render_to_response(request, 'new_thread.html', {'board': board, 'new_thread': {'forked_thread_id': forked_thread_id, 'forked_post_id': forked_post_id}})
	
	elif request.method != 'POST' or not request.local_referer():
		raise Forbidden
	
	# gather input
	sent = dict(
		name = clamp_str(request.post('name', '').strip(), board.MAX_NAME_LENGTH),
		tripcode = request.post('tripcode', ''),
		subject = clamp_str(request.post('subject', '').strip(), board.MAX_SUBJECT_LENGTH),
		comment = request.post('comment', '').strip(),
		file = request.files('file'),
		forked_thread_id = request.post('forked_thread_id'),
		forked_post_id = request.post('forked_post_id'),
		noko = 'noko' in request.post,
		ip = request.remote_host,
		sticky = 'sticky' in request.post,
		lock = 'lock' in request.post,
		mod = 'mod' in request.post
	)
	
	def set_cookies(response):
		cookies.set_noko(response, sent['noko'])
		cookies.set_name(response, sent['name'])
		cookies.set_tripcode(response, sent['tripcode'])
	
	def error(msg):
		response = render_to_response(request, 'new_thread.html', {'board': board, 'error_msg': msg, 'new_thread': sent})
		set_cookies(response)
		return response
	
	db = board.db()
	general_db = GeneralDataBase(request.config)
	
	ban = general_db.get_ban(sent['ip'])
	if ban:
		response = render_to_response(request, 'new_thread.html', {'board': board, 'ban': ban, 'new_thread': sent})
		set_cookies(response)
		return response
	
	if not sent['comment'] and not (board.ALLOW_NO_COMMENT_WITH_IMAGE and sent['file']):
		return error('You must write something.')
	
	if len(sent['comment']) > board.MAX_COMMENT_LENGTH:
		return error('Comment too long, max allowed length is %d characters.' % board.MAX_COMMENT_LENGTH)
	
	mod = sessions.get_mod(request)
	
	#enforce board rules, but not for mods
	if mod is None:
		sent['sticky'] = sent['lock'] = sent['mod'] = False
	
		if sent['file'] is None and not board.ALLOW_THREAD_WITHOUT_FILE:
			return error('You must supply a file/image to start a thread.')
			
		if db.is_thread_flood(sent['ip']):
			return error('You are making threads too fast. Slow down!')
	
		if not board.ALLOW_NAMES:		sent['name'] == ''
		if not board.ALLOW_TRIPCODES:	sent['tripcode'] = ''

	#check forked post is valid - if set
	if board.FORKING and (sent['forked_thread_id'] is not None) and (sent['forked_post_id'] is not None):
		forked_thread_id = psint(sent['forked_thread_id'])
		forked_post_id = psint(sent['forked_post_id'])
		if forked_post_id == 0:
			return error('You can\'t fork the first post.')
		if not db.post_exists(forked_thread_id, forked_post_id):
			return error('You can\'t fork a non existent post.')
	else:
		forked_thread_id = forked_post_id = None
	
	if board.NOKO == 'never':		sent['noko'] = False
	elif board.NOKO == 'always':	sent['noko'] = True
	if not board.VALID_FILE_TYPES:	sent['file'] = None
	
	if sent['file']:
		try:
			file_md5, file_meta, temp_file = uploaded_files.check(sent['file'], board, db, general_db)
		except Img2Exception, e:
			return error(str(e))
	else:
		file_md5, file_meta = None, None
	
	name = sent['name'] or board.get_default_name()
	deleted_threads, thread_id = db.add_new_thread(sent['subject'], sent['sticky'], sent['lock'], forked_thread_id, forked_post_id, name, tripcodes.make(board, sent['tripcode']), sent['mod'], sent['comment'], file_md5, file_meta, sent['ip'])
	
	if file_md5:
		uploaded_files.save(temp_file, board, thread_id, 0, file_meta)
	
	db.anonymize_old_posts()
	pagecache.thread_modified(board, thread_id, request.config)
	if forked_thread_id != None:
		pagecache.thread_modified(board, forked_thread_id, request.config)
	for thread in deleted_threads:
		pagecache.thread_modified(board, thread, request.config)
	
	path = '/%s/' % board.DIR
	if sent['noko']:
		path += str(thread_id)
	response = RedirectResponse(path)
	set_cookies(response)
	cookies.set_post_verifier(response, board, thread_id, 0)
	return response


@check_age
def new_reply(request, board, thread_id):
	
	if request.method == 'GET':
		qid = request.get('q')
		comment = '' if (qid is None) else ('>>%d\r\n\r\n' % psint(qid))
		return render_to_response(request, 'new_reply.html', {'board': board, 'thread': {'id': thread_id}, 'new_post': {'comment': comment}})
	
	elif request.method != 'POST' or not request.local_referer():
		raise Forbidden
	
	# gather input
	sent = dict(
		name = clamp_str(request.post('name', '').strip(), board.MAX_NAME_LENGTH),
		tripcode = request.post('tripcode', ''),
		comment = request.post('comment', '').strip(),
		file = request.files('file'),
		noko = 'noko' in request.post,
		bump = 'bump' in request.post,
		lock = 'lock' in request.post,
		mod = 'mod' in request.post,
		ip = request.remote_host
	)
	
	general_db = GeneralDataBase(request.config)
	db = board.db()
	
	data = db.get_new_reply_data(thread_id)
	if data is None:
		raise NotFound
	thread = data['thread']

	def set_cookies(response):
		cookies.set_noko(response, sent['noko'])
		cookies.set_name(response, sent['name'])
		cookies.set_tripcode(response, sent['tripcode'])
	
	def error(msg):
		response = render_to_response(request, 'new_reply.html', {'board': board, 'error_msg': msg, 'thread': thread, 'new_post': sent})
		set_cookies(response)
		return response
	
	ban = general_db.get_ban(sent['ip'])
	if ban:
		response = render_to_response(request, 'new_reply.html', {'board': board, 'ban': ban, 'thread': thread, 'new_post': sent})
		set_cookies(response)
		return response
	
	if thread['next_id'] >= board.MAX_THREAD_POSTS:
		return error('This thread is full.')

	if not sent['comment'] and not (board.ALLOW_NO_COMMENT_WITH_IMAGE and sent['file']):
		return error('You must write something.')
	
	if len(sent['comment']) > board.MAX_COMMENT_LENGTH:
		return error('Comment too long, max allowed length is %s characters.' % board.MAX_COMMENT_LENGTH)
	
	mod = sessions.get_mod(request)
	
	# enforce board rules, but not for mods
	if mod is None:
		sent['lock'] = sent['mod'] = False
		
		if thread['locked']:
			return error('This thread is locked.')
	
		if sent['file'] is None and not board.ALLOW_REPLY_WITHOUT_FILE:
			return error('You must supply a file/image with a reply.')
		
		if db.is_post_flood(sent['ip']):
			return error('You are posting too fast. Relax!')
		
		if not board.ALLOW_NAMES:		sent['name'] == ''
		if not board.ALLOW_TRIPCODES:	sent['tripcode'] = ''		

	if board.NOKO == 'never':		sent['noko'] = False
	elif board.NOKO == 'always':	sent['noko'] = True
	if board.BUMP == 'never':		sent['bump'] = False
	elif board.BUMP == 'always':	sent['bump'] = True
	if not board.VALID_FILE_TYPES:	sent['file'] = None
	
	if sent['file']:
		try:
			file_md5, file_meta, temp_file = uploaded_files.check(sent['file'], board, db, general_db)
		except Img2Exception, e:
			return error(str(e))
	else:
		file_md5, file_meta = None, None
	
	name = sent['name'] or board.get_default_name()
	post_id = db.add_new_reply(thread_id, name, tripcodes.make(board, sent['tripcode']), sent['mod'], sent['comment'], file_md5, file_meta, sent['ip'], sent['bump'], sent['lock'])
	if post_id == None:
		return error('Error adding reply.')
	
	if file_md5:
		uploaded_files.save(temp_file, board, thread_id, post_id, file_meta)
	
	db.anonymize_old_posts()
	pagecache.thread_modified(board, thread_id, request.config)
	
	path = '/%s/' % board.DIR
	if sent['noko']:
		path += '%d#%d' % (thread_id, post_id)
	response = RedirectResponse(path)
	set_cookies(response)
	cookies.set_post_verifier(response, board, thread_id, post_id)
	return response


def mod_area(request, section):
	
	def set_mod_cookie(response):
		cookies.set_mod(response)
		return response
	
	if sessions.get_mod(request) is None:
		#may be logging in this request
		name = request.post('name')
		password = request.post('password')
		if request.config.is_mod(name, password):
			sessions.set_mod(request, name)
			if request.post('return'):
				return set_mod_cookie(RedirectResponse(util.clamp_to_line(request.post('referer', '/'))))
		else:
			return render_to_response(request, 'mod_login.html', {'section': section, 'referer': request['referer']})
	
	action = request.post('action')
	if not request.local_referer():
		action = None
	msg = None
	
	if section == 'overview':
		data = database.get_mod_area_overview_data(request.config)
		data['section'] = section
		data['msg'] = msg
		return set_mod_cookie(render_to_response(request, 'mod_overview.html', data))
	
	elif section == 'bans':
		
		if action:
			action = limit(action, 'add', 'delete')
			
			if action == 'add':
				import time
				now = int(time.time())
				ip = request.post('ip', '').strip()
				expires_parsed = util.parse_interval(request.post('expires', '').strip())
				reason = request.post('reason').strip()
				
				if expires_parsed is None:
					msg = 'Invalid expiry time'
				elif now + expires_parsed > (1<<31)-1:
					msg = 'Expiry time too far in future'
				elif not util.valid_ip(ip):
					msg = 'Invalid IP'
				else:
					GeneralDataBase(request.config).add_ban(ip, now + expires_parsed, reason)
					msg = 'Ban Added'
			
			elif action == 'delete':
				ip = request.post('ip', '')
				GeneralDataBase(request.config).delete_ban(ip)
				msg = 'Ban Deleted.'
		
		filter = request.get('filter', '')
		data = GeneralDataBase(request.config).get_mod_area_bans_data(filter)
		data['new_ban_ip'] = request.get('new_ban_ip', '')
		data['filter'] = filter
		data['section'] = section
		data['msg'] = msg
		return set_mod_cookie(render_to_response(request, 'mod_bans.html', data))
	
	elif section == 'files':
	
		if action:
			action = limit(action, 'ban', 'unban')
			fh = [x for x in request.post('hashes', '').split() if len(x) == 32 and util.is_hex(x)]
			if action == 'ban':
				GeneralDataBase(request.config).ban_files(fh)
			elif action == 'unban':
				GeneralDataBase(request.config).unban_files(fh)
	
		data = GeneralDataBase(request.config).get_mod_area_files_data()
		data['section'] = section
		data['msg'] = msg
		return set_mod_cookie(render_to_response(request, 'mod_files.html', data))


def mod_logout(request):
	if sessions.get_mod(request) != None and sessions.get_mod_logout_confirm(request) == request.get('c'):
		sessions.delete_mod(request)
		response = RedirectResponse(request['referer'] or '/')
		cookies.delete_mod(response)
		return response
	else:
		raise Forbidden


@check_age
def options(request, board, thread_id, post_id):
	
	action = request.post('action')
	
	if request.method == 'GET' or not action:
		data = board.db().get_post_options_data(thread_id, post_id, request.remote_host, cookies.check_post_verifier(request, board, thread_id, post_id))
		if data is None or data['status'] != 'undeleted':
			raise NotFound
		return render_to_response(request, 'options.html', data)
	
	if request.method == 'POST' and request.local_referer():
		action = limit(action, 'flag', 'cookie_delete', 'tripcode_delete')
		db = board.db()
	
		if not db.post_not_deleted(thread_id, post_id):
			raise NotFound
		
		if action == 'flag':
			board.db().flag_post(thread_id, post_id, request.remote_host)
			msg = 'Post Flagged.'
		
		elif action == 'cookie_delete':
			if cookies.check_post_verifier(request, board, thread_id, post_id):
				db.delete_post(thread_id, post_id)
				pagecache.thread_modified(board, thread_id, request.config)
				msg = 'Post Deleted.'
			else:
				msg = 'Invalid Cookie.'
		
		elif action == 'tripcode_delete':
			trip = db.get_post_tripcode(thread_id, post_id)
			if trip and tripcodes.make(board, request.post('tripcode', '')) == trip:
				db.delete_post(thread_id, post_id)
				pagecache.thread_modified(board, thread_id, request.config)
				msg = 'Post Deleted.'
			else:
				msg = 'Invalid Tripcode.'
	
		data = board.db().get_post_options_data(thread_id, post_id, request.remote_host, cookies.check_post_verifier(request, board, thread_id, post_id), msg)
		if data is None:
			raise NotFound
		return render_to_response(request, 'options.html', data)
	
	raise Forbidden


def mod_options(request, board, thread_id, post_id):

	if sessions.get_mod(request) is None:
		raise Forbidden
	
	if request.method == 'GET':
		data = board.db().get_post_mod_options_data(thread_id, post_id, GeneralDataBase(request.config))
		if data is None:
			raise NotFound
		return render_to_response(request, 'mod_options.html', data)
	
	elif request.method == 'POST' and request.local_referer():
	
		action = limit(request.post('action'), 'move_thread', 'delete_thread', 'modify_thread', 'modify_post')
		msg = None
		
		if action == 'delete_thread':
			board.db().delete_thread(thread_id)
			pagecache.thread_modified(board, thread_id, request.config)
			return RedirectResponse('/%s/' % board.DIR)
		
		elif action == 'move_thread':
			dest_board = request.config.get_board(request.post('dest_board'))
			keep_locked_copy = 'keep_locked_copy' in request.post
			if dest_board != None:
				dest_tid = database.move_thread(board, thread_id, dest_board, keep_locked_copy, request.server_name)
				pagecache.thread_modified(board, thread_id, request.config)
				if keep_locked_copy:
					msg = 'moved thread to <a href="/%s/%s">/%s/%s</a>. locked this copy.' % ((dest_board.DIR, dest_tid)*2)
				else:
					return RedirectResponse('/%s/%d' % (dest_board.DIR, dest_tid))
		
		elif action == 'modify_thread':
			sticky = 'sticky_thread' in request.post
			lock = 'lock_thread' in request.post
			subject = clamp_str(request.post('subject'), board.MAX_SUBJECT_LENGTH)
			board.db().edit_thread(thread_id, subject, sticky, lock)
			pagecache.thread_modified(board, thread_id, request.config)
			msg = 'modified thread'
		
		elif action == 'modify_post':
			reviewed = 'reviewed' in request.post
			status = limit(request.post('status'), 'undeleted', 'user_deleted', 'mod_deleted')
			name = clamp_str(request.post('name'), board.MAX_NAME_LENGTH)
			tripcode = clamp_str(request.post('tripcode'), board.TRIPCODE_LENGTH[2])
			comment = clamp_str(request.post('comment'), board.MAX_COMMENT_LENGTH)
			mod = 'mod' in request.post
			delete_file = 'delete_file' in request.post
			ban_file = 'ban_file' in request.post
			
			db = board.db()
			if ban_file:
				file_md5 = db.get_post_file_md5(thread_id, post_id)
				if file_md5:
					GeneralDataBase(request.config).add_file_ban(file_md5)
			db.edit_post(thread_id, post_id, status, name, tripcode, mod, comment, delete_file, reviewed)
			pagecache.thread_modified(board, thread_id, request.config)
			msg = 'modified post'
	
		data = board.db().get_post_mod_options_data(thread_id, post_id, GeneralDataBase(request.config), msg)
		if data is None:
			raise NotFound
		return render_to_response(request, 'mod_options.html', data)
	
	raise Forbidden


def mod_post_action(request, board, thread_id, post_id, action):
	action = board.get_mod_post_action(action)
	
	if action is None:
		raise http.NotFound
	
	elif sessions.get_mod(request) is not None and request.local_referer():
		board.db().mod_post_action(thread_id, post_id, action, GeneralDataBase(request.config))
		pagecache.thread_modified(board, thread_id, request.config)
		return Response('success', 'text/plain; charset=utf-8')
	
	else:
		raise Forbidden


def mod_thread_action(request, board, thread_id, action):
	action = board.get_mod_thread_action(action)
	
	if action is None:
		raise http.NotFound
	
	elif sessions.get_mod(request) is not None and request.local_referer():
		board.db().mod_thread_action(thread_id, action, GeneralDataBase(request.config))
		pagecache.thread_modified(board, thread_id, request.config)
		return Response('success', 'text/plain; charset=utf-8')
	
	else:
		raise Forbidden



class Handler(http.RegexRequestHandler):
	
	def __init__(self):
		
		def board_trans(request, dir):
			board = request.config.get_board(dir)
			if board is None:
				raise http.NotFound
			else:
				return board
		
		def int_trans(request, i):
			return int(i)
		
		# funtions to transform arguments to view funtions (groups from the regex) before the are passed to the view functions
		trans = dict(
			board = board_trans,
			page = int_trans,
			thread_id = int_trans,
			post_id = int_trans,
			id = int_trans,
		)
		
		urls = (
			http.URLs(r'^$', index),
			http.URLs(r'^api/boards$', boards_api),
			http.URLs(r'^confirm_overage$', confirm_overage),
			http.URLs(r'^mod$', mod_area, {'section': 'overview'}),
			http.URLs(r'^mod_(?P<section>overview|bans|files)$', mod_area),
			http.URLs(r'^mod_logout$', mod_logout),			
			http.URLs(r'^(?P<board>\w+)/$', view_board,  {'page': 0}),
			http.URLs(r'^(?P<board>\w+)/page(?P<page>[0-9]+)$', view_board),
			http.URLs(r'^(?P<board>\w+)/(?P<id>[0-9]+)$', view_thread),
			http.URLs(r'^(?P<board>\w+)/list', list),
			http.URLs(r'^(?P<board>\w+)/new_thread$', new_thread),
			http.URLs(r'^(?P<board>\w+)/(?P<thread_id>[0-9]+)/reply$', new_reply),
			http.URLs(r'^(?P<board>\w+)/(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)$', fetch_post),
			http.URLs(r'^(?P<board>\w+)/(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)/options$', options),
			http.URLs(r'^(?P<board>\w+)/(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)/mod_options$', mod_options),
			http.URLs(r'^(?P<board>\w+)/(?P<thread_id>[0-9]+)/mod_action/(?P<action>[-_a-zA-Z0-9]+)$', mod_thread_action),
			http.URLs(r'^(?P<board>\w+)/(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)/mod_action/(?P<action>[-_a-zA-Z0-9]+)$', mod_post_action),
		)
		
		http.RegexRequestHandler.__init__(self, urls, trans)

	def handle(self, request):
		request.config = common.get_config()
		return http.RegexRequestHandler.handle(self, request)
	
	def notfound(self, request):
		return http.Response(render_to_string(request, '404.html'), status = 404)
	
	def forbidden(self, request):
		return http.Response(render_to_string(request, '403.html'), status = 403)
	
	def error(self, request):
		logging.exception('exception in request handler')
		return http.Response(render_to_string(request, '500.html'), status = 500)
