"""
abstracts database access. sql goes in here.

more info:

There are Global() database connections that open a child Board() connection for every board that exists.
Both Global() and Board() connections cache configuration information when they are created, and have no way of reloading it or knowing if it changed.
A single Global() is cached per thread, and invalidated/recreated when some configuration changes (by calling reload_thread_globals()).
not sure its 100% concurrently robust...
"""

from __future__ import with_statement

import os, sqlite3, math, time, cPickle as pickle, random, re, socket, struct, threading

import common, uploaded_files, fields, util, parser, pagecache, proxycheck, util.sessions, util.cache as cache

from util import ObjDict


POST_STATUS = ['undeleted', 'user_deleted', 'mod_deleted']


NEW_POST = ObjDict(
	name = '',
	tripcode = '',
	noko = False,
	bump = True,
	lock = False,
	mod = False,
	comment = ''
)

NEW_THREAD = ObjDict(
	forked_thread_id = None,
	forked_post_id = None,
	name = '',
	tripcode = '',
	noko = False,
	lock = False,
	sticky = False,
	mod = False,
	subject = '',
	comment = ''
)


_parsed_comment_cache = cache.Cache(10000)		# (dir, thread_id, post_id) => parsed comment html, exists because post parsing was a bottleneck

_conn_cache = threading.local()					# contains a single "global_db" - Global() or None - per thread...
_conn_cache_fresh_time = 0						# connections born after before this time will put into the cache...


def get_thread_global():
	if not hasattr(_conn_cache, 'global_db') or _conn_cache.global_db.created < _conn_cache_fresh_time:
		_conn_cache.global_db = Global()
	return _conn_cache.global_db


def reload_thread_globals():
	global _conn_cache_fresh_time
	_conn_cache_fresh_time = time.time()
	return get_thread_global()


def _get_parsed_comment(comment, board_db, thread_id, post_id):
	post = (board_db.dir, thread_id, post_id)
	return _parsed_comment_cache.get(post, lambda: parser.parse(comment, (board_db, thread_id, post_id)))


def _delete_parsed_comment(board_db, thread_id, post_id):
	_parsed_comment_cache.delete((board_db.dir, thread_id, post_id))


def make_ip_filter(filter_):
	try:
		f = filter_.split('/')
		ip = struct.unpack('!L', socket.inet_aton(f[0]))[0]
		if len(f) == 1:
			return lambda t_ip: t_ip == f[0]
		else:
			nl = util.clamp(f[1], 0, 32)
			hostmask = int('1'*nl + '0'*(32-nl), 2)
			def f(t_ip):
				try:
					return (struct.unpack('!L', socket.inet_aton(t_ip))[0] & hostmask) == (ip & hostmask)
				except:
					return False
			return f
	except:
		return lambda t_ip: True


class ThreadFull(Exception):
	""" raised when a post is attempted to be added to a thread that has reached the maximum number of posts. """

class DoublePost(Exception):
	""" raised when a duplicate post is attempted to be added to a thread. """

class DoubleThread(Exception):
	""" raised when a duplicate thread is made """

class DuplicateFile(Exception):
	""" raised when a duplicate file is attempted to be posted """

class BannedFile(Exception):
	""" raised when a banned file is attempted to be posted """

class Flooding(Exception):
	""" raised when an ip attempts to post too quickly """


class _DataBase(object):
	""" contains setup/utility methods """
	
	def __init__(self, path):
		
		def dict_factory(cursor, row):
			d = ObjDict()
			for i, col in enumerate(cursor.description):
				d[col[0]] = row[i]
			return d

		self.db = sqlite3.connect(path)
		self.db.isolation_level = None
		self.db.row_factory = dict_factory
		self.cur = self.db.cursor()


	def create_functions(self, gmt_offset):
		self.db.create_function('format_time', 0, lambda: util.format_time(None, gmt_offset))
		self.db.create_function('format_time', 1, lambda t: util.format_time(t, gmt_offset))
		self.db.create_function('inow', 0, lambda: int(time.time()))
		self.db.create_function('format_status', 1, lambda s: POST_STATUS[s])
		self.db.create_function('unhex', 1, lambda x: buffer(util.unhex(x)))
		self.db.create_function('fqdn', 1, lambda x: unicode(socket.getfqdn(x)) if x else None)
	
	
	def _transaction(self, immediate = False):
		class C:
			def __enter__(self_):
				self._begin(immediate)
			def __exit__(self_, type, value, traceback):
				if type == value == traceback == None:	#no exception happened
					self._commit()
				else:
					self._rollback()
		return C()


	def _insert(self, query, *params):
		"""does the query, returns nothing"""
		if params == ():
			self.cur.execute(query)
		else:
			self.cur.execute(query, params)


	def _select(self, query, *params):
		"""does the query and returns the fetchall() result in one call"""
		if params == ():
			self.cur.execute(query)
		else:
			self.cur.execute(query, params)
		return self.cur.fetchall()


	def _select_1(self, query, *params):
		"""does the query and returns the first returned row, or None in one call"""
		data = self._select(query, *params)
		return data[0] if data else None


	def _begin(self, immediate = False):
		self.cur.execute("BEGIN IMMEDIATE" if immediate else "BEGIN")
	
	
	def _commit(self):
		self.cur.execute("COMMIT")
	
	
	def _rollback(self):
		self.cur.execute("ROLLBACK")


	def update_tables(self, script):
		""" take a script containing CREATE TABLE statements and ensures the described tables exist - adding new tables/columns as needed. hacky as hell. """
		with self._transaction(immediate = True):
			for m in re.finditer(r'CREATE TABLE IF NOT EXISTS (.+?) \((.+?)\);', script, re.DOTALL):
				name = m.group(1)
				self._insert(m.group())					# creates table if it doesn't exist - but doesnt add any new columns if table already exists
				
				# add new columns
				self._select_1("SELECT * FROM %s LIMIT 0" % name)
				cur_cols = [col[0] for col in self.cur.description]
				for col in m.group(2).splitlines():
					col = col.strip()
					if col.endswith(','):
						col = col[:-1]
					if not col or 'PRIMARY KEY' in col or 'UNIQUE' in col:
						continue
					col_name = col.partition(' ')[0]
					if not col_name in cur_cols:
						self._insert("ALTER TABLE %s ADD COLUMN %s" % (name, col))


class Global(_DataBase):
	"""
	general/global database containing config and ip/file bans.
	Global and Board instances cache some database data, but once created instances are immutable.
	Thus once the cached data changes in the database any cached instances must not be re-used again.
	"""
	
	DEFAULT_MOD = ('admin', 'password')
	
	CREATE_SCRIPT = """
		CREATE TABLE IF NOT EXISTS unique_string (
			unique_string TEXT NOT NULL
		);
		
		CREATE TABLE IF NOT EXISTS board_config_defaults (
			name TEXT PRIMARY KEY NOT NULL,
			value NOT NULL
		);
		
		CREATE TABLE IF NOT EXISTS mods (
			name TEXT PRIMARY KEY NOT NULL,
			password_hash BLOB NOT NULL,
			admin INTEGER NOT NULL,
			created INTEGER NOT NULL,
			last_active INTEGER NOT NULL DEFAULT 0
		);
		
		CREATE TABLE IF NOT EXISTS modlog (
			time INTEGER NOT NULL,
			mod_name TEXT NOT NULL,
			message TEXT NOT NULL
		);
		
		CREATE TABLE IF NOT EXISTS bans (
			ip TEXT PRIMARY KEY NOT NULL,
			expires INTEGER NOT NULL,
			reason TEXT NOT NULL
		);
		
		CREATE TABLE IF NOT EXISTS file_bans (
			md5 BLOB PRIMARY KEY NOT NULL
		);
	"""

	
	def __init__(self):
		self.created = time.time()
		_DataBase.__init__(self, os.path.join(common.DATABASE_DIR, 'global.sqlite'))
		self.update_tables(self.CREATE_SCRIPT)
		with self._transaction(True):
		
			row = self._select_1("SELECT unique_string FROM unique_string")
			if row is None:
				self.unique_string = util.rand_str(128)
				self._insert("INSERT INTO unique_string (unique_string) VALUES (?)", self.unique_string)
			else:
				self.unique_string = row.unique_string
				
			self.board_config_defaults = ObjDict()
			for row in self._select("SELECT name, value from board_config_defaults"):
				self.board_config_defaults[row.name] = row.value
			for field in Board.CONFIG_FIELDS:
				if field[0] not in self.board_config_defaults:
					self._insert("INSERT INTO board_config_defaults (name, value) VALUES (?, ?)", field[0], field[1].default)
					self.board_config_defaults[field[0]] = field[1].default
			
			self.mods = ObjDict()
			for row in self._select("SELECT name, password_hash, admin FROM mods"):
				self.mods[row.name] = (str(row.password_hash), bool(row.admin))
			if not self.mods:
				self._insert("INSERT INTO mods (name, password_hash, admin, created) VALUES ('admin', ?, 1, ?)", buffer(util.sha1(self.unique_string + 'adminpassword', False)), int(time.time()))
			
			self.boards = []
			for file_ in os.listdir(os.path.join(common.DATABASE_DIR, 'boards')):
				if file_.endswith('.sqlite'):
					dir_ = file_[:-len('.sqlite')]
					if re.match(common.BOARD_DIR_REGEX, dir_):
						self.boards.append(Board(self, dir_))
			self.boards.sort(key = lambda b: b.dir)
		
		self.create_functions(self.board_config_defaults.gmt_offset)


	def make_board(self, dir):
		""" return a bool indicating success or failure. """
		if self.get_board(dir):
			return False
		if not re.match(common.BOARD_DIR_REGEX, dir):
			return False
		board = Board(self, dir)
		board.set_config('title', dir.capitalize(), False)
		board.set_config('desc', 'Welcome to /%s/' % dir.capitalize(), False)
		return True


	def set_board_config_default(self, key, value):
		""" assumes value is valid for the field named by key. """
		self._insert("INSERT OR REPLACE INTO board_config_defaults (name, value) VALUES (?, ?)", key, value)


	def modlog(self, mod_name, message):
		""" note: message will be displayed as raw html """
		with self._transaction():
			self._insert("INSERT INTO modlog (time, mod_name, message) VALUES (inow(), ?, ?)", mod_name, message)
			self._insert("DELETE FROM modlog WHERE time < ?", time.time() - common.DAYS_TO_KEEP_MODLOG*24*60*60)
	
	
	def add_mod(self, name, admin):
		password = util.rand_str(12)
		password_hash = util.sha1(self.unique_string + name + password, False)
		self._insert("INSERT OR REPLACE INTO mods (name, password_hash, admin, created, last_active) VALUES (?, ?, ?, inow(), inow())", name, buffer(password_hash), int(admin))
		return password
	

	def add_default_mod(self):
		name = self.DEFAULT_MOD[0]
		password = self.DEFAULT_MOD[1]
		password_hash = util.sha1(self.unique_string + name + password, False)
		self._insert("INSERT OR REPLACE INTO mods (name, password_hash, admin, created, last_active) VALUES (?, ?, 1, inow(), inow())", name, buffer(password_hash))
	
	
	def get_mod(self, name, password = None):
		""" if the name+password is valid the name is returned, otherwise None. """
		if name in self.mods:
			if password is None or (util.sha1(self.unique_string + name + password, False) == self.mods[name][0]):
				return name
		return None

	
	def reset_mod_password(self, name):
		password = util.rand_str(12)
		self.set_mod_password(name, password)
		return password


	def set_mod_password(self, name, password):
		password_hash = util.sha1(self.unique_string + name + password, False)
		self._insert("UPDATE mods SET password_hash = ? WHERE name = ?", buffer(password_hash), name)

	
	def set_mod_admin(self, name, admin):
		self._insert("UPDATE mods SET admin = ? WHERE name = ?", int(admin), name)
	
	
	def delete_mod(self, name):
		self._insert("DELETE FROM mods WHERE name = ?", name)


	def mod_active(self, name):
		self._insert("UPDATE mods SET last_active = inow() WHERE name = ?", name)


	def is_admin(self, name):
		return name in self.mods and self.mods[name][1]


	def is_default_mod(self, name):
		return name == self.DEFAULT_MOD[0] and name in self.mods and self.mods[name][0] == util.sha1(self.unique_string + name + self.DEFAULT_MOD[1], False)
	
	
	def get_board(self, dir):
		for board in self.boards:
			if board.dir == dir:
				return board
		return None
	

	def get_index_data(self):
		data = ObjDict(threads = [], new_post = NEW_POST, on_thread_page = False)
		for board_db in self.boards:
			for thread in board_db.get_index_data():
				data.threads.append(ObjDict(board_db = board_db, thread = thread))
		data.threads.sort(key = lambda e: e.thread.updated, reverse = True)
		return data
	
	
	def get_config_area_overview_data(self):
		return ObjDict(
			boards = [(b, b.get_config_area_overview_data()) for b in self.boards],
			parsed_comment_cache = len(_parsed_comment_cache),
			page_cache = pagecache.size(),
			sessions = util.sessions.size(),
		)
	

	def get_config_area_mods_data(self):
		rows = self._select("SELECT format_time(created) as created, format_time(last_active) as last_active, name, admin FROM mods ORDER BY name")
		return ObjDict(mods = rows)


	def get_config_area_modlog_data(self):
		rows = self._select("""	SELECT format_time(time) as time, time as raw_time, mod_name, message
								FROM modlog
								ORDER BY raw_time DESC""")
		return ObjDict(modlog = rows)
	
	
	def get_config_area_bans_data(self, filter):
		self.db.create_function('filter', 1, make_ip_filter(filter))
		q = """	SELECT ip, expires - inow() as expires, reason
				FROM bans
				WHERE expires > inow() AND filter(ip)
				ORDER BY expires"""
		bans = self._select(q)
		for ban in bans:
			ban.expires = util.format_interval(ban['expires'], 2)
		return ObjDict(bans = bans)
	

	def get_config_area_files_data(self):
		return self._select_1("SELECT COUNT(*) AS num_file_bans FROM file_bans")
	

	def get_config_area_posts_by_ip_data(self, ip):
		boards = []
		for board_db in self.boards:
			data = board_db.get_config_area_posts_by_ip_data(ip)
			if data.threads:
				boards.append((board_db, data))
		return ObjDict(boards = boards)


	def get_boards_api_data(self, server_host):
		data = []
		for board in self.boards:
			data.append(ObjDict(
				title = board.config.title,
				href = 'http://%s/%s/' % (server_host, board.dir),
				updated = board.updated()
			))
		return data
	
	
	def get_flags(self):
		return [(b.dir, b.get_flags()) for b in self.boards]
	
	
	def move_thread(self, src_board_db, src_thread_id, dest_board_db, keep_locked_copy):
		
		# grab data from src database
		with src_board_db._transaction():
			thread = src_board_db._select_1("SELECT * FROM threads WHERE id = ?", src_thread_id)
			posts = src_board_db._select("SELECT * FROM posts WHERE thread_id = ?", src_thread_id)
		
		if thread:
			# put data into dest database
			with dest_board_db._transaction(immediate = True):
			
				q = "INSERT INTO threads (subject, sticky, locked, updated, forked_thread_id, forked_post_id) VALUES (?, ?, ?, inow(), NULL, NULL)"
				dest_board_db._insert(q, thread.subject, thread.sticky, thread.locked)
				dest_thread_id = dest_board_db.cur.lastrowid
				
				for post in posts:
					q = """	INSERT INTO posts (thread_id, id, posted, status, name, tripcode, mod, comment, file_md5, file_meta, ip)
							VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)"""
					dest_board_db._insert(q, dest_thread_id, post.id, post.posted, post.status, post.name, post.tripcode, post.mod, post.comment, post.file_md5, post.file_meta, post.ip)
					if post['file_meta']:
						f = pickle.loads(str(post.file_meta))
						f.copy((src_board_db.dir, src_thread_id, post.id), (dest_board_db.dir, dest_thread_id, post.id))
			
			if keep_locked_copy:
				# link src to dest thread and lock src
				comment = "Please continue this thread in >>/%s/%d" % (dest_board_db.dir, dest_thread_id)
				try:
					src_board_db.add_new_reply(src_thread_id, '', None, True, comment, None, None, None, False, True)
				except:
					pass
			else:
				# delete src
				src_board_db.delete_thread(src_thread_id)
			return dest_thread_id
		else:
			return None


	def get_ban(self, ip):
		q = """SELECT ip, fqdn(ip) as fqdn, format_time(expires) AS expires, reason FROM bans WHERE ip = ? AND expires > inow()"""
		return self._select_1(q, ip)
	

	def add_ban(self, ip, expires, reason):
		q = "INSERT OR REPLACE INTO bans (ip, expires, reason) VALUES (?, ?, ?)"
		self._insert(q, ip, expires, reason)


	def delete_ban(self, ip):
		self._insert("DELETE FROM bans WHERE ip = ?", ip)


	def delete_expired_bans(self):
		self._insert("DELETE FROM bans WHERE expires <= inow()")
	
	
	def file_banned(self, md5):
		return self._select_1("SELECT 1 FROM file_bans WHERE md5 = ?", md5) is not None
	

	def add_file_ban(self, md5):
		self._insert("INSERT OR IGNORE INTO file_bans (md5) VALUES (?)", buffer(md5))


	def ban_files(self, md5s):
		with self._transaction():
			q = "INSERT OR IGNORE INTO file_bans (md5) VALUES (unhex(?))"
			for md5 in md5s:
				self._insert(q, md5)


	def unban_files(self, md5s):
		with self._transaction():
			q = "DELETE FROM file_bans WHERE md5 = unhex(?)"
			for md5 in md5s:
				self._insert(q, md5)


class Board(_DataBase):
	""" each board has its own database, containing config, posts and flagged/reviewed posts data """
	
	CREATE_SCRIPT = """
		CREATE TABLE IF NOT EXISTS config (
			name TEXT PRIMARY KEY NOT NULL,
			value NOT NULL,
			use_default INTEGER NOT NULL DEFAULT 1
		);
		
		CREATE TABLE IF NOT EXISTS threads (
			id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
			forked_thread_id INTEGER,
			forked_post_id INTEGER,
			subject TEXT NOT NULL,
			sticky INTEGER NOT NULL,
			locked INTEGER NOT NULL,
			updated INTEGER NOT NULL
		);
		CREATE INDEX IF NOT EXISTS threads_updated ON threads (updated);
		CREATE INDEX IF NOT EXISTS threads_forked ON threads (forked_thread_id, forked_post_id);
		
		CREATE TABLE IF NOT EXISTS posts (
			thread_id INTEGER NOT NULL,
			id INTEGER NOT NULL,
			posted INTEGER NOT NULL,
			name TEXT NOT NULL,
			tripcode TEXT,
			mod INTEGER NOT NULL,
			ip TEXT,
			comment TEXT NOT NULL,
			file_md5 BLOB,
			file_meta BLOB,
			status INTEGER NOT NULL,
			PRIMARY KEY (thread_id, id)
		);
		CREATE INDEX IF NOT EXISTS posts_posted ON posts (posted);
		CREATE INDEX IF NOT EXISTS posts_file_md5 ON posts (file_md5);
		
		CREATE TABLE IF NOT EXISTS flags (
			thread_id INTEGER NOT NULL,
			post_id INTEGER NOT NULL,
			ip TEXT NOT NULL,
			PRIMARY KEY (thread_id, post_id, ip)
		);
		
		CREATE TABLE IF NOT EXISTS reviewed_posts (
			thread_id INTEGER NOT NULL,
			post_id INTEGER NOT NULL,
			time INTEGER NOT NULL,
			PRIMARY KEY (thread_id, post_id)
		);
	"""
			
		
	CONFIG_FIELDS = (
		('title', fields.String("Title", 0, 256, "Example Board")),
		('desc', fields.String("Description", 0, 1024*1024, "This is an img2 imageboard.", "HTML is valid here.", True)),
	
		('css', fields.String("CSS", 0, 1024, 'futaba.css')),
		('show_age_check_page', fields.Bool("Show Age Check Page", False)),
		
		('default_first_names', fields.String("Default First Names", 0, 1024, "", "One name per line. When no name is entered a default name will be created by adding <random-default-first-name> + \" \" + <random-default-second-name>", True)),
		('default_second_names', fields.String("Default Second Names", 0, 1024, "", "One name per line.", True)),

		('post_filters', fields.String("Post Filters", 0, 1024*1024, "", "One filter per line. <string-to-find> + \" \" + <string-to-replace-with>. Can contain HTML.", True)),

		('allow_images', fields.Bool("Allow Images", True, "Allow Image Uploads.")),
		('allow_flashes', fields.Bool("Allow Flashes", True, "Allow Flash Uploads.")),
		('allow_torrents', fields.Bool("Allow Torrents", True, "Allow Torrent Uploads.")),
		('max_file_length', fields.Int("Max File Length", 0, 32*1024*1024, 2*1024*1024, "In Bytes.")),
		('allow_duplicate_files', fields.Bool("Allow Duplicate Files", True)),
		
		('forking', fields.Bool("Allow Forking", True)),
		('delete_forked_threads', fields.Bool("Delete Forked Threads", False, "If not set then threads wont be deleted if a thread is still alive that was started by forking one of their posts.")),
		
		('allow_names', fields.Bool("Allow Names", True, "If not set a name will be selected from the default names if there are any.")),
		('allow_tripcodes', fields.Bool("Allow Tripcodes", True)),
		('allow_thread_without_file', fields.Bool("Allow Thread Without File", True)),
		('allow_reply_without_file', fields.Bool("Allow Reply Without File", True)),
		('allow_no_comment_with_file', fields.Bool("Allow No Comment With File", True, "If set textless posts will be allowed as long as a file is given.")),

		('gmt_offset', fields.Int("GMT Offset", 0, 23, 0, "Affects displayed times/dates. In hours.")),
	
		('max_image_size', fields.Int("Max Image Width/Height", 16, 1024*32, 4096, "In Pixels.")),
		('min_image_size_for_thumbnailing', fields.Int("Min Image Size For Thumbnailing", 8, 1024*32, 390, "Images this big or bigger (pixels) will be thumbnailed.")),
		('max_image_thumb_size', fields.Int("Max Image Thumbnail Size", 8, 1024*32, 300, "Thumbnails will be created at this size (pixels).")),
		('clean_images', fields.Bool("Clean Images", True, "Remove exif data and embedded files (eg. rars) from image uploads.")),
		
		('max_comment_length', fields.Int("Max Comment Length", 8, 1024**2, 1024, "Max allowed comment length.")),
		('max_subject_length', fields.Int("Max Subject Length", 8, 1024**2, 64, "Max allowed subject length.")),
		('max_name_length', fields.Int("Max Name Length", 8, 1024**2, 32, "Max allowed name length")),
		('max_emote_length', fields.Int("Max Emote Length", 0, 1024*1024, 8, "Posts with length <= this will be wrapped in <div class=\"emote\">")),
		('max_youtube_embeds_per_post', fields.Int("Max YouTubes Per Post", 0, 4096, 1)),
		('posted_url_prefix', fields.String("Posted URL Prefix", 0, 1024, "", "Prefixed on to auto-linked URLs. For referer removal services.")),
		
		('min_replies_for_omitting', fields.Int("Min Replies For Omitting", 2, 1024*1024, 12, "If there are this many or more replies then some will be omitted on the thread preview.")),
		('num_replies_to_show_when_omitting', fields.Int("Num Replies To Show When Omitting", 1, 1024*1024, 6)),
		
		('min_seconds_between_posts', fields.Int("Min Seconds Between Posts", 0, 1024*1024, 5, "Flood Control, per IP.")),
		('min_seconds_between_threads', fields.Int("Min Seconds Between Threads", 0, 1024*1024, 5, "Flood Control, per IP.")),

		('proxy_dnsbls', fields.String("Proxy DNSBLs", 0, 1024**2, "torexit.dan.me.uk", "List of DNS blacklists used to identify proxies. One per line.", True)),
		('block_proxies_files', fields.Bool("Block Proxies Files", True, "Don't allow proxies to post files.")),
		('block_proxies_fully', fields.Bool("Block Proxies Fully", False, "Don't allow proxies to post at all.")),

		('threads_per_page', fields.Int("Threads Per Page", 1, 1024, 12, "Num. thread-previews shown on each board page.")),
		('max_thread_posts', fields.Int("Max Thread Posts", 1, 1024*1024, 1001, "Includes thread head and replies.")),
		
		('max_threads', fields.Int("Max Threads", 10, 1024*1024, 250, "Num threads at any one time are limited to this - old ones are deleted.")),
		
		('max_threads_on_front_page', fields.Int("Max Threads On Front Page", 0, 1024, 4, "Max number of threads from this board to show on the front page.")),
		('max_days_old_for_front_page_threads', fields.Int("Max Days Old For Front Page Threads", 0, 1024*1024, 12, "Max age of threads shown on the front page.")),
		
		('hours_to_keep_post_ips', fields.Int("Hours To Keep Post IPs", 0, 1024*1024, 48, "After this time poster ips will be wiped from the database.")),
		
		('default_ban_reason', fields.String("Default Ban Reason", 0, 2048, "Not respecting the rules.", "This will be used on bans set by the quick-access links on posts.", True)),
		('default_ban_length', fields.Interval("Default Ban Length", 0, 60*60*24*365*4, 60*60*24*30, "Same as above. This field should be formatted as described on the ban page.")),
	)
	
	def __init__(self, global_db, dir):
		self.global_db = global_db
		self.dir = dir
		_DataBase.__init__(self, os.path.join(common.DATABASE_DIR, 'boards/%s.sqlite' % dir))
		self.update_tables(self.CREATE_SCRIPT)
		with self._transaction(True):
		
			self.config = util.ObjDict()
			self.use_config_defaults = util.ObjDict()
			for row in self._select("SELECT name, value, use_default FROM config"):
				self.use_config_defaults[row.name] = bool(row.use_default)
				self.config[row.name] = global_db.board_config_defaults[row.name] if row.use_default else row.value
			for field in self.CONFIG_FIELDS:
				if field[0] not in self.config:
					self._insert("INSERT INTO config (name, value, use_default) VALUES (?, ?, 1)", field[0], field[1].default)
					self.use_config_defaults[field[0]] = True
					self.config[field[0]] = field[1].default
		
		#special case optimizations
		self.config.default_first_names_parsed = self.config.default_first_names.splitlines()
		self.config.default_second_names_parsed = self.config.default_second_names.splitlines()
		froms = {}
		for line in self.config.post_filters.splitlines():
			f, _, t = line.partition(' ')
			if f and t:
				froms[f] = t
		self.config.post_filters_parsed = (
			froms,
			re.compile('|'.join(map(re.escape, froms.keys())))
		)
		
		self.create_functions(self.config.gmt_offset)


	def set_config(self, key, value, use_default):
		""" assumes value is valid for the field named by key. """
		self._insert("INSERT OR REPLACE INTO config (name, value, use_default) VALUES (?, ?, ?)", key, value, int(use_default))


	def get_default_name(self):
		f = self.config.default_first_names_parsed
		s = self.config.default_second_names_parsed
		if f and s:
			return random.choice(f) + ' ' + random.choice(s)
		elif f:
			return random.choice(f)
		elif s:
			return random.choice(s)
		else:
			return ''

	
	def files_allowed(self):
		return self.config.allow_images or self.config.allow_torrents or self.config.allow_flashes
	

	def file_exts(self):
		e = []
		if self.config.allow_images:
			e += common.IMAGE_FILE_EXTS
		if self.config.allow_torrents:
			e += common.TORRENT_FILE_EXTS
		if self.config.allow_flashes:
			e += common.FLASH_FILE_EXTS
		return e
	
	
	def file_dir(self):
		return os.path.join(common.UPLOADED_FILE_DIR, self.dir)

	
	def _get_forked_post_data(self, thread_id, post_id, display_thread_id = -1):
		#display_thread_id will be excluded from the list of other forked threads
		q = """	SELECT thread_id, id, ip, name, format_time(posted) AS posted, tripcode, mod, format_status(status) AS status, comment, file_meta,
				EXISTS (SELECT 1 FROM threads WHERE forked_thread_id = posts.thread_id and forked_post_id = posts.id AND id != ?) AS has_forked_threads
				FROM posts
				WHERE thread_id = ? AND id = ?"""
		post = self._select_1(q, display_thread_id, thread_id, post_id)
		if post is None:
			return ObjDict(status = 'mod_deleted')
		if post.status != 'undeleted':
			return ObjDict(status = post.status)
		post.file_meta = pickle.loads(str(post.file_meta)) if post.file_meta else None
		post.comment = _get_parsed_comment(post.comment, self, thread_id, post_id)
		#get list of existing threads forked from this post
		if post.has_forked_threads > 0:
			q = """	SELECT id, subject FROM threads
					WHERE forked_thread_id = ? AND forked_post_id = ? AND id != ?
					ORDER BY id"""
			post.forked_threads = self._select(q, thread_id, post_id, display_thread_id)
		else:
			post.forked_threads = []
		return post

	
	def _get_forked_thread_data(self, thread_id, post_id):
		
		q = """	SELECT *,
				(SELECT COUNT(*) FROM posts WHERE posts.thread_id = threads.id) AS num_posts,
				(SELECT COUNT(*) FROM posts WHERE posts.thread_id = threads.id AND file_md5 IS NOT NULL) AS num_files
				FROM threads
				WHERE forked_thread_id = ? AND forked_post_id = ?
				ORDER BY id"""
		threads = self._select(q, thread_id, post_id)
		
		for thread in threads:
			q = """	SELECT thread_id, id, ip, name, format_time(posted) AS posted, tripcode, mod, format_status(status) AS status, comment, file_meta
					FROM posts
					WHERE thread_id = ? AND id = 0"""
			post = self._select_1(q, thread.id)
			post.file_meta = pickle.loads(str(post.file_meta)) if post.file_meta else None
			post.comment = _get_parsed_comment(post.comment, self, thread.id, 0)
			thread.posts = [post]
		
		return threads
	

	def get_index_data(self):
		""" largely a c/p of get_view_board_data, some subtle sql diffs tho. """
		with self._transaction():
			#get list of threads
			q = """	SELECT *,
					(SELECT COUNT(*) FROM posts WHERE posts.thread_id = threads.id) AS num_posts,
					(SELECT COUNT(*) FROM posts WHERE posts.thread_id = threads.id AND file_md5 IS NOT NULL) AS num_files
					FROM threads
					WHERE updated > ?
					ORDER BY updated DESC LIMIT ?"""
			threads = self._select(q, time.time() - 60*60*24*self.config.max_days_old_for_front_page_threads, self.config.max_threads_on_front_page)
			#get list of posts for each thread
			q = """	SELECT * FROM (
						SELECT thread_id, id, ip, format_time(posted) AS posted, name, tripcode, mod, format_status(status) AS status, comment, file_meta,
						EXISTS (SELECT 1 FROM threads WHERE forked_thread_id = posts.thread_id AND forked_post_id = posts.id) AS has_forked_threads
						FROM posts
						WHERE thread_id = ?
						ORDER BY (id = 0) DESC, posts.id DESC LIMIT ?
					) ORDER BY id ASC"""
			for thread in threads:
				#get forked post data?
				if thread.forked_thread_id is not None and thread.forked_post_id is not None:
					thread.forked_post = self._get_forked_post_data(thread.forked_thread_id, thread.forked_post_id, thread.id)
				else:
					thread.forked_post = None
				#get posts data
				if thread.num_posts-1 >= self.config.min_replies_for_omitting:
					limit = self.config.num_replies_to_show_when_omitting + 1
				else:
					limit = -1
				posts = self._select(q, thread.id, limit)
				#parse posts for display
				files_shown = 0
				for post in posts:
					if post.file_meta:
						post.file_meta = pickle.loads(str(post.file_meta))
						files_shown += 1
					post.comment = _get_parsed_comment(post.comment, self, thread.id, post.id)
					post.forked_threads = self._get_forked_thread_data(thread.id, post.id) if post.has_forked_threads else []
				thread.posts = posts
				thread.max_post_id = posts[-1].id
				thread.num_posts_omitted = (posts[1].id-1) if (len(posts) > 1) else 0
				thread.num_files_omitted = thread.num_files - files_shown
			return threads


	def get_view_board_data(self, page):
		with self._transaction():
			q = "SELECT COUNT(*) AS count FROM threads"
			num_threads = self._select_1(q).count
			num_pages = int(math.ceil(num_threads / float(self.config.threads_per_page)))
			pages = range(max(1, num_pages))
			#get list of threads
			q = """	SELECT *,
					(SELECT COUNT(*) FROM posts WHERE posts.thread_id = threads.id) AS num_posts,
					(SELECT COUNT(*) FROM posts WHERE posts.thread_id = threads.id AND file_md5 IS NOT NULL) AS num_files
					FROM threads
					ORDER BY sticky DESC, updated DESC LIMIT ?, ?"""
			threads = self._select(q, self.config.threads_per_page * page, self.config.threads_per_page)
			#no threads on this page?
			if page > 0 and len(threads) == 0:
				return None
			#get list of posts for each thread
			q = """	SELECT * FROM (
						SELECT thread_id, id, ip, format_time(posted) AS posted, name, tripcode, mod, format_status(status) AS status, comment, file_meta,
						EXISTS (SELECT 1 FROM threads WHERE forked_thread_id = posts.thread_id AND forked_post_id = posts.id) AS has_forked_threads
						FROM posts
						WHERE thread_id = ?
						ORDER BY (id = 0) DESC, posts.id DESC LIMIT ?
					) ORDER BY id ASC"""
			for thread in threads:
				#get forked post data?
				if thread.forked_thread_id is not None and thread.forked_post_id is not None:
					thread.forked_post = self._get_forked_post_data(thread.forked_thread_id, thread.forked_post_id, thread.id)
				else:
					thread.forked_post = None
				#get posts data
				if (thread.num_posts-1) >= self.config.min_replies_for_omitting:
					limit = self.config.num_replies_to_show_when_omitting + 1
				else:
					limit = -1
				posts = self._select(q, thread.id, limit)
				#parse posts for display
				files_shown = 0
				for post in posts:
					if post.file_meta:
						post.file_meta = pickle.loads(str(post.file_meta))
						files_shown += 1
					post.comment = _get_parsed_comment(post.comment, self, thread.id, post.id)
					post.forked_threads = self._get_forked_thread_data(thread.id, post.id) if post.has_forked_threads else []
				thread.posts = posts
				thread.max_post_id = posts[-1].id
				thread.num_posts_omitted = (posts[1].id-1) if (len(posts) > 1) else 0
				thread.num_files_omitted = (thread.num_files - files_shown)
			return ObjDict(board_db = self, threads = threads, cur_page = page, pages = pages, new_post = NEW_POST, new_thread = NEW_THREAD, on_thread_page = False)
	
	
	def get_view_thread_data(self, id):
		with self._transaction():
			#get thread data
			thread = self._select_1("SELECT * FROM threads WHERE id = ?", id)
			if thread == None:
				return None
			#get forked post data?
			if thread.forked_thread_id is not None and thread.forked_post_id is not None:
				thread.forked_post = self._get_forked_post_data(thread.forked_thread_id, thread.forked_post_id, id)
			else:
				thread.forked_post = None
			#get post data
			q = """	SELECT thread_id, id, ip, format_time(posted) AS posted, name, tripcode, mod, format_status(status) AS status, comment, file_meta,
					EXISTS (SELECT * FROM threads WHERE forked_thread_id = posts.thread_id AND forked_post_id = posts.id) AS has_forked_threads
					FROM posts
					WHERE thread_id = ?
					ORDER BY id"""
			posts = self._select(q, id)
			thread.blurb = parser.blurb(id, thread.subject, posts[0].comment)
			#parse for display
			for post in posts:
				post.file_meta = pickle.loads(str(post.file_meta)) if post.file_meta else None
				post.comment = _get_parsed_comment(post.comment, self, id, post.id)
				post.forked_threads = self._get_forked_thread_data(id, post.id) if post.has_forked_threads else []
			thread.posts = posts
			thread.num_posts = len(posts)
			thread.max_post_id = posts[-1].id
			return ObjDict(board_db = self, thread = thread, new_post = NEW_POST, on_thread_page = True)
	
	
	def get_list_data(self):
		q = """	SELECT *,
				format_time(updated) AS updated,
				(SELECT comment FROM posts WHERE thread_id = threads.id AND posts.id = 0) AS head_comment,
				(SELECT COUNT(*) FROM posts WHERE thread_id = threads.id) AS num_posts,
				(SELECT COUNT(*) FROM posts WHERE thread_id = threads.id AND file_md5 IS NOT NULL) AS num_files
				FROM threads
				ORDER BY sticky DESC, updated DESC"""
		threads = self._select(q)
		num_pages = int(math.ceil(len(threads) / float(self.config.threads_per_page)))
		pages = range(max(1, num_pages))
		for thread in threads:
			thread.blurb = parser.blurb(thread.id, thread.subject, thread.head_comment)
		return ObjDict(board_db = self, threads = threads, cur_page = -1, pages = pages, new_thread = NEW_THREAD)


	def get_board_config_data(self):
		config = ObjDict()
		for row in self._select("SELECT name, value, use_default FROM config"):
			config[row.name] = ObjDict(value = row.value, use_default = row.use_default)
		return ObjDict(board_db = self, config = config)
		
	
	def get_post_data(self, thread_id, post_id):
		with self._transaction():
			q = """	SELECT thread_id, id, ip, format_time(posted) AS posted, name, tripcode, mod, format_status(status) AS status, comment, file_meta,
					EXISTS (SELECT * FROM threads WHERE forked_thread_id = posts.thread_id AND forked_post_id = posts.id) AS has_forked_threads
					FROM posts
					WHERE thread_id = ? AND id = ?"""
			post = self._select_1(q, thread_id, post_id)
			if not post:
				return None
			
			q1 =  """	SELECT id, subject
						FROM threads
						WHERE forked_thread_id = ? AND forked_post_id = ?
						ORDER BY id"""
			post.file_meta = pickle.loads(str(post.file_meta)) if post.file_meta else None
			post.comment = _get_parsed_comment(post.comment, self, thread_id, post_id)
			post.forked_threads = self._select(q1, thread_id, post_id)
		
			# need some thread data too
			q = """	SELECT id, locked, (SELECT COUNT(*) FROM posts WHERE thread_id = threads.id) AS num_posts
					FROM threads
					WHERE id = ?"""
			thread = self._select_1(q, thread_id)
			
			return ObjDict(board_db = self, thread = thread, post = post)
	

	def get_posts_data(self, thread_id, id_start, id_end):
		with self._transaction():
			q = """	SELECT thread_id, id, ip, format_time(posted) AS posted, name, tripcode, mod, format_status(status) AS status, comment, file_meta,
					EXISTS (SELECT * FROM threads WHERE forked_thread_id = posts.thread_id AND forked_post_id = posts.id) AS has_forked_threads
					FROM posts
					WHERE thread_id = ? AND id >= ? AND id < ?
					ORDER BY id"""
			posts = self._select(q, thread_id, id_start, id_end)
			if not posts:
				return None
			
			for post in posts:
				post.file_meta = pickle.loads(str(post.file_meta)) if post.file_meta else None
				post.comment = _get_parsed_comment(post.comment, self, thread_id, post.id)
				post.forked_threads = self._get_forked_thread_data(thread_id, post.id) if post.has_forked_threads else []
		
			# need some thread data too
			q = """	SELECT id, locked, (SELECT COUNT(*) FROM posts WHERE thread_id = threads.id) AS num_posts
					FROM threads
					WHERE id = ?"""
			thread = self._select_1(q, thread_id)
			thread.posts = posts
			
			return ObjDict(board_db = self, thread = thread, on_thread_page = False)


	def get_new_reply_data(self, thread_id):
		q = """	SELECT id, locked,
				(SELECT MAX(id)+1 FROM posts WHERE thread_id = threads.id) AS next_id
				FROM threads
				WHERE id = ?"""
		thread = self._select_1(q, thread_id)
		return None if (thread is None) else ObjDict(thread = thread)
	
	
	def add_new_thread(self, subject, sticky, locked, forked_thread_id, forked_post_id, name, tripcode, mod, comment, file_md5, file_meta, ip):
		""" either return (id-of-deleted-threads, id-of-new-thread) or raise DoubleThread/Flooding/DuplicateFile/BannedFile """
		with self._transaction(immediate = True):
			
			file_md5 = buffer(file_md5) if file_md5 else None
			file_meta = buffer(pickle.dumps(file_meta, True)) if file_meta else None
			
			if self.global_db.file_banned(file_md5):
				raise BannedFile
			if (not self.config.allow_duplicate_files) and self.file_exists(file_md5):
				raise DuplicateFile
		
			q = "SELECT 1 FROM posts WHERE id = 0 AND ((comment != '' AND comment = ?) OR (file_md5 IS NOT NULL AND file_md5 = ?))"
			if self._select_1(q, comment, file_md5):
				raise DoubleThread
			
			q = "SELECT 1 FROM posts WHERE ip = ? AND id = 0 AND posted > ? LIMIT 1"
			if self._select_1(q, ip, time.time() - self.config.min_seconds_between_threads):
				raise Flooding
			
			q = "INSERT INTO threads (subject, sticky, locked, updated, forked_thread_id, forked_post_id) VALUES (?, ?, ?, inow(), ?, ?)"
			self._insert(q, subject, sticky, locked, forked_thread_id, forked_post_id)
			thread_id = self.cur.lastrowid
				
			q = "INSERT INTO posts (thread_id, id, posted, status, name, tripcode, mod, comment, file_md5, file_meta, ip) VALUES (?, 0, inow(), 0, ?, ?, ?, ?, ?, ?, ?)"
			self._insert(q, thread_id, name, tripcode, mod, comment, file_md5, file_meta, ip)
			
			#delete old threads
			if self.config.delete_forked_threads:
				q = """	SELECT id FROM threads
						ORDER BY sticky DESC, updated DESC
						LIMIT ?, -1"""
			else:
				q = """	SELECT id FROM threads
						WHERE id NOT IN (select forked_thread_id FROM threads)
						ORDER BY sticky DESC, updated DESC
						LIMIT ?, -1"""
			dt = self._select(q, self.config.max_threads)
			for t in dt:
				self._delete_thread(t.id)
			deleted_threads = [t.id for t in dt]
			
			return deleted_threads, thread_id
	

	def edit_thread(self, thread_id, subject, sticky, lock):
		q = "UPDATE threads SET sticky = ?, locked = ?, subject = ? WHERE id = ?"
		self._insert(q, sticky, lock, subject, thread_id)
	
	
	def delete_thread(self, thread_id):
		with self._transaction():
			self._delete_thread(thread_id)

	
	def _delete_thread(self, thread_id):
		self._insert("DELETE FROM threads WHERE id = ?", thread_id)
		#delete files
		q = "SELECT id, file_meta FROM posts WHERE thread_id = ? AND file_md5 IS NOT NULL"
		files = self._select(q, thread_id)
		for f in files:
			uploaded_files.delete(self, thread_id, f.id, pickle.loads(str(f.file_meta)))
		try:
			util.rmdirs(os.path.join(self.file_dir(), '%d' % thread_id))
		except:
			pass
		self._insert("DELETE FROM posts WHERE thread_id = ?", thread_id)
		self._insert("DELETE FROM flags WHERE thread_id = ?", thread_id)
		self._insert("DELETE FROM reviewed_posts WHERE thread_id = ?", thread_id)
	
		
	def edit_post(self, thread_id, post_id, status, name, tripcode, mod, comment, delete_file, reviewed):
		with self._transaction(immediate = True):
			q = "SELECT file_md5, file_meta FROM posts WHERE thread_id = ? AND id = ?"
			row = self._select_1(q, thread_id, post_id)
			if row:
				if delete_file and row.file_md5:
					file_meta = pickle.loads(str(row.file_meta))
					uploaded_files.delete(self, thread_id, post_id, file_meta)
					q = "UPDATE posts SET file_md5 = NULL, file_meta = NULL WHERE thread_id = ? AND id = ?"
					self._insert(q, thread_id, post_id)
				q = "UPDATE posts SET status = ?, name = ?, tripcode = ?, mod = ?, comment = ? WHERE thread_id = ? AND id = ?"
				self._insert(q, POST_STATUS.index(status), name, tripcode, int(mod), comment, thread_id, post_id)
				if reviewed:
					self._insert("INSERT OR IGNORE INTO reviewed_posts (thread_id, post_id, time) VALUES (?, ?, inow())", thread_id, post_id)
				_delete_parsed_comment(self, thread_id, post_id)

	
	def delete_post(self, thread_id, post_id, by_mod = False, delete_thread_if_alone = False):
		with self._transaction(immediate = True):
			q = "SELECT file_meta FROM posts WHERE thread_id = ? AND id = ?"
			data = self._select_1(q, thread_id, post_id)
			if data:
				if data.file_meta:
					uploaded_files.delete(self, thread_id, post_id, pickle.loads(str(data.file_meta)))
				q = """	UPDATE posts
						SET status = ?, tripcode = '', name = '', comment = '', file_md5 = NULL, file_meta = NULL
						WHERE thread_id = ? AND id = ?"""
				self._insert(q, 2 if by_mod else 1, thread_id, post_id)
				self._insert("INSERT OR IGNORE INTO reviewed_posts (thread_id, post_id, time) VALUES (?, ?, inow())", thread_id, post_id)
				_delete_parsed_comment(self, thread_id, post_id)
			if delete_thread_if_alone:
				q = "SELECT COUNT(*) AS count FROM posts WHERE thread_id = ? AND status = 0"
				if self._select_1(q, thread_id).count == 0:
					self._delete_thread(thread_id)


	def ban_post_ip(self, thread_id, post_id, expires, reason):
		q = self._select_1("SELECT ip FROM posts WHERE thread_id = ? AND id = ? AND ip IS NOT NULL", thread_id, post_id)
		if q:
			self.global_db.add_ban(q.ip, expires, reason)
			return q.ip


	def ban_post_file(self, thread_id, post_id):
		q = self._select_1("SELECT file_md5 FROM posts WHERE thread_id = ? AND id = ? AND file_md5 IS NOT NULL", thread_id, post_id)
		if q:
			self.global_db.add_file_ban(str(q.file_md5))
			return util.hex(str(q.file_md5))


	def add_new_reply(self, thread_id, name, tripcode, mod, comment, file_md5, file_meta, ip, bump, lock):
		""" either returns the id of the post or raises ThreadFull/DoublePost/Flooding/DuplicateFile/BannedFile """
		with self._transaction(immediate = True):
			
			file_md5 = buffer(file_md5) if file_md5 else None
			file_meta = buffer(pickle.dumps(file_meta, True)) if file_meta else None
			
			if self.global_db.file_banned(file_md5):
				raise BannedFile
			if (not self.config.allow_duplicate_files) and self.file_exists(file_md5):
				raise DuplicateFile
		
			q = "SELECT 1 FROM posts WHERE thread_id = ? AND ((comment != '' AND comment = ?) OR (file_md5 IS NOT NULL AND file_md5 = ?))"
			if self._select_1(q, thread_id, comment, file_md5):
				raise DoublePost
		
			q = "SELECT 1 FROM posts WHERE ip = ? AND posted > ? LIMIT 1"
			if self._select_1(q, ip, time.time() - self.config.min_seconds_between_posts):
				raise Flooding
			
			q = "SELECT MAX(id)+1 AS next_id FROM posts WHERE thread_id = ?"
			row = self._select_1(q, thread_id)
			if not row.next_id or row.next_id >= self.config.max_thread_posts:
				raise ThreadFull
			
			q = """	INSERT INTO posts (thread_id, id, status, posted, name, tripcode, mod, comment, file_md5, file_meta, ip)
					VALUES (?, ?, 0, inow(), ?, ?, ?, ?, ?, ?, ?)"""
			self._insert(q, thread_id, row.next_id, name, tripcode, mod, comment, file_md5, file_meta, ip)
			
			if bump:
				self.bump(thread_id)
			if lock:
				self._insert("UPDATE threads SET locked = 1 WHERE id = ?", thread_id)
			
			return row.next_id


	def flag_post(self, thread_id, post_id, ip):
		with self._transaction(immediate = True):
			#only flag posts that exist
			q = "SELECT 1 FROM posts WHERE thread_id = ? AND id = ?"
			if self._select_1(q, thread_id, post_id):
				#only flag post if not already reviewed
				q = "SELECT 1 FROM reviewed_posts WHERE thread_id = ? AND post_id = ?"
				if not self._select_1(q, thread_id, post_id):
					q = "INSERT OR IGNORE INTO flags (thread_id, post_id, ip) VALUES (?, ?, ?)"
					self._insert(q, thread_id, post_id, ip)


	def get_post_options_data(self, thread_id, post_id, ip):
		q = """	SELECT format_status(status) as status, thread_id, id AS post_id, tripcode,
				EXISTS (SELECT 1 FROM reviewed_posts WHERE reviewed_posts.thread_id = posts.thread_id AND post_id = posts.id) AS reviewed,
				EXISTS (SELECT 1 FROM flags WHERE flags.thread_id = posts.thread_id AND post_id = posts.id AND flags.ip = ?) AS flagged
				FROM posts
				WHERE thread_id = ? AND id = ?"""
		data = self._select_1(q, ip, thread_id, post_id)
		if data is None:
			return None
		data.board_db = self
		return data
	

	def get_post_mod_options_data(self, thread_id, post_id):
		with self._transaction():
			q = "SELECT id, subject, sticky, locked FROM threads WHERE id = ?"
			thread = self._select_1(q, thread_id)
			if thread == None:
				return None
			q = """	SELECT thread_id, id, format_time(posted) as posted, comment, name, tripcode, mod, ip, fqdn(ip) as fqdn, format_status(status) as status, file_md5, file_meta,
					(SELECT COUNT(*) FROM flags WHERE flags.thread_id = posts.thread_id AND post_id = posts.id) AS num_flags,
					EXISTS (SELECT * FROM reviewed_posts WHERE reviewed_posts.thread_id = posts.thread_id AND reviewed_posts.post_id = posts.id) AS reviewed
					FROM `posts` WHERE thread_id = ? AND id = ?"""
			post = self._select_1(q, thread_id, post_id)
			if post is None:
				return None
			if post.file_md5:
				post.file_md5 = util.hex(str(post.file_md5))
				post.file_meta = pickle.loads(str(post.file_meta))
			post.proxy_ip = proxycheck.proxy(self, post.ip) if post.ip else False
			ban = self.global_db.get_ban(post.ip)
			return ObjDict(board_db = self, post = post, thread = thread, ban = ban)
	

	def file_exists(self, file_md5):
		return self._select_1("SELECT 1 FROM posts WHERE file_md5 = ?", file_md5) is not None
	

	def post_exists(self, thread_id, post_id):
		return self._select_1("SELECT 1 FROM posts WHERE thread_id = ? AND id = ?", thread_id, post_id) is not None


	def post_not_deleted(self, thread_id, post_id):
		return self._select_1("SELECT 1 FROM posts WHERE thread_id = ? AND id = ? AND status = 0", thread_id, post_id) is not None

	
	def get_post_file_md5(self, thread_id, post_id):
		data = self._select_1("SELECT file_md5 FROM posts WHERE thread_id = ? AND id = ?", thread_id, post_id)
		return str(data.file_md5) if data else None


	def get_post_tripcode(self, thread_id, post_id):
		q = "SELECT tripcode FROM posts WHERE thread_id = ? AND id = ?"
		row = self._select_1(q, thread_id, post_id)
		return row.tripcode if row else None


	def bump(self, thread_id):
		q = "UPDATE threads SET updated = inow() WHERE id = ?"
		self._insert(q, thread_id)
	
	
	def updated(self):
		q = "SELECT MAX(posted) as updated FROM posts"
		return self._select_1(q).updated or 0
	
	
	def anonymize_old_posts(self):
		q = "UPDATE posts SET ip = NULL WHERE posted < ?"
		self._insert(q, time.time() - self.config.hours_to_keep_post_ips*60*60)


	def get_flags(self):
		q = """	SELECT thread_id, post_id, COUNT(*) as count
				FROM flags
				WHERE NOT EXISTS (SELECT * FROM reviewed_posts WHERE reviewed_posts.thread_id = flags.thread_id AND reviewed_posts.post_id = flags.post_id)
				GROUP BY thread_id, post_id
				ORDER BY count DESC, thread_id, post_id"""
		return self._select(q)
	

	def get_config_area_overview_data(self):
		with self._transaction():
			# get a list of flagged posts
			q = """	SELECT thread_id, post_id, COUNT(*) AS count
					FROM flags
					WHERE NOT EXISTS (SELECT * FROM reviewed_posts WHERE reviewed_posts.thread_id = flags.thread_id AND reviewed_posts.post_id = flags.post_id)
					GROUP BY thread_id, post_id
					ORDER BY count DESC, thread_id, post_id"""
			flags = self._select(q)
			# get a list of reviewed posts
			q = """ SELECT posts.thread_id, post_id, format_time(time) AS time, format_status(status) AS status
					FROM reviewed_posts, posts
					WHERE posts.thread_id = reviewed_posts.thread_id AND posts.id = post_id
					ORDER BY reviewed_posts.time DESC"""
			reviewed_posts = self._select(q)
			return ObjDict(flags = flags, reviewed_posts = reviewed_posts, stats = self.get_stats())


	def get_config_area_posts_by_ip_data(self, ip):
		with self._transaction():
			#get list of threads
			q = """	SELECT *,
					(SELECT COUNT(*) FROM posts WHERE thread_id = threads.id AND ip = ?) AS num_ip_posts,
					(SELECT COUNT(*) FROM posts WHERE thread_id = threads.id AND ip = ? AND file_meta IS NOT NULL) AS num_ip_files,
					(SELECT MAX(posted) FROM posts WHERE thread_id = threads.id AND ip = ?) AS last_ip_post_time
					FROM threads
					WHERE num_ip_posts > 0
					ORDER BY last_ip_post_time DESC"""
			threads = self._select(q, ip, ip, ip)
			total_ip_posts = sum(t.num_ip_posts for t in threads)
			for thread in threads:
				q = "SELECT id FROM posts WHERE thread_id = ? AND ip = ? ORDER BY id"
				thread.ip_posts = [r.id for r in self._select(q, thread.id, ip)]
			return ObjDict(total_ip_posts = total_ip_posts, threads = threads)


	def get_stats(self):
		q = """SELECT
			(SELECT COUNT(*) FROM threads) AS num_threads,
			(SELECT COUNT(*) FROM posts) AS num_posts,
			(SELECT COUNT(*) FROM posts WHERE file_md5 IS NOT NULL) AS num_files"""
		r = self._select_1(q)
		r.uploaded_files_size = util.get_dir_size(self.file_dir())
		return r
