"""
handles the saving/loading of cached pages.
cached pages are index, board index, thread pages and list pages
pages for mods are not cached.
"""

from __future__ import with_statement

import os, time, threading, glob, urllib, tempfile, gzip
from os.path import join

import common


MAX_MINUTES_OF_LIFE = 5			# max time to cache pages for - needs to be lowish for torrent scrapes and time difference display

_last_clean = 0
_lock = threading.Lock()


def purge():
	""" removes all files from the cache """
	with _lock:
		for f in glob.glob(join(common.CACHE_DIR, '*')) + glob.glob(join(common.CACHE_DIR, 'gzip/*')):
			if os.path.isfile(f):
				os.unlink(f)


def clean():
	""" if MAX_MINUTES_OF_LIFE have elapsed since the last cleanup(), clear the cache of items older than MAX_MINUTES_OF_LIFE """
	global _last_clean
	with _lock:
		if (time.time() - _last_clean) > MAX_MINUTES_OF_LIFE*60:
			
			_last_clean = time.time()
			max_mtime = time.time() - MAX_MINUTES_OF_LIFE*60
			
			for f in os.listdir(common.CACHE_DIR):
				f = join(common.CACHE_DIR, f)
				if os.path.isfile(f) and os.path.getmtime(f) < max_mtime:
					os.unlink(f)


def load(name):
	""" return a utf-8 encoded byte-string from the cache that has the specified name, or None if it doesnt exist """
	with _lock:
		try:
			with open(join(common.CACHE_DIR, name), 'rb') as f:
				return f.read()
		except:
			return None


def load_gzip(name):
	""" return a utf-8 encoded, gzipped, byte-string from the cache that has the specified name, or None if it doesnt exist """
	with _lock:
		try:
			with open(join(common.CACHE_DIR, 'gzip/' + name), 'rb') as f:
				return f.read()
		except:
			return None


def store(name, page):
	""" store page in the cache. page must be a utf-8 encoded byte-string """
	with _lock:
		# should be atomic - so put it in a temp file then rename it
		
		fd, path = tempfile.mkstemp(prefix = '*', dir = common.CACHE_DIR)
		with os.fdopen(fd, 'w') as f:
			f.write(page)
		os.rename(path, join(common.CACHE_DIR, name))
		
		fd, path = tempfile.mkstemp(prefix = '*', dir = common.CACHE_DIR)
		with os.fdopen(fd, 'w') as f:
			gzf = gzip.GzipFile(mode = 'wb', compresslevel = 3, fileobj = f)
			gzf.write(page)
			gzf.close()
		os.rename(path, join(common.CACHE_DIR, 'gzip/' + name))



def size():
	return len([f for f in os.listdir(common.CACHE_DIR) if os.path.isfile(f)])


def thread_modified(board_db, thread_id):
	""" clears the cache of any items related to the specified thread """
	with _lock:
		
		exacts = (
			urllib.quote('/%s' % board_db.dir, ''),
			urllib.quote('/%s/%d' % (board_db.dir, thread_id), ''),
			urllib.quote('/%s/list' % board_db.dir, ''),
			urllib.quote('/', '')
		)
		board_page = urllib.quote('/%s/page' % board_db.dir, '')
		
		for f in os.listdir(common.CACHE_DIR):
			if f in exacts or f.startswith(board_page):
				os.unlink(join(common.CACHE_DIR, f))
		
		for f in os.listdir(join(common.CACHE_DIR, 'gzip')):
			if f in exacts or f.startswith(board_page):
				os.unlink(join(common.CACHE_DIR, 'gzip/' + f))
