""" stuff to do with validating, saving, deleting etc uploaded files """

from __future__ import with_statement

import os, shutil, re, urllib, socket, threading, time, logging, Queue

import common, util, util.bencode as bencode


SECONDS_BETWEEN_TORRENT_SCRAPES = 60*15

MAX_TORRENT_TITLE_LENGTH = 64
MAX_TORRENT_COMMENT_LENGTH = 1024
MAX_TORRENT_CREATED_BY_LENGTH = 64
MAX_TORRENT_TRACKER_LENGTH = 256
MAX_TORRENT_TRACKERS = 50
MAX_TORRENT_FILE_NAME_LENGTH = 256
MAX_TORRENT_FILES = 200


_SCRAPE_REGEX = re.compile('(?<=/)announce(?=[^/]*$)')


_scrape_queue = Queue.Queue(0)		#of (info_hash, url)
_scrape_cache = {}					#info_hash => (time, TorrentScrape instance or None for fetch failure)
_scrape_lock = threading.Lock()


def _sh(s):
	return s.replace(r"'", r"\'")


def _scrape_torrent(info_hash, url):
	""" return TorrentScrape instance, or None. """
	_scrape_queue.put((info_hash, url))
	with _scrape_lock:
		x = _scrape_cache.get(info_hash)
		return x[1] if x else None


def _scrape_thread():
	""" poll _scrape_queue and update _scrape_cache """
	try:
		while True:
			info_hash, url = _scrape_queue.get()
			
			with _scrape_lock:
				for key in _scrape_cache.keys():
					if _scrape_cache[key][0] < (time.time() - SECONDS_BETWEEN_TORRENT_SCRAPES):
						del _scrape_cache[key]
				scrape_now = info_hash not in _scrape_cache
			
			#do requested scrape?
			if scrape_now:
			
				socket.setdefaulttimeout(3)	#this is not thread safe, 2.6 fixes the issue
				url = '%s?info_hash=%s' % (url, urllib.quote(info_hash))
				try:
					obj = bencode.decode(urllib.urlopen(url).read(64*1024))	#dont read over 64kb to help stop malicious bastards
					f = obj['files'][info_hash]
					seeders = util.psint(f.get('complete'))
					leechers = util.psint(f.get('incomplete'))
					downloads = util.psint(f.get('downloaded'))
					scraped = TorrentScrape(seeders, leechers, downloads)
				except:
					scraped = None
				
				with _scrape_lock:
					_scrape_cache[info_hash] = (time.time(), scraped)
	except:
		#ignore spurious errors from daemon thread termination fuckery (hope there are no actual bugs...)
		pass


t = threading.Thread(target = _scrape_thread)
t.setDaemon(True)
t.start()
del t


class BadFile(Exception):
	""" raised when an uploaded file is no good for some reason. """


class UploadedFile:
	""" subclasses of this are uploaded file meta data classes. changing this to a new style class will bugger up the unpickling of existing objects. """
	
	def __init__(self, name):
		self.name = name
	
	def get_dir(self, post):
		return os.path.join(common.UPLOADED_FILE_DIR, '%s/%d/%d' % post)
	
	def get_path(self, post):
		return os.path.join(self.get_dir(post), self.name)
	
	def check_dirs(self, post):
		try:
			os.makedirs(self.get_dir(post))
		except:	# assume this means dir already exists
			pass

	def get_url(self, board_db, thread_id, post_id):
		return '%s%s/%d/%d/%s' % (common.UPLOADED_FILE_URL, board_db.dir, thread_id, post_id, urllib.quote(self.name))
	
	def save(self, temp_file, board_db, post):
		self.check_dirs(post)
		shutil.copyfile(temp_file.path, self.get_path(post))
	
	def copy(self, src_post, dest_post):
		self.check_dirs(dest_post)
		shutil.copyfile(self.get_path(src_post), self.get_path(dest_post))
	
	def delete(self, post):
		os.unlink(self.get_path(post))


class ImageFile(UploadedFile):
	""" uploaded jpg/gif/png meta data """
	
	def __init__(self, name, size, thumb_size, img_type):
		UploadedFile.__init__(self, name)
		if name == 'thumb':
			raise ValueError('invalid name - "thumb" is not allowed')
		self.type = img_type
		self.size = size
		self.thumb_size = thumb_size

	def check_dirs(self, post):
		UploadedFile.check_dirs(self, post)
		try:
			os.makedirs(os.path.join(self.get_dir(post), 'thumb'))
		except:	# assume this means dir already exists
			pass
	
	def get_thumb_path(self, post):
		return os.path.join(self.get_dir(post), 'thumb/' + self.name)
	
	def get_thumb_url(self, board_db, thread_id, post_id):
		return '%s%s/%d/%d/thumb/%s' % (common.UPLOADED_FILE_URL, board_db.dir, thread_id, post_id, urllib.quote(self.name))
	
	def save(self, temp_file, board_db, post):
		if board_db.config.clean_images:
			os.system("mogrify -strip '%s'" % _sh(temp_file.path))
		UploadedFile.save(self, temp_file, board_db, post)
		if self.thumb_size:
			x, y = self.thumb_size
			os.system("convert '%s' -thumbnail '%sx%s>' '%s'" % (_sh(temp_file.path), x, y, _sh(self.get_thumb_path(post))))

	def copy(self, src_post, dest_post):
		UploadedFile.copy(self, src_post, dest_post)
		if self.thumb_size:
			self.check_dirs(dest_post)
			shutil.copyfile(self.get_thumb_path(src_post), self.get_thumb_path(dest_post))

	def delete(self, post):
		UploadedFile.delete(self, post)
		if self.thumb_size:
			os.unlink(self.get_thumb_path(post))


class SWFFile(UploadedFile):
	""" uploaded swf meta data """


class TorrentFileEntry:
	""" a file entry inside a torrent file """
	
	def __init__(self, name, length):
		self.name = name
		self.length = length


class TorrentScrape(object):
	""" data scraped from a tracker for a torrent """
	
	def __init__(self, seeders, leechers, downloads):
		self.seeders = seeders
		self.leechers = leechers
		self.downloads = downloads


class TorrentFile(UploadedFile):
	""" uploaded .torrent meta data """
	
	def __init__(self, name, file_name, input):
		UploadedFile.__init__(self, name)
		
		self.info_hash = None
		self.title = util.clamp_str(file_name, MAX_TORRENT_TITLE_LENGTH, '...')
		self.trackers = []
		self.files = []
		self.comment = None
		self.created_by = None
		self.creation_time = None
		
		def get_path(dict):
			if 'path' in dict and isinstance(dict['path'], list):
				path = '/'.join([e.decode('utf-8', 'replace') for e in dict['path'] if isinstance(e, str)])
				return util.clamp_str(path, MAX_TORRENT_FILE_NAME_LENGTH, '...')
			else:
				return None
		
		def get_string(dict, key, max_length):
			if key in dict and isinstance(dict[key], str):
				return util.clamp_str(dict[key].decode('utf-8', 'replace'), max_length, '...')
			else:
				return None
		
		def format(s):
			return util.nl_to_br(util.strip_html(s)) if s else None
		
		d = bencode.decode(input)
		if not isinstance(d, dict):
			raise ValueError('torrent file doesnt contain a dict')
		
		if 'info' in d and isinstance(d['info'], dict):
			info = d['info']
			if 'files' not in info:		#single file only
				name = get_string(info, 'name', MAX_TORRENT_FILE_NAME_LENGTH)
				length = util.psint(info.get('length'))
				if name and length:
					self.files = [TorrentFileEntry(name, length)]
				else:
					raise ValueError('torrent file doesnt contain any files')
			else:						#multiple files
				files = info['files']
				if isinstance(files, list):
					for f in files:
						name = get_path(f)
						length = util.psint(f.get('length'))
						if name and length:
							self.files.append(TorrentFileEntry(name, length))
					self.files = self.files[:MAX_TORRENT_FILES]
		if not self.files:
			raise ValueError('torrent file doesnt contain any files')
		
		if 'announce-list' in d:
			al = d['announce-list']
			if isinstance(al, list) and len(al) and isinstance(al[0], list):
				self.trackers = [util.clamp_str(unicode(t, 'utf-8', 'replace'), MAX_TORRENT_TRACKER_LENGTH, '...') for t in al[0] if isinstance(t, str)]
		elif 'announce' in d and isinstance(d['announce'], str):
			self.trackers = [get_string(d, 'announce', MAX_TORRENT_TRACKER_LENGTH)]
		self.trackers = self.trackers[:MAX_TORRENT_TRACKERS]
		if not self.trackers:
			raise ValueError('Torrent file doesnt contain any trackers')
		
		self.comment = format(get_string(d, 'comment', MAX_TORRENT_COMMENT_LENGTH))
		self.created_by = format(get_string(d, 'created by', MAX_TORRENT_CREATED_BY_LENGTH))
		
		if 'creation date' in d and isinstance(d['creation date'], int):
			t = d['creation date']
			self.creation_time = t if util.valid_time(t) else None
		
		self.info_hash = util.sha1(bencode.encode(d['info']), hex = False)
	
	def get_scrape_url(self):
		for url in self.trackers:
			url, n = _SCRAPE_REGEX.subn('scrape', url)
			if n and url.startswith('http://'):
				return url
		return None
	
	def scrape_data(self):
		url = self.get_scrape_url()
		return _scrape_torrent(self.info_hash, url) if url else None


def calc_thumb_size(board_db, size):
	w, h = size
	mw = mh = board_db.config.max_image_thumb_size
	if w > mw:	#fit to width
		h = h * (mw/float(w))
		w = mw
	if h > mh:	#fit to height
		w = w * (mh/float(h))
		h = mh
	return w, h


def check(file, board_db):
	""" return (file_md5, file_meta, temp_file) or raise BadFile """
	
	temp = util.TempFile()
	md5 = util.copy_file_md5(temp.path, file.file)
	name = util.sanitize_file_name(file.name)
	ext = util.ext(name).lower()
	
	# non type specific checks
	
	if not name or not ext:
		raise BadFile('Invalid file name.')
	
	file_len = os.path.getsize(temp.path)
	if file_len == 0:
		raise BadFile('Zero length file not allowed.')
	if file_len > board_db.config.max_file_length:
		raise BadFile('File too large: maximum allowed size is %dkb.' % (board_db.config.max_file_length/1024))
	
	# type specific parsing/checking
	
	if ext in common.FLASH_FILE_EXTS and board_db.config.allow_flashes:
		return md5, SWFFile(name), temp
	
	elif ext in common.TORRENT_FILE_EXTS and board_db.config.allow_torrents:
		try:
			return md5, TorrentFile(name, file.name, util.file_get_contents(temp.path)), temp
		except:
			logging.exception('error reading torrent file')
			raise BadFile('Error reading torrent file.')
	
	elif (ext in common.IMAGE_FILE_EXTS or file.name.lower().endswith('.jpeg')) and board_db.config.allow_images:
		try:
			f = os.popen("identify '%s'" % _sh(temp.path), 'r')
			res = f.read()
			f.close()
			rs = res.split(' ')
			if res.startswith('identify:') or len(rs) < 3:
				raise BadFile('Unrecognised file format.')
			img_type = rs[1].lower()
			size = map(int, rs[2].split('x'))
	
			if not (file.name.lower().endswith('.' + img_type) or (file.name.lower().endswith('.jpg') and img_type == 'jpeg')):	# avoid ie exploit
				raise BadFile('Extension doesn\'t match file format.')
			if size[0] > board_db.config.max_image_size:
				raise BadFile('Image too wide, maximum allowed width is %dpx.' % board_db.config.max_image_size)
			if size[1] > board_db.config.max_image_size:
				raise BadFile('Image too tall, maximum allowed height is %dpx.' % board_db.config.max_image_size)
	
			if (size[0] >= board_db.config.min_image_size_for_thumbnailing) or (size[1] > board_db.config.min_image_size_for_thumbnailing):
				thumb_size = calc_thumb_size(board_db, size)
			else:
				thumb_size = None
	
			return md5, ImageFile(name, size, thumb_size, img_type), temp
		except BadFile:
			raise
		except:
			logging.exception('unknown error reading image file')
			raise BadFile('Unknown error reading image file.')
	
	raise BadFile('Unrecognised file format.')


def save(temp, board_db, thread_id, post_id, file_meta):
	try:
		file_meta.save(temp, board_db, (board_db.dir, thread_id, post_id))
	except:
		logging.exception('error saving uploaded file')


def delete(board_db, thread_id, post_id, file_meta):
	try:
		file_meta.delete((board_db.dir, thread_id, post_id))
	except:
		logging.exception('error deleting uploaded file')
