""" half compliant http server (stick it behind a proxy) + Request/Response objects + helper stuff """

import re, cgi, urllib, threading, time, socket, Queue, signal, logging, rfc822
import sessions


_REQUEST_LINE_REGEX = re.compile(r'^(?P<method>HEAD|GET|POST) (?P<path>[^ ]*?)(\?(?P<query>[^ ]*))? HTTP/(?P<version>1\.0|1\.1)\r\n$')

_ACCEPT_GZIP_REGEX = re.compile(r'\bgzip\b')

_STANDARD_PAGE = """<html><head><title>Error</title></head><body><h1>%s</h1></body></html>"""
_STANDARD_FORBIDDEN_PAGE = _STANDARD_PAGE % '404 Not Found'
_STANDARD_NOT_FOUND_PAGE = _STANDARD_PAGE % '403 Forbidden'
_STANDARD_ERROR_PAGE = _STANDARD_PAGE % '500 Server Error'

_LOGGER = logging.getLogger(__name__)



class NotFound(Exception):
	""" thrown when a 404 page should be displayed """


class Forbidden(Exception):
	""" thrown when a 503 page should be displayed """


class ListDict(dict):
	""" an extension of dict with a __call__ method for accessing lists easily. used for Request.get, Request.post, Request.cookies and Request.files """
	
	def __init__(self, d = {}):
		self.update(d)
	
	def __call__(self, key, default = None, index = 0):
		if key in self and index < len(self[key]):
			return self[key][index]
		else:
			return default


class RequestHandler(object):
	""" extend this to handle requests. """
	
	def handle(self, request):
		raise NotFound
	
	def notfound(self, request):
		return Response(_STANDARD_NOT_FOUND_PAGE)
	
	def forbidden(self, request):
		return Response(_STANDARD_FORBIDDEN_PAGE)
	
	def error(self, request):
		return Response(_STANDARD_ERROR_PAGE)


class RegexRequestHandler(RequestHandler):

	def __init__(self, urls, trans = {}, trailing_slash_magic = True):
		"""
			urls is a tuple sequence: [(regex, func[, extra]), ...]
			
			regex is a regex-string.
			func is a function with a request argument and some keyword args matching named groups in the regex-string.
			extra is a dict of extra keyword arguments to transform and pass to func
		"""
		self.urls = [(re.compile(e[0]), e[1], (e[2] if len(e)>2 else {})) for e in urls]
		self.trans = trans
		self.trailing_slash_magic = trailing_slash_magic
	
	
	def handle(self, request, *extras):
		path = request.path[1:]
		
		for url in self.urls:
			mo = url[0].match(path)
			if mo:
				d = mo.groupdict()
				# transform matched groups
				for k in d:
					if k in self.trans:
						d[k] = self.trans[k](request, *(extras + (d[k],)))
				# add extra groups
				d.update(url[2])
				return url[1](request, *extras, **d)
				
		if self.trailing_slash_magic and request.method in ('GET', 'HEAD'):
			path = path.rstrip('/') if path.endswith('/') else (path + '/')
			for url in self.urls:
				if url[0].match(path):
					return RedirectResponse('/' + path)
		
		raise NotFound


class Request(object):

	def __init__(self, server, sock, addr):
		file = sock.makefile()
		
		rlmo = _REQUEST_LINE_REGEX.match(file.readline())
		if not rlmo:
			raise ValueError
	
		self.method = rlmo.group('method')
		self.path = urllib.unquote_plus(rlmo.group('path')).decode('utf-8', 'replace')
		self.query = rlmo.group('query') or ''
		self.version = rlmo.group('version')
		
		self.headers = {}
		for line in rfc822.Message(file).headers:
			line = line.partition(':')
			name = line[0].strip().lower()
			value = line[2].strip()
			self.headers[name] = value
		
		if not server.proxies or not self['x-forwarded-host']:
			self.server_name = (self['host'] or 'localhost')
		else:
			self.server_name = self['x-forwarded-host'].split(', ')[-server.proxies]
		
		if not server.proxies or not self['x-forwarded-for']:
			self.remote_host = addr[0]
		else:
			self.remote_host = self['x-forwarded-for'].split(', ')[-server.proxies]
		
		if 'content-length' not in self.headers:
			self.headers['content-length'] = '0'
		if 'content-type' not in self.headers:
			self.headers['content-type'] = 'application/octet-stream'
		
		# parse GET data
		self.get = ListDict(cgi.parse_qs(self.query, keep_blank_values = True))
		for k in self.get:
			self.get[k] = [e.decode('utf-8', 'replace') for e in self.get[k]]
		
		# parse POST data
		self.post = ListDict()
		self.files = ListDict()
		ct = cgi.parse_header(self['content-type'])
		
		if ct[0] == 'application/x-www-form-urlencoded':
			self.post.update(cgi.parse_qs(file.read(int(self.headers['content-length'])), keep_blank_values = True))
			for k in self.post:
				self.post[k] = [e.decode('utf-8', 'replace') for e in self.post[k]]
		
		elif ct[0] == 'multipart/form-data':
			cgi_headers = dict(
				REQUEST_METHOD = self.method,
				CONTENT_TYPE = self['content-type'],
				CONTENT_LENGTH =  self['content-length'],
			)
			fs = cgi.FieldStorage(fp = file, environ = cgi_headers, keep_blank_values = True)
			for k in fs:
				val = fs[k]
				if not isinstance(val, list) and val.name is not None:
					if val.filename:
						self.files[val.name.decode('utf-8', 'replace')] = [UploadedFile(val.filename.decode('utf-8', 'replace'), val.file)]
					else:
						self.post[val.name.decode('utf-8', 'replace')] = [val.value.decode('utf-8', 'replace')]
		
		# parse cookies
		self.cookies = ListDict()
		if self['cookie']:
			for c in self['cookie'].split(';'):
				nv = c.partition('=')
				name = urllib.unquote_plus(nv[0].strip()).decode('utf-8', 'replace')
				value = urllib.unquote_plus(nv[2].strip()).decode('utf-8', 'replace')
				if name in self.cookies:
					self.cookies[name].append(value)
				else:
					self.cookies[name] = [value]
		
		self.session_id = sessions.get_valid_id(self.cookies('sid'))
		self.session = sessions.load(self.session_id)
		self.time = time.time()
	
	def __getitem__(self, key):
		""" return the header called key.lower(), or None if it doesnt exist """
		return self.headers.get(key.lower())
	
	def is_ajax(self):
		""" return if a "X-Requested-With" http header with value "XMLHttpRequest" was sent with this request """
		return self['x-requested-with'] == 'XMLHttpRequest'
	
	def local_referer(self):
		rf = (self['referer'] or '').partition('://')[2]
		return rf.startswith(self.server_name + '/') or (rf == self.server_name)
	
	def accepts_gzip(self):
		return _ACCEPT_GZIP_REGEX.match(self['accept-encoding'] or '') is not None

	def _store_session(self):
		sessions.store(self.session_id, self.session)
	
	def _set_sid_cookie(self, response):
		if self.cookies('sid') != self.session_id:
			response.set_cookie('sid', self.session_id, path = '/')


class Response(object):
	def __init__(self, body = '', type = 'text/html; charset=utf-8', status = 200, headers = {}):
		""" body must be a str or a unicode. if it is a unicode it will be utf-8 encoded and stored as body """
		self.status = status
		self.headers = headers
		self.cookies = {}	#name => Cookie
		if isinstance(body, unicode):
			body = body.encode('utf-8', 'replace')
		self.body = body
		self['Content-Type'] = type
		self['Content-Length'] = str(len(body))
		self['Date'] = rfc822.formatdate(time.time())
		self['Expires'] = self['Date']						# dont allow caching by default
	
	def __setitem__(self, key, value):
		""" set the response header with name key to value """
		self.headers[key] = value
	
	def __getitem__(self, key):
		""" return the response header with name key """
		return self.headers[key]
	
	def expire(self, minutes = 30):
		""" set headers indicating this response can be cached for specified minutes """
		self['Expires'] = rfc822.formatdate(time.time() + minutes*60)
	
	def set_cookie(self, name, value, max_age = None, expires = None, path = None, domain = None, secure = False, http_only = False, raw = False):
		if expires is None and max_age is not None:
			expires = time.time() + max_age
		self.cookies[name] = Cookie(name, value, expires, path, domain, secure, http_only, raw)
	
	def delete_cookie(self, name, path = None, domain = None, secure = False, http_only = False):
		self.set_cookie(name, '', -24*60*60, None, path, domain, secure, http_only)
	
	def _write(self, sock, request):
		file = sock.makefile()
		file.write('HTTP/1.0 %d\r\n' % self.status)
		for k, v in self.headers.items():
			if ('\n' in k) or (':' in k) or ('\n' in v):
				raise ValueError('invalid header')
			file.write(k)
			file.write(': ')
			file.write(str(v))
			file.write('\r\n')
		for name, cookie in self.cookies.items():
			file.write('set-cookie: ')
			file.write(urllib.quote(name.encode('utf-8', 'replace')))
			file.write('=')
			if cookie.raw:
				file.write(cookie.value.encode('utf-8', 'replace'))
			else:
				file.write(urllib.quote(cookie.value.encode('utf-8', 'replace')))
			if cookie.expires is not None:
				file.write('; expires=')
				file.write(time.strftime('%a, %d-%b-%Y %H:%M:%S GMT', time.gmtime(cookie.expires)))
			if cookie.path is not None:
				file.write('; path=')
				file.write(cookie.path)
			if cookie.domain is not None:
				file.write('; domain=')
				file.write(cookie.domain)
			if cookie.secure:
				file.write('; secure')
			if cookie.http_only:
				file.write('; http_only')
			file.write('\r\n')	
		file.write('\r\n')
		file.write(self.body)
	

class RedirectResponse(Response):
	def __init__(self, path):
		Response.__init__(self, status = 302, headers = {'location': path})


class UploadedFile(object):
	def __init__(self, name, file):
		self.name = name
		self.file = file


class Cookie(object):
	def __init__(self, name, value, expires, path, domain, secure, http_only, raw):
		self.name = name
		self.value = value
		self.expires = expires
		self.path = path
		self.domain = domain
		self.secure = secure
		self.http_only = http_only
		self.raw = raw


class _HTTPServerThread(threading.Thread):
	
	def __init__(self, server):
		threading.Thread.__init__(self)
		self.server = server
	
	def run(self):
		while True:
			con = self.server._queue.get()
			if con is None:
				break
			else:
				sock, addr = con
				sock.settimeout(None)
				try:
					request = Request(self.server, sock, addr)
					response = self.server._send_to_handler(request)
					request._store_session()
					request._set_sid_cookie(response)
					response._write(sock, request)
					sessions.clean()
				except:
					_LOGGER.exception('exception handling connection:')
				sock.close()


class HTTPServer(object):
	def __init__(self, addr, request_handler, proxies = 0, num_threads = 10):
		"""
			addr is a tuple (host, port) to listen on.
			proxies is an integer specifying how many trusted proxies are in front of this server.
			request_handler is a subclass of RequestHandler.
		"""
		self.addr = addr
		self.proxies = proxies
		self.num_threads = num_threads
		self.request_handler = request_handler
		self._queue = Queue.Queue(2)
		self._rebind_queue = Queue.Queue(2)
		
		quit = False
		def set_quit(number, frame):
			quit = True
		signal.signal(signal.SIGINT, set_quit)
		signal.signal(signal.SIGTERM, set_quit)
		
		sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
		sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
		sock.settimeout(None)
		sock.bind(addr)
		sock.listen(5)
		
		threads = []
		for i in xrange(num_threads):
			threads.append(_HTTPServerThread(self))
			threads[-1].start()
		
		while not quit:
			try:
				self._queue.put(sock.accept())
			except:
				quit = True
		
		sock.close()
		for thread in threads:
			self._queue.put(None)
		for thread in threads:
			thread.join()
	
	def _send_to_handler(self, request):
		try:
			return self.request_handler.handle(request)
		except NotFound:
			try:
				return self.request_handler.notfound(request)
			except:
				_LOGGER.exception('exception in not found handler:')
				return Response(_STANDARD_ERROR_PAGE, status = 500)
		except Forbidden:
			try:
				return self.request_handler.forbidden(request)
			except:
				_LOGGER.exception('exception in forbidden handler:')
				return Response(_STANDARD_ERROR_PAGE, status = 500)
		except Exception, e:
			try:
				return self.request_handler.error(request)
			except:
				_LOGGER.exception('exception in error handler:')
				return Response(_STANDARD_ERROR_PAGE, status = 500)


if __name__ == '__main__':
	class TestRequestHandler(RequestHandler):
		def handle(self, request):
			return Response('<html><head><title></title></head><body>hello.jpg</body></html>')
	logging.basicConfig()
	HTTPServer(('localhost', 8000), TestRequestHandler())
