""" utility module... """

import re, hashlib, os, os.path, random, cStringIO, StringIO, time, tempfile, socket


_HEX_REGEX = re.compile('^[a-zA-Z0-9]*$')

_AMPM_HOURS = ['12:%.2dam'] \
	+ ['%d:%%.2dam' % i for i in xrange(1, 12)] \
	+ ['12:%.2dpm'] \
	+ ['%d:%%.2dpm' % i for i in xrange(1, 12)]

_TIME_REGEX = re.compile(r'^(?P<number>\d{1,8}(\.\d{1,8})?)( )?(?P<unit>(second|minute|hour|day|week|month|year)s?)$')
_TIME_SECONDS = {'second': 1, 'minute': 60, 'hour': 60*60, 'day': 60*60*24, 'week': 60*60*24*7, 'month': 60*60*24*30, 'year': 60*60*24*365}


class Timer(object):
	""" used to calc the difference in time between two points in time """
	
	def __init__(self, start):
		self.start = start

	def calc(self):
		return time.time() - self.start


class TempFile(object):
	""" holds a file name that will be unlinked when the instance is garbage collected """
	
	def __init__(self):
		self.path = tempfile.mkstemp()[1]

	def __del__(self):
		try:
			os.unlink(self.path)
		except:
			pass


class ObjDict(dict):
	def __init__(self, copy = {}, **d):
		self.update(copy)
		self.update(d)

	def __getattr__(self, name):
		return self[name]
	
	def __setattr__(self, name, value):
		self[name] = value


def is_hex(s):
	return _HEX_REGEX.match(s) is not None


def hex(input):
	a = '0123456789ABCDEF'
	return ''.join([a[ord(c)>>4] + a[ord(c)&0xf] for c in input])


def unhex(input):
	out = ''
	i = 0
	while i+1 < len(input):
		out += chr(int(input[i:i+2], 16))
		i += 2
	return out


def valid_ip(input):
	try:
		socket.inet_aton(input)
		return True
	except:
		return False


def rnd_ip():
	r = lambda: random.randint(0, 255)
	return '%d.%d.%d.%d' % (r(), r(), r(), r())


def pad_ip(input):
	""" pads an ip so it is of the form xxx.xxx.xxx.xxx """
	return '.'.join([x.rjust(3, '0') for x in input.split('.')])


def valid_time(timestamp):
	try:
		time.gmtime(timestamp)
		return True
	except:
		return False


def sint(input, default = 0):
	""" "safe" int cast """
	try:
		return int(input)
	except:
		return default	


def psint(input, default = 0):
	""" positive "safe" int cast """
	return abs(sint(input, default))


def sfloat(input, default = 0):
	""" "safe" float cast """
	try:
		return float(input)
	except:
		return default


def psfloat(input, default = 0):
	""" positive "safe" float cast """
	return abs(sfloat(input, default))


def md5(input, hex = True):
	""" one-call md5 """
	if isinstance(input, unicode):
		input = input.encode('utf-8', 'replace')
	h = hashlib.md5()
	h.update(input)
	return h.hexdigest() if hex else h.digest()


def sha1(input, hex = True):
	""" one-call sha1 """
	if isinstance(input, unicode):
		input = input.encode('utf-8', 'replace')
	h = hashlib.sha1()
	h.update(input)
	return h.hexdigest() if hex else h.digest()


def rand_str(len, pos = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'):
	return ''.join([random.choice(pos) for i in xrange(len)])


def clamp_str(input, max_len, app = ''):
	if not isinstance(input, basestring):
		return ''
	if (len(input) <= max_len):
		return input
	elif len(app) < max_len:
		return input[:max_len - len(app)] + app
	else:
		return app[:max_len]


def clamp_to_line(input):
	a = input.splitlines()
	return '' if not a else a[0]


def limit(input, *limits):
	if input in limits:
		return input
	return limits[0]


def get_limited(dict, key, limits, index = 0):
	if key in dict:
		return limit(dict[key], limits, index)
	return limits[index]


def clamp(val, min, max):
	""" sint's val, then clamps it between min and max """
	val = sint(val)
	if val > max:
		return max
	if val < min:
		return min
	return val


def pad(seq, length, value = ''):
	""" pads a list with empty values so it is at least length in size """
	return seq if (len(seq) >= length) else list(seq) + [value]*(length-len(seq))


def tuplize(x):
	return x if isinstance(x, tuple) else (x,)


def file_get_contents(file):
	""" one call file read """
	f = open(file, 'rb')
	c = f.read()
	f.close()
	return c


def file_set_contents(file, contents):
	""" one call file write """
	f = open(file, 'wb')
	f.write(contents)
	f.close()


def ext(fname):
	i = fname.rfind('.')
	return '' if i == -1 else fname[i+1:]


def quiet_unlink(file):
	try:
		os.unlink(file)
	except:
		pass


def copy_file_md5(path, file):
	""" copies data from file object file into the physical file at path. returns the md5 of the data copied """
	h = hashlib.md5()
	rt = open(path, 'wb')
	c = file.read(4096)
	while c:
		h.update(c)
		rt.write(c)
		c = file.read(4096)
	file.close()
	rt.close()
	return h.digest()


def get_dir_size(path):
	""" recursive directory size """
	size = 0
	for p, d, files in os.walk(path):
		for f in files:
			size += os.path.getsize(os.path.join(p, f))
	return size


def rmdirs(path):
	""" recursive directory delete """
	for root, dirs, files in os.walk(path, topdown = False):
		for file in files:
			os.unlink(os.path.join(root, file))
		for dir in dirs:
			os.rmdir(os.path.join(root, dir))


def sanitize_file_name(input):
	if input.endswith('~'):
		input = input[:-1]
	#prevent more than one extension being present, to workaround a apache/php mod_mime issue
	input = re.sub(r'\.(?=.*?\.)', '_', input)
	#lower the extension
	input = re.sub(r'(?<=\.).+', lambda s: s.group().lower(), input)
	return input[-128:]


def gzip_encode(bin_str):
	""" one call gzip encode """
	import gzip
	f = cStringIO.StringIO()
	g = gzip.GzipFile(mode = 'wb', compresslevel = 5, fileobj = f)
	g.write(bin_str)
	g.close()
	return f.getvalue()


def gzip_decode(bin_str):
	""" one call gzip decode """
	import gzip
	f = cStringIO.StringIO(bin_str)
	g = gzip.GzipFile(mode = 'rb', fileobj = f)
	s = g.read()
	g.close()
	return s


def format_file_size(size):
	size = float(size)
	if size < 1024:
		return '%db' % size
	size = size/1024	#now in kb
	if size < 1024:
		return '%dkb' % size
	size = size/1024	#now in mb
	if size < 1024:
		return '%dmb' % size
	size = size/1024	#now in gb
	return '%dgb' % size


def ampmtime(struct_time):
	return _AMPM_HOURS[struct_time.tm_hour] % struct_time.tm_min


def format_time(timestamp = None, gmt_offset = 0):
	""" formats timestamp, giving inaccurate information (to allow for caching) about how far ago it was if it is in the past """
	
	now = int(time.time())
	if timestamp is None:
		timestamp = now
	
	now_st = time.gmtime(now + gmt_offset*60*60)
	then_st = time.gmtime(timestamp + gmt_offset*60*60)
	apt = ampmtime(then_st)
	
	if (timestamp <= now) and (now_st.tm_year == then_st.tm_year):
		if now_st.tm_yday == then_st.tm_yday:
			min_dif = (now-timestamp)/60
			if min_dif < 2:
				return apt + ' - about a minute ago'
			elif min_dif < 60:
				return apt + ' - about %d minutes ago' % min_dif
			hr_dif = min_dif/60
			if hr_dif == 1:
				return apt + ' - about an hour ago'
			elif hr_dif < 24:
				return apt + ' - about %d hours ago' % hr_dif
		if now_st.tm_yday-1 == then_st.tm_yday:
			return time.strftime('%b %d - Yesterday - ', then_st) + apt
		else:
			return time.strftime('%b %d ', then_st) + apt
	
	return time.strftime('%b %d %Y ', then_st) + apt


def format_interval(time, digits = 0):
	""" format time (interval in seconds) into a readable yet inaccurate format, to digits decimal places. """
	d = '%%.%df ' % digits
	if time < 60:
		return (d+'second') % time + ('s' if time != 1 else '')
	time = time/60.0	#now in minutes
	if time < 60:
		return (d+'minute') % time + ('s' if time != 1 else '')
	time = time/60.0	#now in hours
	if time < 24:
		return (d+'hour') % time + ('s' if time != 1 else '')
	time = time/24.0	#now in days
	if time < 30:
		return (d+'day') % time + ('s' if time != 1 else '')
	day_time = time
	time = time/30.0	#now in months
	if time < 12:
		return (d+'month') % time + ('s' if time != 1 else '')
	time = day_time/365.0	#now in years
	return (d+'year') % time + ('s' if time != 1 else '')


def parse_interval(time):
	""" parse time (interval in seconds) in the format returned by format_interval, return time in int seconds or None with invalid input """
	mo = _TIME_REGEX.match(time)
	if mo is None:
		return None
	else:
		dict = mo.groupdict()
		number = float(dict['number'])
		unit = dict['unit']
		if unit[-1] == 's':
			unit = unit[:-1]
		return int(number * _TIME_SECONDS[unit])


def re_msplit(regex, input):
	""" a regex split that returns a list of interspersed strings and match objects """
	output = []
	start = 0
	for mo in regex.finditer(input):
		if mo.start() > start:
			output.append(input[start:mo.start()])
		output.append(mo)
		start = mo.end()
	if start != len(input):
		output.append(input[start:])
	return output


def html_escape(input):
	if isinstance(input, (int, long, float, bool)):
		return str(input)
	elif isinstance(input, basestring):
		return input.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;').replace('\'', '&#039;')
	else:
		raise TypeError('input not int, long, float, boolean, str or unicode: %s' % input)


def html_escape_nl(input):
	return html_escape(input).replace('\r\n', '<br>')


def strip_html(input):
	""" return input with html tags stripped """
	return re.sub(r'<.*?>', '', input)


def nl_to_br(input):
	""" return input with all occurences of "\r\n" replaced by "<br>" """
	return input.replace('\r\n', '<br>')


def read_checked(file, rlen = 1):
	r = file.read(rlen)
	if len(r) < rlen:
		raise EOFError
	return r
