""" every function in this module returns a specific page """

from __future__ import with_statement

import logging, re, time

import common, util, database, uploaded_files, pagecache, cookies, sessions, tripcodes, proxycheck, util.json as json, util.http as http

from util import get_limited, clamp_str, limit, sint, psint, valid_ip, html_escape, ObjDict
from util.http import Request, Response, RedirectResponse, NotFound, Forbidden
from common import board_link, thread_link, post_link, render_to_response, render_to_string, check_age, cache, mod_login, local_referer


CONFIG_AREA_SECTIONS = ('overview', 'mods', 'modlog', 'bans', 'file-bans', 'posts-by-ip', 'existing-boards', 'new-board', 'change-password')


def boards_api(request, global_db):
	data = global_db.get_boards_api_data(request.server_name)
	return Response(json.encode(data), 'application/json; charset=utf-8')


def confirm_overage(request, global_db):
	response = RedirectResponse(request['referer'] or '/')
	cookies.set_overage(response)
	return response


@check_age
@cache
def index(request, global_db):
	if len(global_db.boards) == 1:
		return RedirectResponse('/%s/' % global_db.boards[0].dir)
	else:
		data = global_db.get_index_data()
		return render_to_response(request, global_db, 'index.html', data)


@check_age
@cache
def view_board(request, global_db, board_db, page):
	data = board_db.get_view_board_data(page)
	if data is None:
		raise NotFound
	return render_to_response(request, global_db, 'view_board.html', data)


@check_age
@cache
def view_thread(request, global_db, board_db, id):
	data = board_db.get_view_thread_data(id)
	if not data:
		raise NotFound
	return render_to_response(request, global_db, 'view_thread.html', data)


@check_age
@cache
def list(request, global_db, board_db):
	data = board_db.get_list_data()
	return render_to_response(request, global_db, 'list.html', data)


def fetch_post(request, global_db, board_db, thread_id, post_id):
	data = board_db.get_post_data(thread_id, post_id)
	if not data:
		raise NotFound
	response = render_to_response(request, global_db, 'fetch_post.html', data)
	if sessions.get_mod(request) is None:
		response.expire(5)
	return response


def fetch_posts(request, global_db, board_db, thread_id, id_start, id_end):
	if id_end is None:
		id_end = id_start+1
	data = board_db.get_posts_data(thread_id, id_start, id_end)
	if not data:
		raise NotFound
	response = render_to_response(request, global_db, 'fetch_posts.html', data)
	if sessions.get_mod(request) is None:
		response.expire(5)
	return response


@check_age
def new_thread(request, global_db, board_db):
	
	if request.method == 'GET':
		new_thread = ObjDict(database.NEW_THREAD)
		if 't' in request.get and 'p' in request.get:
			new_thread.forked_thread_id = psint(request.get('t'))
			new_thread.forked_post_id = psint(request.get('p'))
		return render_to_response(request, global_db, 'new_thread.html', ObjDict(board_db = board_db, new_thread = new_thread))
	
	elif request.method != 'POST' or not request.local_referer():
		raise Forbidden
	
	# gather input
	sent = ObjDict(
		name = request.post('name', '').strip()[:board_db.config.max_name_length],
		tripcode = request.post('tripcode', ''),
		subject = request.post('subject', '').strip()[:board_db.config.max_subject_length],
		comment = request.post('comment', '').strip(),
		file = request.files('file'),
		forked_thread_id = request.post('forked_thread_id'),
		forked_post_id = request.post('forked_post_id'),
		noko = 'noko' in request.post,
		ip = request.remote_host,
		sticky = 'sticky' in request.post,
		lock = 'lock' in request.post,
		mod = 'mod' in request.post
	)
	
	def set_cookies(response):
		cookies.set_noko(response, sent.noko)
		cookies.set_name(response, sent.name)
		cookies.set_tripcode(response, sent.tripcode)
		return response
	
	def error(msg):
		return set_cookies(render_to_response(request, global_db, 'new_thread.html', ObjDict(board_db = board_db, error_msg = msg, new_thread = sent)))
	
	ban = global_db.get_ban(sent.ip)
	if ban:
		response = render_to_response(request, global_db, 'new_thread.html', ObjDict(board_db = board_db, ban = ban, new_thread = sent))
		set_cookies(response)
		return response
	
	if board_db.config.block_proxies_fully and proxycheck.proxy(board_db, sent.ip):
		return error('Proxies are not allowed to post.')
	
	if sent.file and board_db.config.block_proxies_files and proxycheck.proxy(board_db, sent.ip):
		return error('Proxies are not allowed to post files.')
	
	if not sent.comment and not (board_db.config.allow_no_comment_with_file and sent.file):
		return error('You must write a comment.')
	
	if len(sent.comment) > board_db.config.max_comment_length:
		return error('Comment too long, max allowed length is %d characters.' % board_db.config.max_comment_length)
	
	mod = sessions.get_mod(request)
	
	if mod is None:
		sent.sticky = sent.lock = sent.mod = False
		if sent.file is None and not board_db.config.allow_thread_without_file:
			return error('You must supply a file to start a thread.')
		if not board_db.config.allow_names:
			sent.name == ''
		if not board_db.config.allow_tripcodes:
			sent.tripcode = ''

	#check forked post is valid - if set
	if board_db.config.forking and (sent.forked_thread_id is not None) and (sent.forked_post_id is not None):
		forked_thread_id = psint(sent.forked_thread_id)
		forked_post_id = psint(sent.forked_post_id)
		if forked_post_id == 0:
			return error('You can\'t fork the first post in a thread.')
		if not board_db.post_exists(forked_thread_id, forked_post_id):
			return error('You can\'t fork a non existent post.')
	else:
		forked_thread_id = forked_post_id = None
	
	if sent.file and board_db.files_allowed():
		try:
			file_md5, file_meta, temp_file = uploaded_files.check(sent.file, board_db)
		except uploaded_files.BadFile, e:
			return error(str(e))
	else:
		file_md5, file_meta = None, None
	
	try:
		deleted_threads, thread_id = board_db.add_new_thread(sent.subject, sent.sticky, sent.lock, forked_thread_id, forked_post_id, sent.name or board_db.get_default_name(), tripcodes.make(global_db, sent.tripcode), sent.mod, sent.comment, file_md5, file_meta, sent.ip)
	except database.DoubleThread:
		return error('Your thread was not created - it looks like you are double posting.')
	except database.Flooding:
		return error('Your thread was not created - you are making threads too fast. Slow down!')
	except database.DuplicateFile:
		return error('Your thread was not created - you are posting a file that already exists.')
	except database.BannedFile:
		return error('Your thread was not created - the file you are posting is banned.')
	
	if file_md5:
		uploaded_files.save(temp_file, board_db, thread_id, 0, file_meta)
	
	global_db.delete_expired_bans()
	board_db.anonymize_old_posts()
	pagecache.thread_modified(board_db, thread_id)		#delete caches of pages that should now contain this thread
	if forked_thread_id != None:
		pagecache.thread_modified(board_db, forked_thread_id)
	for thread in deleted_threads:
		pagecache.thread_modified(board_db, thread)
	
	path = '/%s/' % board_db.dir
	if sent.noko:
		path += str(thread_id)
	response = set_cookies(RedirectResponse(path))
	cookies.set_post_verifier(response, board_db, thread_id, 0)
	return response


@check_age
def new_reply(request, global_db, board_db, thread_id):
	
	if request.method == 'GET':
		new_post = ObjDict(database.NEW_POST)
		if 'q' in request.get:
			new_post.comment = '>>%d\r\n\r\n' % psint(request.get('q'))
		return render_to_response(request, global_db, 'new_reply.html', ObjDict(board_db = board_db, thread = ObjDict(id = thread_id), new_post = new_post))
	
	elif request.method != 'POST' or not request.local_referer():
		raise Forbidden
	
	# gather input
	sent = ObjDict(
		name = request.post('name', '').strip()[:board_db.config.max_name_length],
		tripcode = request.post('tripcode', ''),
		comment = request.post('comment', '').strip(),
		file = request.files('file'),
		noko = 'noko' in request.post,
		bump = 'bump' in request.post,
		lock = 'lock' in request.post,
		mod = 'mod' in request.post,
		ip = request.remote_host
	)
	
	data = board_db.get_new_reply_data(thread_id)
	if data is None:
		raise NotFound
	thread = data.thread

	def set_cookies(response):
		cookies.set_noko(response, sent.noko)
		cookies.set_name(response, sent.name)
		cookies.set_tripcode(response, sent.tripcode)
		return response
	
	def error(msg):
		return set_cookies(render_to_response(request, global_db, 'new_reply.html', ObjDict(board_db = board_db, error_msg = msg, thread = thread, new_post = sent)))
	
	ban = global_db.get_ban(sent.ip)
	if ban:
		return set_cookies(render_to_response(request, global_db, 'new_reply.html', ObjDict(board_db = board_db, ban = ban, thread = thread, new_post = sent)))
	
	if board_db.config.block_proxies_fully and proxycheck.proxy(board_db, sent.ip):
		return error('Proxies are not allowed to post.')
	
	if sent.file and board_db.config.block_proxies_files and proxycheck.proxy(board_db, sent.ip):
		return error('Proxies are not allowed to post files.')
	
	if thread.next_id >= board_db.config.max_thread_posts:
		return error('This thread is full.')
	
	if not sent.comment and not (board_db.config.allow_no_comment_with_file and sent.file):
		return error('You must write a comment.')
	
	if len(sent.comment) > board_db.config.max_comment_length:
		return error('Comment too long, max allowed length is %s characters.' % board_db.config.max_comment_length)
	
	mod = sessions.get_mod(request)
	
	if mod is None:
		sent.lock = sent.mod = False
		if thread.locked:
			return error('This thread is locked.')
		if sent.file is None and not board_db.config.allow_reply_without_file:
			return error('You must supply a file with a reply.')
		if not board_db.config.allow_names:
			sent.name == ''
		if not board_db.config.allow_tripcodes:
			sent.tripcode = ''

	if sent.file and board_db.files_allowed():
		try:
			file_md5, file_meta, temp_file = uploaded_files.check(sent.file, board_db)
		except uploaded_files.BadFile, e:
			return error(str(e))
	else:
		file_md5, file_meta = None, None
	
	try:
		post_id = board_db.add_new_reply(thread_id, sent.name or board_db.get_default_name(), tripcodes.make(global_db, sent.tripcode), sent.mod, sent.comment, file_md5, file_meta, sent.ip, sent.bump, sent.lock)
	except database.ThreadFull:
		return error('Your reply was not added - the thread is full.')
	except database.Flooding:
		return error('Your reply was not added - you are posting too fast. Relax!')
	except database.DoublePost:
		return error('Your reply was not added - it looks like you are double posting.')
	except database.DuplicateFile:
		return error('Your reply was not added - you are posting a file that already exists.')
	except database.BannedFile:
		return error('Your reply was not added - the file you are posting is banned.')
	
	if file_md5:
		uploaded_files.save(temp_file, board_db, thread_id, post_id, file_meta)
	
	global_db.delete_expired_bans()
	board_db.anonymize_old_posts()
	pagecache.thread_modified(board_db, thread_id)
	
	if sent.noko:
		path = '/%s/%d#%d' % (board_db.dir, thread_id, post_id)
	else:
		path = '/%s/' % board_db.dir
	
	response = set_cookies(RedirectResponse(path))
	cookies.set_post_verifier(response, board_db, thread_id, post_id)
	return response


@mod_login
def config(request, global_db, section):
	
	mod = sessions.get_mod(request)
	admin = global_db.is_admin(mod)
	
	action = request.post('action') if request.local_referer() else None
	data = ObjDict(section = section, sections = CONFIG_AREA_SECTIONS, msg = None, is_default_mod = global_db.is_default_mod(mod))
	
	
	if section == 'overview':
		data.update(global_db.get_config_area_overview_data())
		return render_to_response(request, global_db, 'config_overview.html', data)
	

	elif section == 'bans':
		
		if action == 'add':
			now = int(time.time())
			ip = request.post('ip', '').strip()
			expires_parsed = util.parse_interval(request.post('expires', '').strip())
			reason = request.post('reason').strip()
			
			if expires_parsed is None:
				data.msg = 'Ban not added: Invalid expiry time.'
			elif now + expires_parsed > (1<<31)-1:
				data.msg = 'Ban not added: Expiry time too far in future.'
			elif not util.valid_ip(ip):
				data.msg = 'Ban not added: Invalid IP'
			else:
				global_db.add_ban(ip, now + expires_parsed, reason)
				global_db.modlog(mod, 'Added ban on %s for reason "%s" expiring on %s.' % (ip, html_escape(reason), util.format_time(now + expires_parsed)))
				data.msg = 'Added Ban on %s.' % ip
		
		elif action == 'delete':
			ip = request.post('ip', '')
			global_db.delete_ban(ip)
			global_db.modlog(mod, 'Deleted ban on %s.' % ip)
			data.msg = 'Deleted Ban on %s.' % ip
		
		filter = request.get('filter', '')
		data.update(global_db.get_config_area_bans_data(filter))
		data.new_ban_ip = request.get('new_ban_ip', '')
		data.filter = filter
		return render_to_response(request, global_db, 'config_bans.html', data)
	

	elif section == 'file-bans':
	
		if action:
			fh = [x for x in request.post('hashes', '').split() if len(x) == 32 and util.is_hex(x)]
			
			if action == 'ban':
				global_db.ban_files(fh)
				msg = 'Added %d hashes to banned file list.' % len(fh)
				global_db.modlog(mod, msg)
				data.msg = msg
			
			elif action == 'unban':
				global_db.unban_files(fh)
				msg = 'Removed %d hashes from banned file list.' % len(fh)
				global_db.modlog(mod, msg)
				data.msg = msg
	
		data.update(global_db.get_config_area_files_data())
		return render_to_response(request, global_db, 'config_file_bans.html', data)
	

	elif section == 'new-board':
	
		if admin and action == 'new_board':
			dir = request.post('dir', '')
			if global_db.make_board(dir):
				global_db.modlog(mod, 'Created board %s.' % board_link(dir))
				global_db = database.reload_thread_globals()
				return RedirectResponse('/%s/config' % dir)
			else:
				data.msg = 'Unable to make new board: "%s".' % dir
	
		return render_to_response(request, global_db, 'config_new_board.html', data)
	

	elif section == 'existing-boards':
	
		if admin and action == 'update_board_config_defaults':
			for key, field in database.Board.CONFIG_FIELDS:
				try:
					global_db.set_board_config_default(key, field.cast(request.post(key)))
				except:
					pass
			msg = 'Updated board config defaults.'
			global_db.modlog(mod, msg)
			data.msg = msg
			pagecache.purge()
			global_db = database.reload_thread_globals()
	
		return render_to_response(request, global_db, 'config_existing_boards.html', data)
	

	elif section == 'mods':
		
		if admin and action == 'new_mod':
			name = request.post('name', '')
			as_admin = 'admin' in request.post
			
			if admin:
				if global_db.get_mod(name) is None:
					if re.match(common.MOD_NAME_REGEX, name):
						password = global_db.add_mod(name, as_admin)
						global_db.modlog(mod, 'Created new mod (%s) "%s".' % ('admin' if as_admin else 'non admin', name))
						data.msg = 'New mod "%s" created, their password is "%s". Tell them to log in and change it.' % (name, password)
						global_db = database.reload_thread_globals()
					else:
						data.msg = 'Did not create new mod: "%s" is not a valid mod name.' % name
				else:
					data.msg = 'Did not create new mod: "%s" already exists.' % name
			else:
				data.msg = 'Did not create new mod: You are not an admin so you do not have the power.'
		
		if admin and action == 'give_admin':
			name = request.post('name', '')
			
			if admin:
				if name in global_db.mods:
					if not global_db.mods[name][1]:
						global_db.set_mod_admin(name, True)
						msg = 'Gave admin to mod "%s".' % name
						global_db.modlog(mod, msg)
						data.msg = msg
						global_db = database.reload_thread_globals()
					else:
						data.msg = 'Did not give admin: mod "%s" is already an admin.' % name
				else:
					data.msg = 'Did not give admin: mod "%s" does not exist.' % name
			else:
				data.msg = 'Did not give admin: You are not an admin yourself so you do not have the power.'
		
		if admin and action == 'take_admin':
			name = request.post('name', '')
			
			if admin:
				if name != mod:
					if name in global_db.mods:
						if global_db.mods[name][1]:
							global_db.set_mod_admin(name, False)
							msg = 'Took admin from mod "%s".' % name
							global_db.modlog(mod, msg)
							data.msg = msg
							global_db = database.reload_thread_globals()
						else:
							data.msg = 'Did not take admin: mod "%s" is not an admin.' % name
					else:
						data.msg = 'Did not take admin: mod "%s" does not exist.' % name
				else:
					data.msg = 'Did not take admin: You may not take admin from yourself.'
			else:
				data.msg = 'Did not take admin: You are not admin yourself so you do not have the power'
		
		if admin and action == 'reset_password':
			name = request.post('name', '')
			
			if admin:
				if name in global_db.mods:
					password = global_db.reset_mod_password(name)
					global_db.modlog(mod, 'Reset password for mod "%s".' % name)
					data.msg = 'Reset password for mod "%s": it is now "%s". Tell them to log in and change it.' % (name, password)
					global_db = database.reload_thread_globals()
				else:
					data.msg = 'Did not reset password: mod "%s" does not exist.' % name
			else:
				data.msg = 'Did not reset password: You are not an admin so you do not have the power.'
		
		if admin and action == 'delete':
			name = request.post('name', '')
			
			if admin:
				if name != mod:
					if name in global_db.mods:
						global_db.delete_mod(name)
						msg = 'Deleted mod "%s".' % name
						global_db.modlog(mod, msg)
						data.msg = msg
						global_db = database.reload_thread_globals()
					else:
						data.msg = 'Did not delete: mod "%s" does not exist.' % name
				else:
					data.msg = 'Did not delete mod "%s": you may not delete yourself.' % name
			else:
				data.msg = 'Did not delete mod "%s": You are not an admin so you do not have the power.'
		
		data.update(global_db.get_config_area_mods_data())
		return render_to_response(request, global_db, 'config_mods.html', data)
	

	elif section == 'modlog':
		data.update(global_db.get_config_area_modlog_data())
		return render_to_response(request, global_db, 'config_modlog.html', data)


	elif section == 'posts-by-ip':
		filter = request.get('filter', '')
		data.update(global_db.get_config_area_posts_by_ip_data(filter))
		data.filter = filter
		return render_to_response(request, global_db, 'config_posts_by_ip.html', data)
	

	elif section == 'change-password':
		
		if action == 'update':
			cp = request.post('current_password') or ''
			np = request.post('new_password') or ''
			cnp = request.post('confirm_new_password') or ''
			
			if global_db.get_mod(mod, cp):
				if np == cnp:
					if np:
						global_db.set_mod_password(mod, np)
						global_db.modlog(mod, 'Changed password.')
						global_db = database.reload_thread_globals()
						data.msg = 'Password changed.'
					else:
						data.msg = 'Password not changed: You entered an empty password, which is not allowed.'
				else:
					data.msg = 'Password not changed: You failed to confirm your new password.'
			else:
				data.msg = 'Password not changed: You failed to input your current password correctly.'
		
		return render_to_response(request, global_db, 'config_change_password.html', data)


@mod_login
def board_config(request, global_db, board_db):
	
	mod = sessions.get_mod(request)
	action = request.post('action')
	
	if action and not global_db.is_admin(mod):
		raise Forbidden
	
	if action == 'update':
		for key, field in database.Board.CONFIG_FIELDS:
			try:
				board_db.set_config(key, field.cast(request.post(key)), ('%s_use_default' % key) in request.post)
			except:
				pass
		global_db.modlog(mod, 'Updated config for board %s.' % board_link(board_db.dir))
		pagecache.purge()
		global_db = database.reload_thread_globals()
		board_db = global_db.get_board(board_db.dir)
	
	elif action == 'delete':
	
		if 'confirm' in request.post:
			common.delete_board(board_db.dir)
			global_db.modlog(mod, 'Deleted board %s.' % board_link(board_db.dir))
			pagecache.purge()
			database.reload_thread_globals()
			return RedirectResponse('/')
	
	data = board_db.get_board_config_data()
	return render_to_response(request, global_db, 'board_config.html', data)


def mod_logout(request, global_db):
	if sessions.get_mod(request) != None and sessions.get_mod_logout_confirm(request) == request.get('c'):
		sessions.delete_mod(request)
		response = RedirectResponse(request['referer'] or '/')
		cookies.delete_mod(response)
		return response
	else:
		raise Forbidden


@check_age
def options(request, global_db, board_db, thread_id, post_id):
	
	action = request.post('action')
	msg = None
	correct_cookie = cookies.check_post_verifier(request, board_db, thread_id, post_id)
	
	if request.method == 'GET' or not action:
		data = board_db.get_post_options_data(thread_id, post_id, request.remote_host)
		if data is None or data.status != 'undeleted':
			raise NotFound
		data.msg = msg
		data.correct_cookie = correct_cookie
		return render_to_response(request, global_db, 'options.html', data)
	
	if request.method == 'POST' and request.local_referer():
	
		if not board_db.post_not_deleted(thread_id, post_id):
			raise NotFound
		
		if action == 'flag':
			board_db.flag_post(thread_id, post_id, request.remote_host)
			msg = 'Post Flagged.'
		
		elif action == 'cookie_delete':
			if cookies.check_post_verifier(request, board_db, thread_id, post_id):
				board_db.delete_post(thread_id, post_id, False, True)
				pagecache.thread_modified(board_db, thread_id)
				msg = 'Post Deleted.'
			else:
				msg = 'Invalid Cookie.'
		
		elif action == 'tripcode_delete':
			trip = board_db.get_post_tripcode(thread_id, post_id)
			if trip and tripcodes.make(global_db, request.post('tripcode', '')) == trip:
				board_db.delete_post(thread_id, post_id, False, True)
				pagecache.thread_modified(board_db, thread_id)
				msg = 'Post Deleted.'
			else:
				msg = 'Invalid Tripcode.'
	
		data = board_db.get_post_options_data(thread_id, post_id, request.remote_host)
		if data is None:
			raise NotFound
		data.msg = msg
		data.correct_cookie = correct_cookie
		return render_to_response(request,global_db,  'options.html', data)
	
	raise Forbidden


@mod_login
def mod_options(request, global_db, board_db, thread_id, post_id):

	mod = sessions.get_mod(request)
	
	if request.method == 'GET':
		data = board_db.get_post_mod_options_data(thread_id, post_id)
		if data is None:
			raise NotFound
		data.msg = None
		return render_to_response(request, global_db, 'mod_options.html', data)
	
	elif request.method == 'POST' and request.local_referer():
		
		action = request.post('action')
		msg = None
		
		if action == 'delete_thread':
			global_db.modlog(mod, 'Deleted %s' % thread_link(board_db.dir, thread_id))
			board_db.delete_thread(thread_id)
			pagecache.thread_modified(board_db, thread_id)
			return RedirectResponse('/%s/' % board_db.dir)
		
		elif action == 'move_thread':
			dest_board_db = global_db.get_board(request.post('dest_board'))
			keep_locked_copy = 'keep_locked_copy' in request.post
			if dest_board_db:
				dest_tid = global_db.move_thread(board_db, thread_id, dest_board_db, keep_locked_copy)
				global_db.modlog(mod, 'Moved %s to %s.' % (thread_link(board_db.dir, thread_id), thread_link(dest_board_db.dir, dest_tid)))
				pagecache.thread_modified(board_db, thread_id)
				if keep_locked_copy:
					msg = 'Moved Thread to %s. Locked this copy.' % thread_link(dest_board_db.dir, dest_tid)
				else:
					return RedirectResponse('/%s/%d' % (dest_board_db.dir, dest_tid))
		
		elif action == 'modify_thread':
			sticky = 'sticky_thread' in request.post
			lock = 'lock_thread' in request.post
			subject = request.post('subject', '').strip()[:board_db.config.max_subject_length]
			
			board_db.edit_thread(thread_id, subject, sticky, lock)
			global_db.modlog(mod, 'Modified thread %s' % thread_link(board_db.dir, thread_id))
			pagecache.thread_modified(board_db, thread_id)
			msg = 'Modified Thread.'
		
		elif action == 'modify_post':
			reviewed = 'reviewed' in request.post
			status = limit(request.post('status'), *database.POST_STATUS)
			name = request.post('name', '')[:board_db.config.max_name_length]
			tripcode = request.post('tripcode', '')
			comment = request.post('comment', '')[:board_db.config.max_comment_length]
			show_mod = 'mod' in request.post
			delete_file = 'delete_file' in request.post
			ban_file = 'ban_file' in request.post
			
			if ban_file:
				file_md5 = board_db.get_post_file_md5(thread_id, post_id)
				if file_md5:
					global_db.add_file_ban(file_md5)
					global_db.modlog(mod, 'Banned file posted with %s - %s.' % (post_link(board_db.dir, thread_id, post_id), util.hex(file_md5)))
			board_db.edit_post(thread_id, post_id, status, name, tripcode, show_mod, comment, delete_file, reviewed)
			global_db.modlog(mod, 'Edited post %s.' % post_link(board_db.dir, thread_id, post_id))
			pagecache.thread_modified(board_db, thread_id)
			msg = 'Modified Post.'
	
		data = board_db.get_post_mod_options_data(thread_id, post_id)
		if data is None:
			raise NotFound
		data.msg = msg
		return render_to_response(request, global_db, 'mod_options.html', data)
	
	raise Forbidden


@mod_login
@local_referer
def mod_post_action(request, global_db, board_db, thread_id, post_id, action):
	mod = sessions.get_mod(request)
	
	if not board_db.post_not_deleted(thread_id, post_id):
		raise NotFound
	
	if action in ('delete-and-ban', 'delete-and-ban-file'):
		ip = board_db.ban_post_ip(thread_id, post_id, int(time.time()) + global_db.board_config_defaults.default_ban_length, global_db.board_config_defaults.default_ban_reason)
		if ip:
			global_db.modlog(mod, 'Banned ip used to post %s - %s' % (post_link(board_db.dir, thread_id, post_id), ip))
	
	if action == 'delete-and-ban-file':
		md5 = board_db.ban_post_file(thread_id, post_id)
		if md5:
			global_db.modlog(mod, 'Banned file posted with %s - %s' % (post_link(board_db.dir, thread_id, post_id), md5))
	
	#all actions delete the post
	global_db.modlog(mod, 'Deleted post %s' % post_link(board_db.dir, thread_id, post_id))
	board_db.delete_post(thread_id, post_id, True, True)
	
	pagecache.thread_modified(board_db, thread_id)
	return Response('success', 'text/plain; charset=utf-8')


class Handler(http.RegexRequestHandler):
	
	def __init__(self):
		
		def board_db_trans(request, global_db, dir):
			board_db = global_db.get_board(dir)
			if board_db is None:
				raise http.NotFound
			else:
				return board_db
		
		def int_trans(request, global_db, i):
			return int(i)
		
		# funtions to transform arguments to view funtions (groups from the regex) before the are passed to the view functions
		trans = dict(
			board_db = board_db_trans,
			page = int_trans,
			thread_id = int_trans,
			post_id = int_trans,
			id_start = int_trans,
			id_end = int_trans,
			id = int_trans,
		)
		
		urls = (
			(r'^$', index),
			(r'^api/boards$', boards_api),
			(r'^confirm-overage$', confirm_overage),
			(r'^config$', config, {'section': CONFIG_AREA_SECTIONS[0]}),
			(r'^config-(?P<section>%s)$' % '|'.join(map(re.escape, CONFIG_AREA_SECTIONS)), config),
			(r'^mod-logout$', mod_logout),
			(r'^(?P<board_db>\w+)/config$', board_config),
			(r'^(?P<board_db>\w+)$', view_board,  {'page': 0}),
			(r'^(?P<board_db>\w+)/page(?P<page>[0-9]+)$', view_board),
			(r'^(?P<board_db>\w+)/(?P<id>[0-9]+)$', view_thread),
			(r'^(?P<board_db>\w+)/list$', list),
			(r'^(?P<board_db>\w+)/new-thread$', new_thread),
			(r'^(?P<board_db>\w+)/(?P<thread_id>[0-9]+)/reply$', new_reply),
			(r'^(?P<board_db>\w+)/(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)$', fetch_post),
			(r'^(?P<board_db>\w+)/(?P<thread_id>[0-9]+)/(?P<id_start>[0-9]+)-(?P<id_end>[0-9]+)$', fetch_posts),
			(r'^(?P<board_db>\w+)/(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)/options$', options),
			(r'^(?P<board_db>\w+)/(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)/mod-options$', mod_options),
			(r'^(?P<board_db>\w+)/(?P<thread_id>[0-9]+)/(?P<post_id>[0-9]+)/mod-action/(?P<action>%s)$' % '|'.join(map(re.escape, common.MOD_POST_ACTIONS)), mod_post_action),
		)
		
		http.RegexRequestHandler.__init__(self, urls, trans)

	def handle(self, request):
		global_db = database.get_thread_global()
		return http.RegexRequestHandler.handle(self, request, global_db)
	
	def notfound(self, request):
		global_db = database.get_thread_global()
		return http.Response(render_to_string(request, global_db, '404.html'), status = 404)
	
	def forbidden(self, request):
		global_db = database.get_thread_global()
		return http.Response(render_to_string(request, global_db, '403.html'), status = 403)
	
	def error(self, request):
		logging.exception('exception in request handler')
		global_db = database.get_thread_global()
		return http.Response(render_to_string(request, global_db, '500.html'), status = 500)
