"""
handles the saving/loading of cached pages.
cached pages are index, board index, thread pages and list pages
cached pages are stored as files in cache/identity/ and cache/gzip/, with "/" in the file names replaced by "#"
pages for mods are not cached.
"""

from __future__ import with_statement

import os, time, threading, glob, tempfile, gzip

from os.path import join

import common

from util import utf8


MAX_MINUTES_OF_LIFE = 5			# max time to cache pages for - needs to be lowish for torrent scrapes and time difference display

_last_clean = 0
_lock = threading.Lock()


def purge():
	""" removes all files from the cache """
	with _lock:
		for f in glob.glob(join(common.CACHE_DIR, '*', '*')):
			os.unlink(f)


def clean():
	""" if MAX_MINUTES_OF_LIFE have elapsed since the last clean, clear the cache of items older than MAX_MINUTES_OF_LIFE """
	global _last_clean
	with _lock:
		if (time.time() - _last_clean) > MAX_MINUTES_OF_LIFE*60:
			
			_last_clean = time.time()
			min_mtime = time.time() - MAX_MINUTES_OF_LIFE*60
			
			for f in glob.glob(join(common.CACHE_DIR, '*', '*')):
				if os.path.getmtime(f) < min_mtime:
					os.unlink(f)


def load(name):
	""" return a utf-8 encoded byte-string from the cache that has the specified name, or None if it doesnt exist """
	with _lock:
		try:
			with open(join(common.CACHE_DIR, 'identity', utf8(name)), 'rb') as f:
				return f.read()
		except:
			return None


def load_gzip(name):
	""" return a utf-8 encoded, gzipped, byte-string from the cache that has the specified name, or None if it doesnt exist """
	with _lock:
		try:
			with open(join(common.CACHE_DIR, 'gzip', utf8(name)), 'rb') as f:
				return f.read()
		except:
			return None


def store(name, page):
	""" store page in the cache. page must be a utf-8 encoded byte-string """
	with _lock:
		# should be atomic - so put it in a temp file then rename it
		
		fd, path = tempfile.mkstemp(prefix = '*', dir = common.CACHE_DIR)
		with os.fdopen(fd, 'wb') as f:
			f.write(page)
		os.rename(path, join(common.CACHE_DIR, 'identity', utf8(name)))
		
		fd, path = tempfile.mkstemp(prefix = '*', dir = common.CACHE_DIR)
		with os.fdopen(fd, 'wb') as f:
			gzf = gzip.GzipFile(mode = 'wb', compresslevel = 3, fileobj = f)
			gzf.write(page)
			gzf.close()
		os.rename(path, join(common.CACHE_DIR, 'gzip', utf8(name)))


def size():
	""" return the number of cached files, not counting extra for compressed versions of the same file """
	return len(os.listdir(join(common.CACHE_DIR, 'identity')))


def thread_modified(board_db, thread_id):
	""" clears the cache of any items related to the specified thread """
	with _lock:
		
		b = '#%s#' % board_db.dir.encode('utf-8')
		
		exacts = (    b,
					  b + str(thread_id),
					  b + 'list',
					  '#'	)
		board_page = b + 'page'
		
		for f in glob.glob(join(common.CACHE_DIR, '*', '*')):
			name = os.path.basename(f)
			if name in exacts or name.startswith(board_page):
				os.unlink(f)
