""" provides in memory session storage and functions for pickling session data to/from disk """

from __future__ import with_statement

import copy, random, time, threading, cPickle, re


MAX_LIFETIME = 60*30
ID_LENGTH = 16
ID_REGEX = re.compile(r'^[-_a-zA-Z0-9]{%d}$' % ID_LENGTH)


_lock = threading.Lock()
_data = {}		#id => (last-save-time, _data-dict)
_last_clean = 0		#time of last clean


def generate_id():
	""" return a randomly generated session id """
	return ''.join([random.choice('-_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789') for i in xrange(ID_LENGTH)])


def get_valid_id(input):
	""" if input is a valid session id (string matching ID_REGEX), return it. otherwise return generate_id() """
	return input if (isinstance(input, basestring) and ID_REGEX.match(input)) else generate_id()


def unpickle(path):
	""" load all pickled session data from the specified file, ignoring failure """
	try:
		global _data
		_data = cPickle.load(open(path, 'r'))
	except:
		pass


def pickle(path):
	""" store all pickled session data in the specified file """
	with _lock:
		cPickle.dump(_data, open(path, 'w'))


def clean():
	""" delete sessions from memory that are older than MAX_LIFETIME, but only do this if MAX_LIFETIME has passed since the last time it was done. """
	global _last_clean
	with _lock:
		if (time.time() - _last_clean) > MAX_LIFETIME:
			_last_clean = time.time()
			exp = []
			for k in _data:
				if (time.time() - _data[k][0]) > MAX_LIFETIME:
					exp.append(k)
			for k in exp:
				del _data[k]


def load(id):
	""" return a shallow copy of the session data for the specified id, returning {} if no data exists for that id """
	with _lock:
		try:
			s = _data[id]
			if (time.time() - s[0]) <= MAX_LIFETIME:
				return copy.copy(s[1])
			else:
				return {}
		except KeyError:
			return {}


def store(id, obj):
	""" set the session data associated with the specified id, setting the age of the data to 0 """
	with _lock:
		_data[id] = (time.time(), obj)


def size():
	return len(_data)
