"""
simple python-programmer-centric template system where templates are translated into python code.

these are the tag types.

output tags:
	{{ expr }} -> output += escape(expr) (expr must be a string, int, float or bool)
	{$ expr $} -> output += expr (expr must be a string)

expression tags:
	{E exp E} -> where E = "%" or ":". ":" at start causes an indent reversal, at end it causes an indent on next line. "%" doesnt cause any indent change
	{:} -> tags containing only x ":"s cause x indent reversals
	example: {% for x in y :}
	example: {% x += 2 %}
	example: {: elif t == z :}
	example: {: x = x*4 %}

include another template:
	{" path "}

localized text tag:
	{_text to translate_}

comment tags:
	{# comment #}

spaces between tag start and end bits and middle bits are optional. eg. {{expr}} and {{       expr }}
only comment tags can contain newlines

the global namespace is the data argument to the render method.
names starting with "__" are reserved, as is the name "_".
included templates get a shallow copy of the parents local variables as their own

example usage:

print templates.render('templates', 'test.html', {'msg': 'hello'})

"""

from __future__ import with_statement

import re, os.path, copy, threading

import util, i18n


_TAG_REGEX = (
	r'\{\s*(?P<esc_output>.+?)\s*\}',
	r'\$\s*(?P<output>.+?)\s*\$',
	r'(?P<rev_ind>:+)',
	r'(?P<exp_start>%|:)\s*(?P<exp>.+?)\s*(?P<exp_end>%|:)',
	r'_(?P<local_text>.*?)_',
	r'"\s*(?P<inc_path>.+?)\s*"'
)
_TAG_REGEX = re.compile(r'\{\s*(' + '|'.join(_TAG_REGEX) + r')\s*\}')
_COMMENT_REGEX = re.compile(r'\{#(?P<comment>.*?)#\}', re.DOTALL)
_WHITESPACE_REGEX = re.compile(r'(\s)\s+')
_WHITESPACE_ENDS_REGEX = re.compile(r'^\s+|\s+$')


class TemplateException(Exception):
	pass


_lock = threading.Lock()
_cache = {}	#path => (file_mod_time, Template)


def render(dir, path, data = {}, localize = lambda k: k):
	"""
		return a string rendered from the template loaded from the file at os.path.join(dir, path).
		use dir as the base file path for any included templates to be loaded from.
		raise TemplateException upon error.
	"""
	_purge_modified()
	out = []
	data['__dir'] = dir
	data['__out'] = out
	data['__app'] = out.append
	data['_'] = localize
	data['__html_escape'] = util.html_escape
	data['__render_to_list'] = _render_to_list
	data['__globals'] = data
	_render_to_list(dir, path, {}, data)
	return ''.join(out)


def flush_cache():
	with _lock:
		_cache.clear()


def extract_local_strings(dir):

	inner_exp = re.compile(r'_\(([\'"])(.+?)(\1)\)')

	strs = {}	# msgid => ['filename:linenum', ...]

	def line_num(m):
		return 1 + m.string[:m.start()].count('\n')

	def extract(f):
		for m in util.re_msplit(_TAG_REGEX, open(f, 'rb').read().decode('utf-8')):
			if not isinstance(m, basestring):
				d = m.groupdict()

				def found(s):
					strs.setdefault(s, [])
					strs[s].append('%s:%d' % (f, line_num(m)))

				if d['local_text']:
					found(d['local_text'])
				elif d['esc_output']:
					for t in inner_exp.findall(d['esc_output']):
						found(t[1])
				elif d['output']:
					for t in inner_exp.findall(d['output']):
						found(t[1])
	
	for dirpath, dirnames, filenames in os.walk(dir):
		for f in filenames:
			if f.endswith('.html'):
				extract(os.path.join(dirpath, f))

	s = ''
	for msgid, lines in sorted(strs.items()):
		s += ''.join('#: %s\n' % line for line in lines)
		s += 'msgid "%s"' % i18n.esc_msg(msgid) + '\n'
		s += 'msgstr ""\n\n'
	return s


def _render_to_list(dir, path, locals, globals):
	tpl = _load(dir, path)
	try:
		tpl.render_to_list(locals, globals)
	except Exception, e:
		raise TemplateException('exception in %s: %s %s' % (path, type(e).__name__, e))


def _purge_modified():
	""" remove all modified templates from the cache. they will be loaded upon next use """
	with _lock:
		for path, tpl in _cache.items():
			if os.path.getmtime(path) != tpl[0]:
				del _cache[path]


def _load(dir, path):
	with _lock:
		full_path = os.path.join(dir, path)
		if full_path not in _cache:
			try:
				mod = os.path.getmtime(full_path)
				tpl = _Template(dir, full_path, open(full_path, 'rb').read().decode('utf-8'))
				_cache[full_path] = (mod, tpl)
			except Exception, e:
				raise TemplateException('error loading template %s - %s' % (path, e))
		return _cache[full_path][1]


class _Template(object):

	def __init__(self, dir, path, string):
		self.dir = dir
		self.path = path
		
		string = _COMMENT_REGEX.sub('', string)
		string = _WHITESPACE_REGEX.sub(r'\1', string)
		string = _WHITESPACE_ENDS_REGEX.sub('', string)
		code = ''
		line_count = [0]	# indent => count
		
		for m in util.re_msplit(_TAG_REGEX, string):
			
			if isinstance(m, basestring):
				code += '\t'*(len(line_count)-1) + '__app(' + repr(m) + ')\n'
				line_count[-1] += 1
			
			else:
				d = m.groupdict()
				
				if d['inc_path']:
					code += '\t'*(len(line_count)-1) + '__render_to_list(__dir, "' + d['inc_path'] + '", locals(), __globals)\n'
					line_count[-1] += 1
				
				if d['exp']:
					s = d['exp_start']
					e = d['exp_end']
					if s == ':':
						if len(line_count) == 1:
							raise TemplateException('reverse-indented too far')
						if not line_count.pop():
							code += '\t'*len(line_count) + 'pass\n'
					code += '\t'*(len(line_count)-1) + d['exp']
					line_count[-1] += 1
					if e == ':':
						code += ':'
						line_count += [0]
					code += '\n'
			
				elif d['rev_ind']:
					for c in d['rev_ind']:
						if len(line_count) == 1:
							raise TemplateException('reverse-indented too far')
						if not line_count.pop():
							code += '\t'*len(line_count) + 'pass\n'
				
				elif d['local_text']:
					exp = d['local_text']
					code += '\t'*(len(line_count)-1) + '__app(_(u"""' + exp + '"""))\n'
					line_count[-1] += 1

				elif d['esc_output']:
					exp = d['esc_output']
					code += '\t'*(len(line_count)-1) + '__app(__html_escape(' + exp + '))\n'
					line_count[-1] += 1
				
				elif d['output']:
					exp = d['output']
					code += '\t'*(len(line_count)-1) + '__app(' + exp + ')\n'
					line_count[-1] += 1
		
		try:
			self.code = compile(code, path, 'exec')
		except Exception, e:
			raise TemplateException('invalid template syntax: %s' % e)

	def render_to_list(self, locals, globals):
		exec self.code in globals, copy.copy(locals)
